@@ -18,10 +18,13 @@ import os
 if sys.version_info < (3, 6):
     sys.exit("This script requires Python 3.6 or newer")
 
-from subprocess import run, PIPE
-from distutils.version import LooseVersion
+from sympy.external.importtools import version_tuple
+from subprocess import check_output, PIPE
 from collections import OrderedDict
 
+def run(cmd):
+    return check_output(cmd, encoding='utf-8')
+
 def red(text):
     return "\033[31m%s\033[0m" % text
 
@@ -43,8 +46,9 @@ from sympy.utilities.misc import filldedent
 
 # check git version
 minimal = '1.8.4.2'
-git_ver = run(['git', '--version'], stdout=PIPE, encoding='utf-8').stdout[12:]
-if LooseVersion(git_ver) < LooseVersion(minimal):
+git_ver = run(['git', '--version'])[12:]
+
+if version_tuple(git_ver) < version_tuple(minimal):
     print(yellow("Please use a git version >= %s" % minimal))
 
 def author_name(line):
@@ -61,7 +65,7 @@ def move(l, i1, i2, who):
 
 # find who git knows ahout
 git_command = ["git", "log", "--topo-order", "--reverse", "--format=%aN <%aE>"]
-git_people = run(git_command, stdout=PIPE, encoding='utf-8').stdout.strip().split("\n")
+git_people = run(git_command).strip().split("\n")
 
 # remove duplicates, keeping the original order
 git_people = list(OrderedDict.fromkeys(git_people))
@@ -163,4 +167,5 @@ else:
     print(yellow("The AUTHORS file was updated."))
 
 print(red("Changes were made in the authors file"))
+run(['git', 'diff']) # Show the changes
 sys.exit(1)
