@@ -16,9 +16,11 @@ def test_executable(path):
             with open(path, 'r') as f:
                 if f.readline()[:2] != "#!":
                     exn_msg = "File at " + path + " either should not be executable or should have a shebang line"
-                    raise SystemError(exn_msg)
+                    raise OSError(exn_msg)
     else:
         for file in os.listdir(path):
+            if file in ('.git', 'venv_main'):
+                continue
             test_executable(os.path.join(path, file))
 
 test_executable(base_dir)
