@@ -33,18 +33,6 @@ if [[ "${TEST_SPHINX}" == "true" ]]; then
     }
 fi
 
-if [[ "${TEST_SAGE}" == "true" ]]; then
-    echo "Testing SAGE"
-    source deactivate
-    source activate sage
-    sage -v
-    sage -python bin/test sympy/external/tests/test_sage.py
-    PYTHONPATH=. sage -t sympy/external/tests/test_sage.py
-    export MPMATH_NOSAGE=1
-    source deactivate
-    source activate test-environment
-fi
-
 if [[ -n "${TEST_OPT_DEPENDENCY}" ]]; then
     python bin/test_external_imports.py
     python bin/test_submodule_imports.py
@@ -157,8 +145,8 @@ test_list = [
     # llvmlite
     '*llvm*',
 
-    # theano
-    '*theano*',
+    # aesara
+    '*aesara*',
 
     # gmpy
     'polys',
@@ -207,8 +195,8 @@ doctest_list = [
     # llvmlite
     '*llvm*',
 
-    # theano
-    '*theano*',
+    # aesara
+    '*aesara*',
 
     # gmpy
     'polys',
