@@ -16,8 +16,6 @@ blacklist_path = os.path.join(os.path.dirname(__file__), '.ci', 'blacklisted.jso
 # Collecting tests from rubi_tests under pytest leads to errors even if the
 # tests will be skipped.
 collect_ignore = ["sympy/integrals/rubi"] + _get_doctest_blacklist()
-if sys.version_info < (3,):
-    collect_ignore.append('doc/src/gotchas.rst')
 
 # Set up printing for doctests
 setup_pprint()
