@@ -34,7 +34,10 @@ git_whitelist = {
     '.mailmap',
     # Travis and CI
     '.travis.yml',
+    '.github/workflows/ci-sage.yml',
+    '.github/workflows/comment-on-pr.yml',
     '.github/workflows/runtests.yml',
+    '.github/workflows/release.yml',
     '.ci/durations.json',
     '.ci/generate_durations_log.sh',
     '.ci/parse_durations_log.py',
@@ -45,10 +48,12 @@ git_whitelist = {
     '.coveragerc',
     'CODEOWNERS',
     'asv.conf.travis.json',
+    'asv.conf.actions.json',
     'coveragerc_travis',
     'codecov.yml',
     'pytest.ini',
     'MANIFEST.in',
+    'banner.svg',
     # Code of conduct
     'CODE_OF_CONDUCT.md',
     # Pull request template
@@ -86,6 +91,10 @@ git_whitelist = {
     'bin/test_setup.py',
     'bin/test_submodule_imports.py',
     'bin/test_travis.sh',
+    'bin/test_optional_dependencies.py',
+    'bin/test_sphinx.sh',
+    'bin/test_symengine.py',
+    'bin/test_tensorflow.py',
     # The notebooks are not ready for shipping yet. They need to be cleaned
     # up, and preferably doctested.  See also
     # https://github.com/sympy/sympy/issues/6039.
@@ -131,6 +140,7 @@ git_whitelist = {
     'release/test_install.py',
     'release/sha256.py',
     'release/authors.py',
+    'release/ci_release_script.sh',
     # This is just a distribute version of setup.py. Used mainly for setup.py
     # develop, which we don't care about in the release tarball
     'setupegg.py',
