@@ -1,7 +1,6 @@
-from __future__ import print_function, division
-
 from sympy.core import S, Basic, Dict, Symbol, Tuple, sympify
 from sympy.core.compatibility import iterable
+from sympy.core.symbol import Str
 from sympy.sets import Set, FiniteSet, EmptySet
 
 
@@ -9,6 +8,9 @@ class Class(Set):
     r"""
     The base class for any kind of class in the set-theoretic sense.
 
+    Explanation
+    ===========
+
     In axiomatic set theories, everything is a class.  A class which
     can be a member of another class is a set.  A class which is not a
     member of another class is a proper class.  The class `\{1, 2\}`
@@ -25,6 +27,9 @@ class Object(Symbol):
     """
     The base class for any kind of object in an abstract category.
 
+    Explanation
+    ===========
+
     While technically any instance of :class:`~.Basic` will do, this
     class is the recommended way to create abstract objects in
     abstract categories.
@@ -35,6 +40,9 @@ class Morphism(Basic):
     """
     The base class for any morphism in an abstract category.
 
+    Explanation
+    ===========
+
     In abstract categories, a morphism is an arrow between two
     category objects.  The object where the arrow starts is called the
     domain, while the object where the arrow ends is called the
@@ -139,6 +147,9 @@ class IdentityMorphism(Morphism):
     """
     Represents an identity morphism.
 
+    Explanation
+    ===========
+
     An identity morphism is a morphism with equal domain and codomain,
     which acts as an identity with respect to composition.
 
@@ -162,13 +173,20 @@ class IdentityMorphism(Morphism):
     Morphism
     """
     def __new__(cls, domain):
-        return Basic.__new__(cls, domain, domain)
+        return Basic.__new__(cls, domain)
+
+    @property
+    def codomain(self):
+        return self.domain
 
 
 class NamedMorphism(Morphism):
     """
     Represents a morphism which has a name.
 
+    Explanation
+    ===========
+
     Names are used to distinguish between morphisms which have the
     same domain and codomain: two named morphisms are equal if they
     have the same domains, codomains, and names.
@@ -194,7 +212,10 @@ class NamedMorphism(Morphism):
         if not name:
             raise ValueError("Empty morphism names not allowed.")
 
-        return Basic.__new__(cls, domain, codomain, Symbol(name))
+        if not isinstance(name, Str):
+            name = Str(name)
+
+        return Basic.__new__(cls, domain, codomain, name)
 
     @property
     def name(self):
@@ -219,6 +240,9 @@ class CompositeMorphism(Morphism):
     r"""
     Represents a morphism which is a composition of other morphisms.
 
+    Explanation
+    ===========
+
     Two composite morphisms are equal if the morphisms they were
     obtained from (components) are the same and were listed in the
     same order.
@@ -249,6 +273,9 @@ class CompositeMorphism(Morphism):
         """
         Intelligently adds ``morphism`` to tuple ``t``.
 
+        Explanation
+        ===========
+
         If ``morphism`` is a composite morphism, its components are
         added to the tuple.  If ``morphism`` is an identity, nothing
         is added to the tuple.
@@ -371,6 +398,9 @@ class CompositeMorphism(Morphism):
         """
         Forgets the composite structure of this morphism.
 
+        Explanation
+        ===========
+
         If ``new_name`` is not empty, returns a :class:`NamedMorphism`
         with the supplied name, otherwise returns a :class:`Morphism`.
         In both cases the domain of the new morphism is the domain of
@@ -397,6 +427,9 @@ class Category(Basic):
     r"""
     An (abstract) category.
 
+    Explanation
+    ===========
+
     A category [JoyOfCats] is a quadruple `\mbox{K} = (O, \hom, id,
     \circ)` consisting of
 
@@ -442,13 +475,20 @@ class Category(Basic):
 
     See Also
     ========
+
     Diagram
     """
     def __new__(cls, name, objects=EmptySet, commutative_diagrams=EmptySet):
         if not name:
             raise ValueError("A Category cannot have an empty name.")
 
-        new_category = Basic.__new__(cls, Symbol(name), Class(objects),
+        if not isinstance(name, Str):
+            name = Str(name)
+
+        if not isinstance(objects, Class):
+            objects = Class(objects)
+
+        new_category = Basic.__new__(cls, name, objects,
                                      FiniteSet(*commutative_diagrams))
         return new_category
 
@@ -482,7 +522,7 @@ class Category(Basic):
         >>> B = Object("B")
         >>> K = Category("K", FiniteSet(A, B))
         >>> K.objects
-        Class(FiniteSet(Object("A"), Object("B")))
+        Class({Object("A"), Object("B")})
 
         """
         return self.args[1]
@@ -493,6 +533,9 @@ class Category(Basic):
         Returns the :class:`~.FiniteSet` of diagrams which are known to
         be commutative in this category.
 
+        Examples
+        ========
+
         >>> from sympy.categories import Object, NamedMorphism, Diagram, Category
         >>> from sympy import FiniteSet
         >>> A = Object("A")
@@ -521,6 +564,9 @@ class Diagram(Basic):
     r"""
     Represents a diagram in a certain category.
 
+    Explanation
+    ===========
+
     Informally, a diagram is a collection of objects of a category and
     certain morphisms between them.  A diagram is still a monoid with
     respect to morphism composition; i.e., identity morphisms, as well
@@ -552,7 +598,7 @@ class Diagram(Basic):
     ========
 
     >>> from sympy.categories import Object, NamedMorphism, Diagram
-    >>> from sympy import FiniteSet, pprint, default_sort_key
+    >>> from sympy import pprint, default_sort_key
     >>> A = Object("A")
     >>> B = Object("B")
     >>> C = Object("C")
@@ -566,11 +612,12 @@ class Diagram(Basic):
     {g*f:A-->C: EmptySet, id:A-->A: EmptySet, id:B-->B: EmptySet, id:C-->C: EmptyS
     et, f:A-->B: EmptySet, g:B-->C: EmptySet}
     >>> d = Diagram([f, g], {g * f: "unique"})
-    >>> pprint(d.conclusions)
+    >>> pprint(d.conclusions,use_unicode=False)
     {g*f:A-->C: {unique}}
 
     References
     ==========
+
     [Pare1970] B. Pareigis: Categories and functors.  Academic Press,
     1970.
     """
@@ -644,6 +691,9 @@ class Diagram(Basic):
         """
         Construct a new instance of Diagram.
 
+        Explanation
+        ===========
+
         If no arguments are supplied, an empty diagram is created.
 
         If at least an argument is supplied, ``args[0]`` is
@@ -677,7 +727,7 @@ class Diagram(Basic):
         True
         >>> d = Diagram([f, g], {g * f: "unique"})
         >>> d.conclusions[g * f]
-        FiniteSet(unique)
+        {unique}
 
         """
         premises = {}
@@ -809,15 +859,15 @@ class Diagram(Basic):
         >>> g = NamedMorphism(B, C, "g")
         >>> d = Diagram([f, g])
         >>> d.objects
-        FiniteSet(Object("A"), Object("B"), Object("C"))
+        {Object("A"), Object("B"), Object("C")}
 
         """
         return self.args[2]
 
     def hom(self, A, B):
         """
-        Returns a 2-tuple of sets of morphisms between objects A and
-        B: one set of morphisms listed as premises, and the other set
+        Returns a 2-tuple of sets of morphisms between objects ``A`` and
+        ``B``: one set of morphisms listed as premises, and the other set
         of morphisms listed as conclusions.
 
         Examples
