@@ -82,12 +82,10 @@ References
 [Xypic] http://xy-pic.sourceforge.net/
 
 """
-from __future__ import print_function, division
-
 from sympy.categories import (CompositeMorphism, IdentityMorphism,
                               NamedMorphism, Diagram)
 from sympy.core import Dict, Symbol
-from sympy.core.compatibility import iterable, range
+from sympy.core.compatibility import iterable
 from sympy.printing import latex
 from sympy.sets import FiniteSet
 from sympy.utilities import default_sort_key
@@ -99,10 +97,13 @@ from itertools import chain
 __doctest_requires__ = {('preview_diagram',): 'pyglet'}
 
 
-class _GrowableGrid(object):
+class _GrowableGrid:
     """
     Holds a growable grid of objects.
 
+    Explanation
+    ===========
+
     It is possible to append or prepend a row or a column to the grid
     using the corresponding methods.  Prepending rows or columns has
     the effect of changing the coordinates of the already existing
@@ -172,10 +173,13 @@ class _GrowableGrid(object):
             self._array[i].insert(0, None)
 
 
-class DiagramGrid(object):
+class DiagramGrid:
     r"""
     Constructs and holds the fitting of the diagram into a grid.
 
+    Explanation
+    ===========
+
     The mission of this class is to analyse the structure of the
     supplied diagram and to place its objects on a grid such that,
     when the objects and the morphisms are actually drawn, the diagram
@@ -298,7 +302,7 @@ class DiagramGrid(object):
     References
     ==========
 
-    [FiveLemma] https://en.wikipedia.org/wiki/Five_lemma
+    .. [FiveLemma] https://en.wikipedia.org/wiki/Five_lemma
     """
     @staticmethod
     def _simplify_morphisms(morphisms):
@@ -560,10 +564,10 @@ class DiagramGrid(object):
     @staticmethod
     def _find_triangle_to_weld(triangles, fringe, grid):
         """
-        Finds, if possible, a triangle and an edge in the fringe to
+        Finds, if possible, a triangle and an edge in the ``fringe`` to
         which the triangle could be attached.  Returns the tuple
         containing the triangle and the index of the corresponding
-        edge in the fringe.
+        edge in the ``fringe``.
 
         This function relies on the fact that objects are unique in
         the diagram.
@@ -706,7 +710,7 @@ class DiagramGrid(object):
     @staticmethod
     def _grow_pseudopod(triangles, fringe, grid, skeleton, placed_objects):
         """
-        Starting from an object in the existing structure on the grid,
+        Starting from an object in the existing structure on the ``grid``,
         adds an edge to which a triangle from ``triangles`` could be
         welded.  If this method has found a way to do so, it returns
         the object it has just added.
@@ -1357,7 +1361,7 @@ class DiagramGrid(object):
         return repr(self._grid._array)
 
 
-class ArrowStringDescription(object):
+class ArrowStringDescription:
     r"""
     Stores the information necessary for producing an Xy-pic
     description of an arrow.
@@ -1500,7 +1504,7 @@ class ArrowStringDescription(object):
                 self.label_displacement, self.label)
 
 
-class XypicDiagramDrawer(object):
+class XypicDiagramDrawer:
     r"""
     Given a :class:`~.Diagram` and the corresponding
     :class:`DiagramGrid`, produces the Xy-pic representation of the
@@ -2026,7 +2030,7 @@ class XypicDiagramDrawer(object):
             two supplied objects.
             """
             return len([m for m in morphisms_str_info
-                        if set([m.domain, m.codomain]) == set([A, B])])
+                        if {m.domain, m.codomain} == {A, B}])
 
         def count_morphisms_filtered(dom, cod, curving):
             """
