@@ -59,6 +59,7 @@ AST Type Tree
 
 Predefined types
 ----------------
+
 A number of ``Type`` instances are provided in the ``sympy.codegen.ast`` module
 for convenience. Perhaps the two most common ones for code-generation (of numeric
 codes) are ``float32`` and ``float64`` (known as single and double precision respectively).
@@ -77,6 +78,7 @@ The other ``Type`` instances defined are:
 
 Using the nodes
 ---------------
+
 It is possible to construct simple algorithms using the AST nodes. Let's construct a loop applying
 Newton's method::
 
@@ -109,7 +111,7 @@ Newton's method::
 
 If we want to generate Fortran code for the same while loop we simple call ``fcode``::
 
-    >>> from sympy.printing.fcode import fcode
+    >>> from sympy.printing import fcode
     >>> print(fcode(whl, standard=2003, source_format='free'))
     do while (abs(delta) > tol)
        delta = (val**3 - cos(val))/(-3*val**2 - sin(val))
@@ -122,21 +124,19 @@ There is a function constructing a loop (or a complete function) like this in
 
 """
 
-from __future__ import print_function, division
+from typing import Any, Dict, List
 
-from itertools import chain
 from collections import defaultdict
+
+from sympy import Lt, Le, Ge, Gt
 from sympy.core import Symbol, Tuple, Dummy
 from sympy.core.basic import Basic
-from sympy.core.compatibility import string_types
 from sympy.core.expr import Expr
 from sympy.core.numbers import Float, Integer, oo
-from sympy.core.relational import Lt, Le, Ge, Gt
 from sympy.core.sympify import _sympify, sympify, SympifyError
 from sympy.utilities.iterables import iterable
 
 
-
 def _mk_Tuple(args):
     """
     Create a Sympy Tuple object from an iterable, converting Python strings to
@@ -153,13 +153,16 @@ def _mk_Tuple(args):
 
     sympy.Tuple
     """
-    args = [String(arg) if isinstance(arg, string_types) else arg for arg in args]
+    args = [String(arg) if isinstance(arg, str) else arg for arg in args]
     return Tuple(*args)
 
 
 class Token(Basic):
     """ Base class for the AST types.
 
+    Explanation
+    ===========
+
     Defining fields are set in ``__slots__``. Attributes (defined in __slots__)
     are only allowed to contain instances of Basic (unless atomic, see
     ``String``). The arguments to ``__new__()`` correspond to the attributes in
@@ -172,9 +175,9 @@ class Token(Basic):
     in the class attribute ``not_in_args`` are not passed to :class:`~.Basic`.
     """
 
-    __slots__ = []
-    defaults = {}
-    not_in_args = []
+    __slots__ = ()
+    defaults = {}  # type: Dict[str, Any]
+    not_in_args = []  # type: List[str]
     indented_args = ['body']
 
     @property
@@ -256,7 +259,7 @@ class Token(Basic):
         return tuple([getattr(self, attr) for attr in self.__slots__])
 
     def __hash__(self):
-        return super(Token, self).__hash__()
+        return super().__hash__()
 
     def _joiner(self, k, indent_level):
         return (',\n' + ' '*indent_level) if k in self.indented_args else ', '
@@ -278,12 +281,11 @@ class Token(Basic):
         else:
             return _print(v)
 
-    def _sympyrepr(self, printer, *args, **kwargs):
+    def _sympyrepr(self, printer, *args, joiner=', ', **kwargs):
         from sympy.printing.printer import printer_context
         exclude = kwargs.get('exclude', ())
         values = [getattr(self, k) for k in self.__slots__]
         indent_level = printer._context.get('indent_level', 0)
-        joiner = kwargs.pop('joiner', ', ')
 
         arg_reprs = []
 
@@ -300,7 +302,7 @@ class Token(Basic):
                 indented = self._indented(printer, attr, value, *args, **kwargs)
             arg_reprs.append(('{1}' if i == 0 else '{0}={1}').format(attr, indented.lstrip()))
 
-        return "{0}({1})".format(self.__class__.__name__, joiner.join(arg_reprs))
+        return "{}({})".format(self.__class__.__name__, joiner.join(arg_reprs))
 
     _sympystr = _sympyrepr
 
@@ -385,7 +387,7 @@ class NoneToken(Token):
         return ()
 
     def __hash__(self):
-        return super(NoneToken, self).__hash__()
+        return super().__hash__()
 
 
 none = NoneToken()
@@ -407,7 +409,7 @@ class AssignmentBase(Basic):
 
         cls._check_args(lhs, rhs)
 
-        return super(AssignmentBase, cls).__new__(cls, lhs, rhs)
+        return super().__new__(cls, lhs, rhs)
 
     @property
     def lhs(self):
@@ -499,6 +501,7 @@ class AugmentedAssignment(AssignmentBase):
        Symbol for binary operation being applied in the assignment, such as "+",
        "*", etc.
     """
+    binop = None  # type: str
 
     @property
     def op(self):
@@ -538,6 +541,9 @@ def aug_assign(lhs, op, rhs):
     """
     Create 'lhs op= rhs'.
 
+    Explanation
+    ===========
+
     Represents augmented variable assignment for code generation. This is a
     convenience function. You can also use the AugmentedAssignment classes
     directly, like AddAugmentedAssignment(x, y).
@@ -576,7 +582,10 @@ def aug_assign(lhs, op, rhs):
 
 class CodeBlock(Basic):
     """
-    Represents a block of code
+    Represents a block of code.
+
+    Explanation
+    ===========
 
     For now only assignments are supported. This restriction will be lifted in
     the future.
@@ -634,14 +643,14 @@ class CodeBlock(Basic):
         il = printer._context.get('indent_level', 0)
         joiner = ',\n' + ' '*il
         joined = joiner.join(map(printer._print, self.args))
-        return ('{0}(\n'.format(' '*(il-4) + self.__class__.__name__,) +
+        return ('{}(\n'.format(' '*(il-4) + self.__class__.__name__,) +
                 ' '*il + joined + '\n' + ' '*(il - 4) + ')')
 
     _sympystr = _sympyrepr
 
     @property
     def free_symbols(self):
-        return super(CodeBlock, self).free_symbols - set(self.left_hand_sides)
+        return super().free_symbols - set(self.left_hand_sides)
 
     @classmethod
     def topological_sort(cls, assignments):
@@ -649,6 +658,9 @@ class CodeBlock(Basic):
         Return a CodeBlock with topologically sorted assignments so that
         variables are assigned before they are used.
 
+        Examples
+        ========
+
         The existing order of assignments is preserved as much as possible.
 
         This function assumes that variables are assigned to only once.
@@ -728,7 +740,10 @@ class CodeBlock(Basic):
     def cse(self, symbols=None, optimizations=None, postprocess=None,
         order='canonical'):
         """
-        Return a new code block with common subexpressions eliminated
+        Return a new code block with common subexpressions eliminated.
+
+        Explanation
+        ===========
 
         See the docstring of :func:`sympy.simplify.cse_main.cse` for more
         information.
@@ -830,7 +845,7 @@ class For(Token):
         ))
     ))
     """
-    __slots__ = ['target', 'iterable', 'body']
+    __slots__ = ('target', 'iterable', 'body')
     _construct_target = staticmethod(_sympify)
 
     @classmethod
@@ -874,13 +889,13 @@ class String(Token):
     String('foo')
 
     """
-    __slots__ = ['text']
+    __slots__ = ('text',)
     not_in_args = ['text']
     is_Atom = True
 
     @classmethod
     def _construct_text(cls, text):
-        if not isinstance(text, string_types):
+        if not isinstance(text, str):
             raise TypeError("Argument text is not a string type.")
         return text
 
@@ -915,9 +930,9 @@ class Node(Token):
 
     """
 
-    __slots__ = ['attrs']
+    __slots__ = ('attrs',)
 
-    defaults = {'attrs': Tuple()}
+    defaults = {'attrs': Tuple()}  # type: Dict[str, Any]
 
     _construct_attrs = staticmethod(_mk_Tuple)
 
@@ -931,6 +946,9 @@ class Node(Token):
 class Type(Token):
     """ Represents a type.
 
+    Explanation
+    ===========
+
     The naming is a super-set of NumPy naming. Type has a classmethod
     ``from_expr`` which offer type deduction. It also has a method
     ``cast_check`` which casts the argument to its type, possibly raising an
@@ -969,8 +987,7 @@ class Type(Token):
       ...
     ValueError: Casting gives a significantly different value.
     >>> boost_mp50 = Type('boost::multiprecision::cpp_dec_float_50')
-    >>> from sympy import Symbol
-    >>> from sympy.printing.cxxcode import cxxcode
+    >>> from sympy.printing import cxxcode
     >>> from sympy.codegen.ast import Declaration, Variable
     >>> cxxcode(Declaration(Variable('x', type=boost_mp50)))
     'boost::multiprecision::cpp_dec_float_50 x'
@@ -981,7 +998,7 @@ class Type(Token):
     .. [1] https://docs.scipy.org/doc/numpy/user/basics.types.html
 
     """
-    __slots__ = ['name']
+    __slots__ = ('name',)
 
     _construct_name = String
 
@@ -1034,7 +1051,7 @@ class Type(Token):
     def _check(self, value):
         pass
 
-    def cast_check(self, value, rtol=None, atol=0, limits=None, precision_targets=None):
+    def cast_check(self, value, rtol=None, atol=0, precision_targets=None):
         """ Casts a value to the data type of the instance.
 
         Parameters
@@ -1045,15 +1062,13 @@ class Type(Token):
             Relative tolerance. (will be deduced if not given).
         atol : floating point number
             Absolute tolerance (in addition to ``rtol``).
-        limits : dict
-            Values given by ``limits.h``, x86/IEEE754 defaults if not given.
         type_aliases : dict
             Maps substitutions for Type, e.g. {integer: int64, real: float32}
 
         Examples
         ========
 
-        >>> from sympy.codegen.ast import Type, integer, float32, int8
+        >>> from sympy.codegen.ast import integer, float32, int8
         >>> integer.cast_check(3.0) == 3
         True
         >>> float32.cast_check(1e-40)  # doctest: +ELLIPSIS
@@ -1106,12 +1121,12 @@ class Type(Token):
 
 class IntBaseType(Type):
     """ Integer base type, contains no size information. """
-    __slots__ = ['name']
+    __slots__ = ('name',)
     cast_nocheck = lambda self, i: Integer(int(i))
 
 
 class _SizedIntType(IntBaseType):
-    __slots__ = ['name', 'nbits']
+    __slots__ = ('name', 'nbits',)
 
     _construct_nbits = Integer
 
@@ -1169,7 +1184,7 @@ class FloatType(FloatBaseType):
     Examples
     ========
 
-    >>> from sympy import S, Float
+    >>> from sympy import S
     >>> from sympy.codegen.ast import FloatType
     >>> half_precision = FloatType('f16', nbits=16, nmant=10, nexp=5)
     >>> half_precision.max
@@ -1190,7 +1205,7 @@ class FloatType(FloatBaseType):
     ValueError: Maximum value for data type smaller than new value.
     """
 
-    __slots__ = ['name', 'nbits', 'nmant', 'nexp']
+    __slots__ = ('name', 'nbits', 'nmant', 'nexp',)
 
     _construct_nbits = _construct_nmant = _construct_nexp = Integer
 
@@ -1240,6 +1255,9 @@ class FloatType(FloatBaseType):
     def decimal_dig(self):
         """ Number of digits needed to store & load without loss.
 
+        Explanation
+        ===========
+
         Number of decimal digits needed to guarantee that two consecutive conversions
         (float -> text -> float) to be idempotent. This is useful when one do not want
         to loose precision due to rounding errors when storing a floating point value
@@ -1270,14 +1288,14 @@ class ComplexBaseType(FloatBaseType):
         """ Casts without checking if out of bounds or subnormal. """
         from sympy.functions import re, im
         return (
-            super(ComplexBaseType, self).cast_nocheck(re(value)) +
-            super(ComplexBaseType, self).cast_nocheck(im(value))*1j
+            super().cast_nocheck(re(value)) +
+            super().cast_nocheck(im(value))*1j
         )
 
     def _check(self, value):
         from sympy.functions import re, im
-        super(ComplexBaseType, self)._check(re(value))
-        super(ComplexBaseType, self)._check(im(value))
+        super()._check(re(value))
+        super()._check(im(value))
 
 
 class ComplexType(ComplexBaseType, FloatType):
@@ -1321,6 +1339,7 @@ class Attribute(Token):
 
     Parameters
     ==========
+
     name : str
     parameters : Tuple
 
@@ -1339,8 +1358,9 @@ class Attribute(Token):
     >>> a.parameters == (1, 2, 3)
     True
     """
-    __slots__ = ['name', 'parameters']
+    __slots__ = ('name', 'parameters')
     defaults = {'parameters': Tuple()}
+
     _construct_name = String
     _construct_parameters = staticmethod(_mk_Tuple)
 
@@ -1356,7 +1376,7 @@ pointer_const = Attribute('pointer_const')
 
 
 class Variable(Node):
-    """ Represents a variable
+    """ Represents a variable.
 
     Parameters
     ==========
@@ -1405,11 +1425,10 @@ class Variable(Node):
 
     """
 
-    __slots__ = ['symbol', 'type', 'value'] + Node.__slots__
-    defaults = dict(chain(Node.defaults.items(), {
-        'type': untyped,
-        'value': none
-    }.items()))
+    __slots__ = ('symbol', 'type', 'value') + Node.__slots__
+
+    defaults = Node.defaults.copy()
+    defaults.update({'type': untyped, 'value': none})
 
     _construct_symbol = staticmethod(sympify)
     _construct_value = staticmethod(sympify)
@@ -1462,6 +1481,9 @@ class Variable(Node):
     def as_Declaration(self, **kwargs):
         """ Convenience method for creating a Declaration instance.
 
+        Explanation
+        ===========
+
         If the variable of the Declaration need to wrap a modified
         variable keyword arguments may be passed (overriding e.g.
         the ``value`` of the Variable instance).
@@ -1500,9 +1522,6 @@ class Variable(Node):
     __ge__ = lambda self, other: self._relation(other, Ge)
     __gt__ = lambda self, other: self._relation(other, Gt)
 
-
-
-
 class Pointer(Variable):
     """ Represents a pointer. See ``Variable``.
 
@@ -1546,7 +1565,7 @@ class Element(Token):
     'x[i*l + j*m + k*n + o]'
 
     """
-    __slots__ = ['symbol', 'indices', 'strides', 'offset']
+    __slots__ = ('symbol', 'indices', 'strides', 'offset')
     defaults = {'strides': none, 'offset': none}
     _construct_symbol = staticmethod(sympify)
     _construct_indices = staticmethod(lambda arg: Tuple(*arg))
@@ -1565,8 +1584,7 @@ class Declaration(Token):
     Examples
     ========
 
-    >>> from sympy import Symbol
-    >>> from sympy.codegen.ast import Declaration, Type, Variable, NoneToken, integer, untyped
+    >>> from sympy.codegen.ast import Declaration, NoneToken, untyped
     >>> z = Declaration('z')
     >>> z.variable.type == untyped
     True
@@ -1578,7 +1596,7 @@ class Declaration(Token):
     >>> z.variable.value == NoneToken()  # OK
     True
     """
-    __slots__ = ['variable']
+    __slots__ = ('variable',)
     _construct_variable = Variable
 
 
@@ -1609,7 +1627,7 @@ class While(Token):
     ... ])
 
     """
-    __slots__ = ['condition', 'body']
+    __slots__ = ('condition', 'body')
     _construct_condition = staticmethod(lambda cond: _sympify(cond))
 
     @classmethod
@@ -1630,7 +1648,7 @@ class Scope(Token):
         When passed an iterable it is used to instantiate a CodeBlock.
 
     """
-    __slots__ = ['body']
+    __slots__ = ('body',)
 
     @classmethod
     def _construct_body(cls, itr):
@@ -1663,7 +1681,7 @@ class Stream(Token):
     print("x", file=sys.stderr)
 
     """
-    __slots__ = ['name']
+    __slots__ = ('name',)
     _construct_name = String
 
 stdout = Stream('stdout')
@@ -1689,7 +1707,7 @@ class Print(Token):
 
     """
 
-    __slots__ = ['print_args', 'format_string', 'file']
+    __slots__ = ('print_args', 'format_string', 'file')
     defaults = {'format_string': none, 'file': none}
 
     _construct_print_args = staticmethod(_mk_Tuple)
@@ -1715,7 +1733,7 @@ class FunctionPrototype(Node):
 
     >>> from sympy import symbols
     >>> from sympy.codegen.ast import real, FunctionPrototype
-    >>> from sympy.printing.ccode import ccode
+    >>> from sympy.printing import ccode
     >>> x, y = symbols('x y', real=True)
     >>> fp = FunctionPrototype(real, 'foo', [x, y])
     >>> ccode(fp)
@@ -1723,7 +1741,7 @@ class FunctionPrototype(Node):
 
     """
 
-    __slots__ = ['return_type', 'name', 'parameters', 'attrs']
+    __slots__ = ('return_type', 'name', 'parameters', 'attrs')
 
     _construct_return_type = Type
     _construct_name = String
@@ -1763,7 +1781,7 @@ class FunctionDefinition(FunctionPrototype):
 
     >>> from sympy import symbols
     >>> from sympy.codegen.ast import real, FunctionPrototype
-    >>> from sympy.printing.ccode import ccode
+    >>> from sympy.printing import ccode
     >>> x, y = symbols('x y', real=True)
     >>> fp = FunctionPrototype(real, 'foo', [x, y])
     >>> ccode(fp)
@@ -1777,7 +1795,7 @@ class FunctionDefinition(FunctionPrototype):
     }
     """
 
-    __slots__ = FunctionPrototype.__slots__[:-1] + ['body', 'attrs']
+    __slots__ = FunctionPrototype.__slots__[:-1] + ('body', 'attrs')
 
     @classmethod
     def _construct_body(cls, itr):
@@ -1816,7 +1834,7 @@ class FunctionCall(Token, Expr):
     foo(bar, baz)
 
     """
-    __slots__ = ['name', 'function_args']
+    __slots__ = ('name', 'function_args')
 
     _construct_name = String
     _construct_function_args = staticmethod(lambda args: Tuple(*args))
