@@ -4,7 +4,7 @@ This module implements sums and products containing the Kronecker Delta function
 References
 ==========
 
-- http://mathworld.wolfram.com/KroneckerDelta.html
+.. [1] http://mathworld.wolfram.com/KroneckerDelta.html
 
 """
 from sympy.core import Add, Mul, S, Dummy
@@ -39,6 +39,9 @@ def _extract_delta(expr, index):
     """
     Extract a simple KroneckerDelta from the expression.
 
+    Explanation
+    ===========
+
     Returns the tuple ``(delta, newexpr)`` where:
 
       - ``delta`` is a simple KroneckerDelta expression if one was found,
@@ -228,6 +231,9 @@ def deltasummation(f, limit, no_piecewise=False):
     """
     Handle summations containing a KroneckerDelta.
 
+    Explanation
+    ===========
+
     The idea for summation is the following:
 
     - If we are dealing with a KroneckerDelta expression, i.e. KroneckerDelta(g(x), j),
