@@ -1,23 +1,21 @@
-from __future__ import print_function, division
-
 from sympy.core.add import Add
+from sympy.core.compatibility import is_sequence
+from sympy.core.containers import Tuple
 from sympy.core.expr import Expr
 from sympy.core.mul import Mul
-from sympy.core.relational import Equality
-from sympy.sets.sets import Interval
+from sympy.core.relational import Equality, Relational
 from sympy.core.singleton import S
 from sympy.core.symbol import Symbol, Dummy
 from sympy.core.sympify import sympify
-from sympy.core.compatibility import is_sequence, range
-from sympy.core.containers import Tuple
-from sympy.core.relational import Relational
-from sympy.logic.boolalg import BooleanFunction
 from sympy.functions.elementary.piecewise import (piecewise_fold,
     Piecewise)
+from sympy.logic.boolalg import BooleanFunction
+from sympy.tensor.indexed import Idx
+from sympy.sets.sets import Interval
+from sympy.sets.fancysets import Range
 from sympy.utilities import flatten
 from sympy.utilities.iterables import sift
-from sympy.matrices import Matrix
-from sympy.tensor.indexed import Idx
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 
 def _common_new(cls, function, *symbols, **assumptions):
@@ -26,7 +24,18 @@ def _common_new(cls, function, *symbols, **assumptions):
     both ExprWithLimits and AddWithLimits."""
     function = sympify(function)
 
-    if hasattr(function, 'func') and isinstance(function, Equality):
+    if isinstance(function, Equality):
+        # This transforms e.g. Integral(Eq(x, y)) to Eq(Integral(x), Integral(y))
+        # but that is only valid for definite integrals.
+        limits, orientation = _process_limits(*symbols)
+        if not (limits and all(len(limit) == 3 for limit in limits)):
+            SymPyDeprecationWarning(
+                feature='Integral(Eq(x, y))',
+                useinstead='Eq(Integral(x, z), Integral(y, z))',
+                issue=18053,
+                deprecated_since_version=1.6,
+            ).warn()
+
         lhs = function.lhs
         rhs = function.rhs
         return Equality(cls(lhs, *symbols, **assumptions), \
@@ -37,6 +46,10 @@ def _common_new(cls, function, *symbols, **assumptions):
 
     if symbols:
         limits, orientation = _process_limits(*symbols)
+        for i, li in enumerate(limits):
+            if len(li) == 4:
+                function = function.subs(li[0], li[-1])
+                limits[i] = Tuple(*li[:-1])
     else:
         # symbol not provided -- we can still try to compute a general form
         free = function.free_symbols
@@ -54,7 +67,7 @@ def _common_new(cls, function, *symbols, **assumptions):
     # top level. We only fold Piecewise that contain the integration
     # variable.
     reps = {}
-    symbols_of_integration = set([i[0] for i in limits])
+    symbols_of_integration = {i[0] for i in limits}
     for p in function.atoms(Piecewise):
         if not p.has(*symbols_of_integration):
             reps[p] = Dummy()
@@ -91,35 +104,54 @@ def _process_limits(*symbols):
                 limits.append(Tuple(V))
             continue
         elif is_sequence(V, Tuple):
-            V = sympify(flatten(V))
+            if len(V) == 2 and isinstance(V[1], Range):
+                lo = V[1].inf
+                hi = V[1].sup
+                dx = abs(V[1].step)
+                V = [V[0]] + [0, (hi - lo)//dx, dx*V[0] + lo]
+            V = sympify(flatten(V))  # a list of sympified elements
             if isinstance(V[0], (Symbol, Idx)) or getattr(V[0], '_diff_wrt', False):
                 newsymbol = V[0]
-                if len(V) == 2 and isinstance(V[1], Interval):
+                if len(V) == 2 and isinstance(V[1], Interval):  # 2 -> 3
+                    # Interval
                     V[1:] = [V[1].start, V[1].end]
-
-                if len(V) == 3:
-                    if V[1] is None and V[2] is not None:
-                        nlim = [V[2]]
-                    elif V[1] is not None and V[2] is None:
+                elif len(V) == 3:
+                    # general case
+                    if V[2] is None and not V[1] is None:
                         orientation *= -1
-                        nlim = [V[1]]
-                    elif V[1] is None and V[2] is None:
-                        nlim = []
-                    else:
-                        nlim = V[1:]
-                    limits.append(Tuple(newsymbol, *nlim))
-                    if isinstance(V[0], Idx):
-                        if V[0].lower is not None and not bool(nlim[0] >= V[0].lower):
-                            raise ValueError("Summation exceeds Idx lower range.")
-                        if V[0].upper is not None and not bool(nlim[1] <= V[0].upper):
-                            raise ValueError("Summation exceeds Idx upper range.")
-                    continue
-                elif len(V) == 1 or (len(V) == 2 and V[1] is None):
-                    limits.append(Tuple(newsymbol))
-                    continue
-                elif len(V) == 2:
-                    limits.append(Tuple(newsymbol, V[1]))
-                    continue
+                    V = [newsymbol] + [i for i in V[1:] if i is not None]
+
+                if not isinstance(newsymbol, Idx) or len(V) == 3:
+                    if len(V) == 4:
+                        limits.append(Tuple(*V))
+                        continue
+                    if len(V) == 3:
+                        if isinstance(newsymbol, Idx):
+                            # Idx represents an integer which may have
+                            # specified values it can take on; if it is
+                            # given such a value, an error is raised here
+                            # if the summation would try to give it a larger
+                            # or smaller value than permitted. None and Symbolic
+                            # values will not raise an error.
+                            lo, hi = newsymbol.lower, newsymbol.upper
+                            try:
+                                if lo is not None and not bool(V[1] >= lo):
+                                    raise ValueError("Summation will set Idx value too low.")
+                            except TypeError:
+                                pass
+                            try:
+                                if hi is not None and not bool(V[2] <= hi):
+                                    raise ValueError("Summation will set Idx value too high.")
+                            except TypeError:
+                                pass
+                        limits.append(Tuple(*V))
+                        continue
+                    if len(V) == 1 or (len(V) == 2 and V[1] is None):
+                        limits.append(Tuple(newsymbol))
+                        continue
+                    elif len(V) == 2:
+                        limits.append(Tuple(newsymbol, V[1]))
+                        continue
 
         raise ValueError('Invalid limits given: %s' % str(symbols))
 
@@ -127,7 +159,7 @@ def _process_limits(*symbols):
 
 
 class ExprWithLimits(Expr):
-    __slots__ = ['is_commutative']
+    __slots__ = ('is_commutative',)
 
     def __new__(cls, function, *symbols, **assumptions):
         pre = _common_new(cls, function, *symbols, **assumptions)
@@ -168,6 +200,10 @@ class ExprWithLimits(Expr):
         """
         return self._args[0]
 
+    @property
+    def kind(self):
+        return self.function.kind
+
     @property
     def limits(self):
         """Return the limits of expression.
@@ -189,7 +225,7 @@ class ExprWithLimits(Expr):
 
     @property
     def variables(self):
-        """Return a list of the dummy variables
+        """Return a list of the limit variables.
 
         >>> from sympy import Sum
         >>> from sympy.abc import x, i
@@ -201,10 +237,31 @@ class ExprWithLimits(Expr):
 
         function, limits, free_symbols
         as_dummy : Rename dummy variables
-        transform : Perform mapping on the dummy variable
+        sympy.integrals.integrals.Integral.transform : Perform mapping on the dummy variable
         """
         return [l[0] for l in self.limits]
 
+    @property
+    def bound_symbols(self):
+        """Return only variables that are dummy variables.
+
+        Examples
+        ========
+
+        >>> from sympy import Integral
+        >>> from sympy.abc import x, i, j, k
+        >>> Integral(x**i, (i, 1, 3), (j, 2), k).bound_symbols
+        [i, j]
+
+        See Also
+        ========
+
+        function, limits, free_symbols
+        as_dummy : Rename dummy variables
+        sympy.integrals.integrals.Integral.transform : Perform mapping on the dummy variable
+        """
+        return [l[0] for l in self.limits if len(l) != 1]
+
     @property
     def free_symbols(self):
         """
@@ -241,55 +298,6 @@ class ExprWithLimits(Expr):
         """Return True if the Sum has no free symbols, else False."""
         return not self.free_symbols
 
-    def as_dummy(self):
-        """
-        Replace instances of the given dummy variables with explicit dummy
-        counterparts to make clear what are dummy variables and what
-        are real-world symbols in an object.
-
-        Examples
-        ========
-
-        >>> from sympy import Integral
-        >>> from sympy.abc import x, y
-        >>> Integral(x, (x, x, y), (y, x, y)).as_dummy()
-        Integral(_x, (_x, x, _y), (_y, x, y))
-
-        If the object supperts the "integral at" limit ``(x,)`` it
-        is not treated as a dummy, but the explicit form, ``(x, x)``
-        of length 2 does treat the variable as a dummy.
-
-        >>> Integral(x, x).as_dummy()
-        Integral(x, x)
-        >>> Integral(x, (x, x)).as_dummy()
-        Integral(_x, (_x, x))
-
-        If there were no dummies in the original expression, then the
-        the symbols which cannot be changed by subs() are clearly seen as
-        those with an underscore prefix.
-
-        See Also
-        ========
-
-        variables : Lists the integration variables
-        transform : Perform mapping on the integration variable
-        """
-        reps = {}
-        f = self.function
-        limits = list(self.limits)
-        for i in range(-1, -len(limits) - 1, -1):
-            xab = list(limits[i])
-            if len(xab) == 1:
-                continue
-            x = xab[0]
-            xab[0] = x.as_dummy()
-            for j in range(1, len(xab)):
-                xab[j] = xab[j].subs(reps)
-            reps[x] = xab[0]
-            limits[i] = xab
-        f = f.subs(reps)
-        return self.func(f, *limits)
-
     def _eval_interval(self, x, a, b):
         limits = [(i if i[0] != x else (x, a, b)) for i in self.limits]
         integrand = self.function
@@ -341,7 +349,10 @@ class ExprWithLimits(Expr):
             sub_into_func = True
             for i, xab in enumerate(limits):
                 if 1 == len(xab) and old == xab[0]:
-                    xab = (old, old)
+                    if new._diff_wrt:
+                        xab = (new,)
+                    else:
+                        xab = (old, old)
                 limits[i] = Tuple(xab[0], *[l._subs(old, new) for l in xab[1:]])
                 if len(xab[0].free_symbols.intersection(old.free_symbols)) != 0:
                     sub_into_func = False
@@ -372,6 +383,111 @@ class ExprWithLimits(Expr):
 
         return self.func(func, *limits)
 
+    @property
+    def has_finite_limits(self):
+        """
+        Returns True if the limits are known to be finite, either by the
+        explicit bounds, assumptions on the bounds, or assumptions on the
+        variables.  False if known to be infinite, based on the bounds.
+        None if not enough information is available to determine.
+
+        Examples
+        ========
+
+        >>> from sympy import Sum, Integral, Product, oo, Symbol
+        >>> x = Symbol('x')
+        >>> Sum(x, (x, 1, 8)).has_finite_limits
+        True
+
+        >>> Integral(x, (x, 1, oo)).has_finite_limits
+        False
+
+        >>> M = Symbol('M')
+        >>> Sum(x, (x, 1, M)).has_finite_limits
+
+        >>> N = Symbol('N', integer=True)
+        >>> Product(x, (x, 1, N)).has_finite_limits
+        True
+
+        See Also
+        ========
+
+        has_reversed_limits
+
+        """
+
+        ret_None = False
+        for lim in self.limits:
+            if len(lim) == 3:
+                if any(l.is_infinite for l in lim[1:]):
+                    # Any of the bounds are +/-oo
+                    return False
+                elif any(l.is_infinite is None for l in lim[1:]):
+                    # Maybe there are assumptions on the variable?
+                    if lim[0].is_infinite is None:
+                        ret_None = True
+            else:
+                if lim[0].is_infinite is None:
+                    ret_None = True
+
+        if ret_None:
+            return None
+        return True
+
+    @property
+    def has_reversed_limits(self):
+        """
+        Returns True if the limits are known to be in reversed order, either
+        by the explicit bounds, assumptions on the bounds, or assumptions on the
+        variables.  False if known to be in normal order, based on the bounds.
+        None if not enough information is available to determine.
+
+        Examples
+        ========
+
+        >>> from sympy import Sum, Integral, Product, oo, Symbol
+        >>> x = Symbol('x')
+        >>> Sum(x, (x, 8, 1)).has_reversed_limits
+        True
+
+        >>> Sum(x, (x, 1, oo)).has_reversed_limits
+        False
+
+        >>> M = Symbol('M')
+        >>> Integral(x, (x, 1, M)).has_reversed_limits
+
+        >>> N = Symbol('N', integer=True, positive=True)
+        >>> Sum(x, (x, 1, N)).has_reversed_limits
+        False
+
+        >>> Product(x, (x, 2, N)).has_reversed_limits
+
+        >>> Product(x, (x, 2, N)).subs(N, N + 2).has_reversed_limits
+        False
+
+        See Also
+        ========
+
+        sympy.concrete.expr_with_intlimits.ExprWithIntLimits.has_empty_sequence
+
+        """
+        ret_None = False
+        for lim in self.limits:
+            if len(lim) == 3:
+                var, a, b = lim
+                dif = b - a
+                if dif.is_extended_negative:
+                    return True
+                elif dif.is_extended_nonnegative:
+                    continue
+                else:
+                    ret_None = True
+            else:
+                return None
+        if ret_None:
+            return None
+        return False
+
 
 class AddWithLimits(ExprWithLimits):
     r"""Represents unevaluated oriented additions.
@@ -425,12 +541,13 @@ class AddWithLimits(ExprWithLimits):
         return self
 
     def _eval_expand_basic(self, **hints):
+        from sympy.matrices.matrices import MatrixBase
+
         summand = self.function.expand(**hints)
         if summand.is_Add and summand.is_commutative:
             return Add(*[self.func(i, *self.limits) for i in summand.args])
-        elif summand.is_Matrix:
-            return Matrix._new(summand.rows, summand.cols,
-                [self.func(i, *self.limits) for i in summand._mat])
+        elif isinstance(summand, MatrixBase):
+            return summand.applyfunc(lambda x: self.func(x, *self.limits))
         elif summand != self.function:
             return self.func(summand, *self.limits)
         return self
