@@ -5,10 +5,16 @@ from sympy.core.exprtools import factor_terms
 from sympy.functions.elementary.exponential import exp, log
 from sympy.polys import quo, roots
 from sympy.simplify import powsimp
+from sympy.core.function import Derivative
+from sympy.core.symbol import Dummy, Symbol
 
 
 class Product(ExprWithIntLimits):
-    r"""Represents unevaluated products.
+    r"""
+    Represents unevaluated products.
+
+    Explanation
+    ===========
 
     ``Product`` represents a finite or infinite product, with the first
     argument being the general form of terms in the series, and the second
@@ -92,7 +98,7 @@ class Product(ExprWithIntLimits):
     Product(4*i**2/((2*i - 1)*(2*i + 1)), (i, 1, n))
     >>> W2e = W2.doit()
     >>> W2e
-    2**(-2*n)*4**n*factorial(n)**2/(RisingFactorial(1/2, n)*RisingFactorial(3/2, n))
+    4**n*factorial(n)**2/(2**(2*n)*RisingFactorial(1/2, n)*RisingFactorial(3/2, n))
     >>> limit(W2e, n, oo)
     pi/2
 
@@ -395,11 +401,29 @@ class Product(ExprWithIntLimits):
         (k, a, n) = limits
         return Mul(*[term.subs(k, a + i) for i in range(n - a + 1)])
 
+    def _eval_derivative(self, x):
+        from sympy.concrete.summations import Sum
+        if isinstance(x, Symbol) and x not in self.free_symbols:
+            return S.Zero
+        f, limits = self.function, list(self.limits)
+        limit = limits.pop(-1)
+        if limits:
+            f = self.func(f, *limits)
+        i, a, b = limit
+        if x in a.free_symbols or x in b.free_symbols:
+            return None
+        h = Dummy()
+        rv = Sum( Product(f, (i, a, h - 1)) * Product(f, (i, h + 1, b)) * Derivative(f, x, evaluate=True).subs(i, h), (h, a, b))
+        return rv
+
     def is_convergent(self):
         r"""
         See docs of :obj:`.Sum.is_convergent()` for explanation of convergence
         in SymPy.
 
+        Explanation
+        ===========
+
         The infinite product:
 
         .. math::
@@ -458,8 +482,8 @@ class Product(ExprWithIntLimits):
         """
         Reverse the order of a limit in a Product.
 
-        Usage
-        =====
+        Explanation
+        ===========
 
         ``reverse_order(expr, *indices)`` reverses some limits in the expression
         ``expr`` which can be either a ``Sum`` or a ``Product``. The selectors in
@@ -545,6 +569,9 @@ def product(*args, **kwargs):
     r"""
     Compute the product.
 
+    Explanation
+    ===========
+
     The notation for symbols is similar to the notation used in Sum or
     Integral. product(f, (i, a, b)) computes the product of f with
     respect to i from a to b, i.e.,
@@ -560,6 +587,9 @@ def product(*args, **kwargs):
     If it cannot compute the product, it returns an unevaluated Product object.
     Repeated products can be computed by introducing additional symbols tuples::
 
+    Examples
+    ========
+
     >>> from sympy import product, symbols
     >>> i, n, m, k = symbols('i n m k', integer=True)
 
