@@ -11,20 +11,29 @@ from sympy.core.singleton import S
 from sympy.core.symbol import Dummy, Wild, Symbol
 from sympy.functions.special.zeta_functions import zeta
 from sympy.functions.elementary.piecewise import Piecewise
+from sympy.functions.elementary.trigonometric import cot, csc
 from sympy.logic.boolalg import And
-from sympy.polys import apart, PolynomialError, together
+from sympy.polys import apart, together
+from sympy.polys.polyerrors import PolynomialError, PolificationFailed
+from sympy.polys.polytools import parallel_poly_from_expr
 from sympy.series.limitseq import limit_seq
 from sympy.series.order import O
+from sympy.series.residues import residue
 from sympy.sets.sets import FiniteSet
 from sympy.simplify import denom
 from sympy.simplify.combsimp import combsimp
 from sympy.simplify.powsimp import powsimp
 from sympy.solvers import solve
 from sympy.solvers.solveset import solveset
+from sympy.utilities.iterables import sift
 import itertools
 
 class Sum(AddWithLimits, ExprWithIntLimits):
-    r"""Represents unevaluated summation.
+    r"""
+    Represents unevaluated summation.
+
+    Explanation
+    ===========
 
     ``Sum`` represents a finite or infinite series, with the first argument
     being the general form of terms in the series, and the second argument
@@ -269,6 +278,9 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         Differentiate wrt x as long as x is not in the free symbols of any of
         the upper or lower limits.
 
+        Explanation
+        ===========
+
         Sum(a*b*x, (x, 1, a)) can be differentiated wrt x or b but not `a`
         since the value of the sum is discontinuous in `a`. In a case
         involving a limit variable, the unevaluated derivative is returned.
@@ -344,7 +356,11 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         return factor_sum(result, limits=self.limits)
 
     def is_convergent(self):
-        r"""Checks for the convergence of a Sum.
+        r"""
+        Checks for the convergence of a Sum.
+
+        Explanation
+        ===========
 
         We divide the study of convergence of infinite sums and products in
         two parts.
@@ -765,8 +781,8 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         """
         Reverse the order of a limit in a Sum.
 
-        Usage
-        =====
+        Explanation
+        ===========
 
         ``reverse_order(self, *indices)`` reverses some limits in the expression
         ``self`` which can be either a ``Sum`` or a ``Product``. The selectors in
@@ -845,6 +861,9 @@ def summation(f, *symbols, **kwargs):
     r"""
     Compute the summation of f with respect to symbols.
 
+    Explanation
+    ===========
+
     The notation for symbols is similar to the notation used in Integral.
     summation(f, (i, a, b)) computes the sum of f with respect to i from a to b,
     i.e.,
@@ -861,6 +880,9 @@ def summation(f, *symbols, **kwargs):
     If it cannot compute the sum, it returns an unevaluated Sum object.
     Repeated sums can be computed by introducing additional symbols tuples::
 
+    Examples
+    ========
+
     >>> from sympy import summation, oo, symbols, log
     >>> i, n, m = symbols('i n m', integer=True)
 
@@ -889,13 +911,18 @@ def summation(f, *symbols, **kwargs):
 
 
 def telescopic_direct(L, R, n, limits):
-    """Returns the direct summation of the terms of a telescopic sum
+    """
+    Returns the direct summation of the terms of a telescopic sum
+
+    Explanation
+    ===========
 
     L is the term with lower index
     R is the term with higher index
     n difference between the indexes of L and R
 
-    For example:
+    Examples
+    ========
 
     >>> from sympy.concrete.summations import telescopic_direct
     >>> from sympy.abc import k, a, b
@@ -911,9 +938,10 @@ def telescopic_direct(L, R, n, limits):
 
 
 def telescopic(L, R, limits):
-    '''Tries to perform the summation using the telescopic property
+    '''
+    Tries to perform the summation using the telescopic property.
 
-    return None if not possible
+    Return None if not possible.
     '''
     (i, a, b) = limits
     if L.is_Add or R.is_Add:
@@ -1161,14 +1189,11 @@ def eval_sum_symbolic(f, limits):
             if e_exp is not None:
                 e.update(e_exp)
 
-        if e is not None:
-            p = (c1**c3).subs(e)
-            q = (c1**c2).subs(e)
-
-            r = p*(q**a - q**(b + 1))/(1 - q)
-            l = p*(b - a + 1)
-
-            return Piecewise((l, Eq(q, S.One)), (r, True))
+                p = (c1**c3).subs(e)
+                q = (c1**c2).subs(e)
+                r = p*(q**a - q**(b + 1))/(1 - q)
+                l = p*(b - a + 1)
+                return Piecewise((l, Eq(q, S.One)), (r, True))
 
         r = gosper_sum(f, (i, a, b))
 
@@ -1198,6 +1223,10 @@ def eval_sum_symbolic(f, limits):
     if h is not None:
         return h
 
+    r = eval_sum_residue(f_orig, (i, a, b))
+    if r is not None:
+        return r
+
     factored = f_orig.factor()
     if factored != f_orig:
         return eval_sum_symbolic(factored, (i, a, b))
@@ -1311,6 +1340,226 @@ def eval_sum_hyper(f, i_a_b):
         return Piecewise(res, (old_sum, True))
 
 
+def eval_sum_residue(f, i_a_b):
+    r"""Compute the infinite summation with residues
+
+    Notes
+    =====
+
+    If $f(n), g(n)$ are polynomials with $\deg(g(n)) - \deg(f(n)) >= 2$,
+    some infinite summations can be computed by the following residue
+    evaluations.
+
+    .. math::
+        \sum_{n=-\infty, g(n) \ne 0}^{\infty} \frac{f(n)}{g(n)} =
+        -\pi \sum_{\alpha|g(\alpha)=0}
+        \text{Res}(\cot(\pi x) \frac{f(x)}{g(x)}, \alpha)
+
+    .. math::
+        \sum_{n=-\infty, g(n) \ne 0}^{\infty} (-1)^n \frac{f(n)}{g(n)} =
+        -\pi \sum_{\alpha|g(\alpha)=0}
+        \text{Res}(\csc(\pi x) \frac{f(x)}{g(x)}, \alpha)
+
+    Examples
+    ========
+
+    >>> from sympy import Sum, oo, Symbol
+    >>> x = Symbol('x')
+
+    Doubly infinite series of rational functions.
+
+    >>> Sum(1 / (x**2 + 1), (x, -oo, oo)).doit()
+    pi/tanh(pi)
+
+    Doubly infinite alternating series of rational functions.
+
+    >>> Sum((-1)**x / (x**2 + 1), (x, -oo, oo)).doit()
+    pi/sinh(pi)
+
+    Infinite series of even rational functions.
+
+    >>> Sum(1 / (x**2 + 1), (x, 0, oo)).doit()
+    1/2 + pi/(2*tanh(pi))
+
+    Infinite series of alternating even rational functions.
+
+    >>> Sum((-1)**x / (x**2 + 1), (x, 0, oo)).doit()
+    pi/(2*sinh(pi)) + 1/2
+
+    This also have heuristics to transform arbitrarily shifted summand or
+    arbitrarily shifted summation range to the canonical problem the
+    formula can handle.
+
+    >>> Sum(1 / (x**2 + 2*x + 2), (x, -1, oo)).doit()
+    1/2 + pi/(2*tanh(pi))
+    >>> Sum(1 / (x**2 + 4*x + 5), (x, -2, oo)).doit()
+    1/2 + pi/(2*tanh(pi))
+    >>> Sum(1 / (x**2 + 1), (x, 1, oo)).doit()
+    -1/2 + pi/(2*tanh(pi))
+    >>> Sum(1 / (x**2 + 1), (x, 2, oo)).doit()
+    -1 + pi/(2*tanh(pi))
+
+    References
+    ==========
+
+    .. [#] http://www.supermath.info/InfiniteSeriesandtheResidueTheorem.pdf
+
+    .. [#] Asmar N.H., Grafakos L. (2018) Residue Theory.
+           In: Complex Analysis with Applications.
+           Undergraduate Texts in Mathematics. Springer, Cham.
+           https://doi.org/10.1007/978-3-319-94063-2_5
+    """
+    i, a, b = i_a_b
+
+    def is_even_function(numer, denom):
+        """Test if the rational function is an even function"""
+        numer_even = all([i % 2 == 0 for (i,) in numer.monoms()])
+        denom_even = all([i % 2 == 0 for (i,) in denom.monoms()])
+        numer_odd = all([i % 2 == 1 for (i,) in numer.monoms()])
+        denom_odd = all([i % 2 == 1 for (i,) in denom.monoms()])
+        return (numer_even and denom_even) or (numer_odd and denom_odd)
+
+    def match_rational(f, i):
+        numer, denom = f.as_numer_denom()
+        try:
+            (numer, denom), opt = parallel_poly_from_expr((numer, denom), i)
+        except (PolificationFailed, PolynomialError):
+            return None
+        return numer, denom
+
+    def get_poles(denom):
+        roots = denom.sqf_part().all_roots()
+        roots = sift(roots, lambda x: x.is_integer)
+        if None in roots:
+            return None
+        int_roots, nonint_roots = roots[True], roots[False]
+        return int_roots, nonint_roots
+
+    def get_shift(denom):
+        n = denom.degree(i)
+        a = denom.coeff_monomial(i**n)
+        b = denom.coeff_monomial(i**(n-1))
+        shift = - b / a / n
+        return shift
+
+    def get_residue_factor(numer, denom, alternating):
+        if not alternating:
+            residue_factor = (numer.as_expr() / denom.as_expr()) * cot(S.Pi * i)
+        else:
+            residue_factor = (numer.as_expr() / denom.as_expr()) * csc(S.Pi * i)
+        return residue_factor
+
+    # We don't know how to deal with symbolic constants in summand
+    if f.free_symbols - set([i]):
+        return None
+
+    if not (a.is_Integer or a in (S.Infinity, S.NegativeInfinity)):
+        return None
+    if not (b.is_Integer or b in (S.Infinity, S.NegativeInfinity)):
+        return None
+
+    # Quick exit heuristic for the sums which doesn't have infinite range
+    if a != S.NegativeInfinity and b != S.Infinity:
+        return None
+
+    match = match_rational(f, i)
+    if match:
+        alternating = False
+        numer, denom = match
+    else:
+        match = match_rational(f / (-1)**i, i)
+        if match:
+            alternating = True
+            numer, denom = match
+        else:
+            return None
+
+    if denom.degree(i) - numer.degree(i) < 2:
+        return None
+
+    if (a, b) == (S.NegativeInfinity, S.Infinity):
+        poles = get_poles(denom)
+        if poles is None:
+            return None
+        int_roots, nonint_roots = poles
+
+        if int_roots:
+            return None
+
+        residue_factor = get_residue_factor(numer, denom, alternating)
+        residues = [residue(residue_factor, i, root) for root in nonint_roots]
+        return -S.Pi * sum(residues)
+
+    if not (a.is_finite and b is S.Infinity):
+        return None
+
+    if not is_even_function(numer, denom):
+        # Try shifting summation and check if the summand can be made
+        # and even function from the origin.
+        # Sum(f(n), (n, a, b)) => Sum(f(n + s), (n, a - s, b - s))
+        shift = get_shift(denom)
+
+        if not shift.is_Integer:
+            return None
+        if shift == 0:
+            return None
+
+        numer = numer.shift(shift)
+        denom = denom.shift(shift)
+
+        if not is_even_function(numer, denom):
+            return None
+
+        if alternating:
+            f = (-1)**i * ((-1)**shift * numer.as_expr() / denom.as_expr())
+        else:
+            f = numer.as_expr() / denom.as_expr()
+        return eval_sum_residue(f, (i, a-shift, b-shift))
+
+    poles = get_poles(denom)
+    if poles is None:
+        return None
+    int_roots, nonint_roots = poles
+
+    if int_roots:
+        int_roots = [int(root) for root in int_roots]
+        int_roots_max = max(int_roots)
+        int_roots_min = min(int_roots)
+        # Integer valued poles must be next to each other
+        # and also symmetric from origin (Because the function is even)
+        if not len(int_roots) == int_roots_max - int_roots_min + 1:
+            return None
+
+        # Check whether the summation indices contain poles
+        if a <= max(int_roots):
+            return None
+
+    residue_factor = get_residue_factor(numer, denom, alternating)
+    residues = [residue(residue_factor, i, root) for root in int_roots + nonint_roots]
+    full_sum = -S.Pi * sum(residues)
+
+    if not int_roots:
+        # Compute Sum(f, (i, 0, oo)) by adding a extraneous evaluation
+        # at the origin.
+        half_sum = (full_sum + f.xreplace({i: 0})) / 2
+
+        # Add and subtract extraneous evaluations
+        extraneous_neg = [f.xreplace({i: i0}) for i0 in range(int(a), 0)]
+        extraneous_pos = [f.xreplace({i: i0}) for i0 in range(0, int(a))]
+        result = half_sum + sum(extraneous_neg) - sum(extraneous_pos)
+
+        return result
+
+    # Compute Sum(f, (i, min(poles) + 1, oo))
+    half_sum = full_sum / 2
+
+    # Subtract extraneous evaluations
+    extraneous = [f.xreplace({i: i0}) for i0 in range(max(int_roots) + 1, int(a))]
+    result = half_sum - sum(extraneous)
+
+    return result
+
+
 def _eval_matrix_sum(expression):
     f = expression.function
     for n, limit in enumerate(expression.limits):
