@@ -1,5 +1,6 @@
 from sympy.core.numbers import nan
 from .function import Function
+from .kind import NumberKind
 
 
 class Mod(Function):
@@ -31,12 +32,15 @@ class Mod(Function):
 
     """
 
+    kind = NumberKind
+
     @classmethod
     def eval(cls, p, q):
         from sympy.core.add import Add
         from sympy.core.mul import Mul
         from sympy.core.singleton import S
         from sympy.core.exprtools import gcd_terms
+        from sympy.polys.polyerrors import PolynomialError
         from sympy.polys.polytools import gcd
 
         def doit(p, q):
@@ -163,10 +167,13 @@ class Mod(Function):
         # XXX other possibilities?
 
         # extract gcd; any further simplification should be done by the user
-        G = gcd(p, q)
-        if G != 1:
-            p, q = [
-                gcd_terms(i/G, clear=False, fraction=False) for i in (p, q)]
+        try:
+            G = gcd(p, q)
+            if G != 1:
+                p, q = [gcd_terms(i/G, clear=False, fraction=False)
+                        for i in (p, q)]
+        except PolynomialError:  # issue 21373
+            G = S.One
         pwas, qwas = p, q
 
         # simplify terms
