@@ -1,6 +1,8 @@
 """sympify -- convert objects SymPy internal format"""
 
-# from typing import Any, Callable, Dict, Type
+import typing
+if typing.TYPE_CHECKING:
+    from typing import Any, Callable, Dict, Type
 
 from inspect import getmro
 
@@ -23,7 +25,7 @@ class SympifyError(ValueError):
 
 
 # See sympify docstring.
-converter = {}  ## type: Dict[Type[Any], Callable[[Any], Basic]]
+converter = {}  # type: Dict[Type[Any], Callable[[Any], Basic]]
 
 
 class CantSympify:
@@ -89,20 +91,17 @@ def _convert_numpy_types(a, **sympify_args):
 
 def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
         evaluate=None):
-    """Converts an arbitrary expression to a type that can be used inside SymPy.
+    """
+    Converts an arbitrary expression to a type that can be used inside SymPy.
 
-    For example, it will convert Python ints into instances of sympy.Integer,
+    Explanation
+    ===========
+
+    It will convert Python ints into instances of sympy.Integer,
     floats into instances of sympy.Float, etc. It is also able to coerce symbolic
     expressions which inherit from Basic. This can be useful in cooperation
     with SAGE.
 
-    It currently accepts as arguments:
-       - any object defined in SymPy
-       - standard numeric python types: int, long, float, Decimal
-       - strings (like "0.09" or "2e-19")
-       - booleans, including ``None`` (will leave ``None`` unchanged)
-       - dict, lists, sets or tuples containing any of the above
-
     .. warning::
         Note that this function uses ``eval``, and thus shouldn't be used on
         unsanitized input.
@@ -111,6 +110,9 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     nothing but return that value. This can be used at the beginning of a
     function to ensure you are working with the correct type.
 
+    Examples
+    ========
+
     >>> from sympy import sympify
 
     >>> sympify(2).is_integer
@@ -130,7 +132,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     >>> sympify("x***2")
     Traceback (most recent call last):
     ...
-    SympifyError: SympifyError: "could not parse u'x***2'"
+    SympifyError: SympifyError: "could not parse 'x***2'"
 
     Locals
     ------
@@ -155,9 +157,8 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     In order to have ``bitcount`` be recognized it can be imported into a
     namespace dictionary and passed as locals:
 
-    >>> from sympy.core.compatibility import exec_
     >>> ns = {}
-    >>> exec_('from sympy.core.evalf import bitcount', ns)
+    >>> exec('from sympy.core.evalf import bitcount', ns)
     >>> sympify(s, locals=ns)
     6
 
@@ -167,7 +168,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
 
     >>> from sympy import Symbol
     >>> ns["O"] = Symbol("O")  # method 1
-    >>> exec_('from sympy.abc import O', ns)  # method 2
+    >>> exec('from sympy.abc import O', ns)  # method 2
     >>> ns.update(dict(O=Symbol("O")))  # method 3
     >>> sympify("O + 1", locals=ns)
     O + 1
@@ -179,8 +180,8 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     names that are defined in abc).
 
     >>> from sympy.abc import _clash1
-    >>> _clash1
-    {'C': C, 'E': E, 'I': I, 'N': N, 'O': O, 'Q': Q, 'S': S}
+    >>> set(_clash1)
+    {'E', 'I', 'N', 'O', 'Q', 'S'}
     >>> sympify('I & Q', _clash1)
     I & Q
 
@@ -206,12 +207,21 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     ``evaluate=False`` option will be added. Nested ``Add`` or ``Mul`` will
     be denested first. This is done via an AST transformation that replaces
     operators with their SymPy equivalents, so if an operand redefines any
-    of those operations, the redefined operators will not be used.
+    of those operations, the redefined operators will not be used. If
+    argument a is not a string, the mathematical expression is evaluated
+    before being passed to sympify, so adding evaluate=False will still
+    return the evaluated result of expression.
 
     >>> sympify('2**2 / 3 + 5')
     19/3
     >>> sympify('2**2 / 3 + 5', evaluate=False)
     2**2/3 + 5
+    >>> sympify('4/2+7', evaluate=True)
+    9
+    >>> sympify('4/2+7', evaluate=False)
+    4/2 + 7
+    >>> sympify(4/2+7, evaluate=False)
+    9.00000000000000
 
     Extending
     ---------
@@ -257,6 +267,22 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     The keywords ``rational`` and ``convert_xor`` are only used
     when the input is a string.
 
+    convert_xor
+    -----------
+
+    >>> sympify('x^y',convert_xor=True)
+    x**y
+    >>> sympify('x^y',convert_xor=False)
+    x ^ y
+
+    rational
+    --------
+
+    >>> sympify('0.1',rational=False)
+    0.1
+    >>> sympify('0.1',rational=True)
+    1/10
+
     Sometimes autosimplification during sympification results in expressions
     that are very different in structure than what was entered. Until such
     autosimplification is no longer done, the ``kernS`` function might be of
@@ -273,10 +299,57 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     >>> kernS(s)
     -2*(-(-x + 1/x)/(x*(x - 1/x)**2) - 1/(x*(x - 1/x))) - 1
 
+    Parameters
+    ==========
+
+    a :
+        - any object defined in SymPy
+        - standard numeric python types: int, long, float, Decimal
+        - strings (like "0.09", "2e-19" or 'sin(x)')
+        - booleans, including ``None`` (will leave ``None`` unchanged)
+        - dict, lists, sets or tuples containing any of the above
+
+    convert_xor : boolean, optional
+        If true, treats XOR as exponentiation.
+        If False, treats XOR as XOR itself.
+        Used only when input is a string.
+
+    locals : any object defined in SymPy, optional
+        In order to have strings be recognized it can be imported
+        into a namespace dictionary and passed as locals.
+
+    strict : boolean, optional
+        If the option strict is set to True, only the types for which
+        an explicit conversion has been defined are converted. In the
+        other cases, a SympifyError is raised.
+
+    rational : boolean, optional
+        If true, converts floats into Rational.
+        If false, it lets floats remain as it is.
+        Used only when input is a string.
+
+    evaluate : boolean, optional
+        If False, then arithmetic and operators will be converted into
+        their SymPy equivalents. If True the expression will be evaluated
+        and the result will be returned.
+
     """
+    # XXX: If a is a Basic subclass rather than instance (e.g. sin rather than
+    # sin(x)) then a.__sympy__ will be the property. Only on the instance will
+    # a.__sympy__ give the *value* of the property (True). Since sympify(sin)
+    # was used for a long time we allow it to pass. However if strict=True as
+    # is the case in internal calls to _sympify then we only allow
+    # is_sympy=True.
+    #
+    # https://github.com/sympy/sympy/issues/20124
     is_sympy = getattr(a, '__sympy__', None)
-    if is_sympy is not None:
+    if is_sympy is True:
         return a
+    elif is_sympy is not None:
+        if not strict:
+            return a
+        else:
+            raise SympifyError(a)
 
     if isinstance(a, CantSympify):
         raise SympifyError(a)
@@ -362,14 +435,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     if iterable(a):
         try:
             return type(a)([sympify(x, locals=locals, convert_xor=convert_xor,
-                rational=rational) for x in a])
-        except TypeError:
-            # Not all iterables are rebuildable with their type.
-            pass
-    if isinstance(a, dict):
-        try:
-            return type(a)([sympify(x, locals=locals, convert_xor=convert_xor,
-                rational=rational) for x in a.items()])
+                rational=rational, evaluate=evaluate) for x in a])
         except TypeError:
             # Not all iterables are rebuildable with their type.
             pass
@@ -448,7 +514,7 @@ def kernS(s):
     ========
 
     >>> from sympy.core.sympify import kernS
-    >>> from sympy.abc import x, y, z
+    >>> from sympy.abc import x, y
 
     The 2-arg Mul distributes a number (or minus sign) across the terms
     of an expression, but kernS will prevent that:
@@ -513,7 +579,9 @@ def kernS(s):
             while kern in s:
                 kern += choice(string.ascii_letters + string.digits)
             s = s.replace(' ', kern)
-        hit = kern in s
+            hit = kern in s
+        else:
+            hit = False
 
     for i in range(2):
         try:
@@ -542,4 +610,4 @@ def kernS(s):
 
 
 # Avoid circular import
-# from .basic import Basic
+from .basic import Basic
