@@ -4,7 +4,8 @@ from sympy.multipledispatch import dispatch
 from sympy.testing.pytest import XFAIL, raises, warns_deprecated_sympy
 from sympy import (S, Symbol, symbols, nan, oo, I, pi, Float, And, Or,
                    Not, Implies, Xor, zoo, sqrt, Rational, simplify, Function,
-                   log, cos, sin, Add, Mul, Pow, floor, ceiling, trigsimp, Reals, Basic, Expr)
+                   log, cos, sin, Add, Mul, Pow, floor, ceiling, trigsimp, Reals,
+                   Basic, Expr, Q, exp, exp_polar)
 from sympy.core.relational import (Relational, Equality, Unequality,
                                    GreaterThan, LessThan, StrictGreaterThan,
                                    StrictLessThan, Rel, Eq, Lt, Le,
@@ -786,6 +787,13 @@ def test_simplify_relational():
     assert Lt(x, 2).simplify() == Lt(x, 2)
     assert Lt(-x, 2).simplify() == Gt(x, -2)
 
+    # Test particulat branches of _eval_simplify
+    m = exp(1) - exp_polar(1)
+    assert simplify(m*x > 1) is S.false
+    # These two tests the same branch
+    assert simplify(m*x + 2*m*y > 1) is S.false
+    assert simplify(m*x + y > 1 + y) is S.false
+
 
 def test_equals():
     w, x, y, z = symbols('w:z')
@@ -1159,7 +1167,24 @@ def test_EvalEq():
     assert is_eq(PowTest(3, 4), _sympify(4)) is None
     assert is_neq(PowTest(3, 4), PowTest(3,7))
 
+
 def test_is_eq():
+    # test assumptions
+    assert is_eq(x, y, Q.infinite(x) & Q.finite(y)) is False
+    assert is_eq(x, y, Q.infinite(x) & Q.infinite(y) & Q.extended_real(x) & ~Q.extended_real(y)) is False
+    assert is_eq(x, y, Q.infinite(x) & Q.infinite(y) & Q.extended_positive(x) & Q.extended_negative(y)) is False
+
+    assert is_eq(x+I, y+I, Q.infinite(x) & Q.finite(y)) is False
+    assert is_eq(1+x*I, 1+y*I, Q.infinite(x) & Q.finite(y)) is False
+
+    assert is_eq(x, S(0), assumptions=Q.zero(x))
+    assert is_eq(x, S(0), assumptions=~Q.zero(x)) is False
+    assert is_eq(x, S(0), assumptions=Q.nonzero(x)) is False
+    assert is_neq(x, S(0), assumptions=Q.zero(x)) is False
+    assert is_neq(x, S(0), assumptions=~Q.zero(x))
+    assert is_neq(x, S(0), assumptions=Q.nonzero(x))
+
+    # test registration
     class PowTest(Expr):
         def __new__(cls, base, exp):
             return Basic.__new__(cls, _sympify(base), _sympify(exp))
@@ -1173,7 +1198,13 @@ def test_is_eq():
     assert is_eq(PowTest(3, 4), _sympify(4)) is None
     assert is_neq(PowTest(3, 4), PowTest(3,7))
 
+
 def test_is_ge_le():
+    # test assumptions
+    assert is_ge(x, S(0), Q.nonnegative(x)) is True
+    assert is_ge(x, S(0), Q.negative(x)) is False
+
+    # test registration
     class PowTest(Expr):
         def __new__(cls, base, exp):
             return Basic.__new__(cls, _sympify(base), _sympify(exp))
