@@ -3,7 +3,7 @@ from sympy import AccumBounds, Symbol, floor, nan, oo, zoo, E, symbols, \
         Le, Ge, Gt, Lt, Ne, sqrt, S
 
 from sympy.core.expr import unchanged
-from sympy.utilities.pytest import XFAIL
+from sympy.testing.pytest import XFAIL
 
 x = Symbol('x')
 i = Symbol('i', imaginary=True)
@@ -548,6 +548,12 @@ def test_issue_4149():
     assert floor(3 + E + pi*I + y*I) == 5 + floor(pi + y)*I
 
 
+def test_issue_21651():
+    k = Symbol('k', positive=True, integer=True)
+    exp = 2*2**(-k)
+    assert isinstance(floor(exp), floor)
+
+
 def test_issue_11207():
     assert floor(floor(x)) == floor(x)
     assert floor(ceiling(x)) == ceiling(x)
@@ -560,3 +566,12 @@ def test_nested_floor_ceiling():
     assert ceiling(-floor(ceiling(x**3)/y)) == -floor(ceiling(x**3)/y)
     assert floor(ceiling(-floor(x**Rational(7, 2)/y))) == -floor(x**Rational(7, 2)/y)
     assert -ceiling(-ceiling(floor(x)/y)) == ceiling(floor(x)/y)
+
+def test_issue_18689():
+    assert floor(floor(floor(x)) + 3) == floor(x) + 3
+    assert ceiling(ceiling(ceiling(x)) + 1) == ceiling(x) + 1
+    assert ceiling(ceiling(floor(x)) + 3) == floor(x) + 3
+
+def test_issue_18421():
+    assert floor(float(0)) is S.Zero
+    assert ceiling(float(0)) is S.Zero
