@@ -15,7 +15,7 @@ from sympy.functions.elementary.trigonometric import cos, sin
 from sympy.functions.special.delta_functions import Heaviside
 
 from sympy.utilities.lambdify import lambdify
-from sympy.utilities.pytest import raises, skip, ignore_warnings
+from sympy.testing.pytest import raises, skip, ignore_warnings
 
 def test_Min():
     from sympy.abc import x, y, z
@@ -118,7 +118,6 @@ def test_Min():
 
     # issue 7233
     e = Min(0, x)
-    assert e.evalf == e.n
     assert e.n().args == (0, x)
 
     # issue 8643
@@ -185,7 +184,6 @@ def test_Max():
         + Heaviside(x - Max(1, x**2) + 1)
 
     e = Max(0, x)
-    assert e.evalf == e.n
     assert e.n().args == (0, x)
 
     # issue 8643
@@ -412,14 +410,18 @@ def test_issue_12638():
     assert Min(a, b, Max(a, b, c)) == Min(a, b)
     assert Min(a, b, Max(a, c)) == Min(a, b)
 
+def test_issue_21399():
+    from sympy.abc import a, b, c
+    assert Max(Min(a, b), Min(a, b, c)) == Min(a, b)
+
 
 def test_instantiation_evaluation():
     from sympy.abc import v, w, x, y, z
     assert Min(1, Max(2, x)) == 1
     assert Max(3, Min(2, x)) == 3
     assert Min(Max(x, y), Max(x, z)) == Max(x, Min(y, z))
-    assert set(Min(Max(w, x), Max(y, z)).args) == set(
-        [Max(w, x), Max(y, z)])
+    assert set(Min(Max(w, x), Max(y, z)).args) == {
+        Max(w, x), Max(y, z)}
     assert Min(Max(x, y), Max(x, z), w) == Min(
         w, Max(x, Min(y, z)))
     A, B = Min, Max
@@ -457,3 +459,9 @@ def test_issue_14000():
 
     assert root(16, 4, 2, evaluate=False).has(Pow) == True
     assert real_root(-8, 3, evaluate=False).has(Pow) == True
+
+def test_issue_6899():
+    from sympy import Lambda
+    x = Symbol('x')
+    eqn = Lambda(x, x)
+    assert eqn.func(*eqn.args) == eqn
