@@ -1,28 +1,26 @@
-from __future__ import print_function, division
-
 from functools import wraps
 
-from sympy import S, pi, I, Rational, Wild, cacheit, sympify
-from sympy.core.function import Function, ArgumentIndexError
+from sympy import Add, S, pi, I, Rational, Wild, cacheit, sympify
+from sympy.core.function import Function, ArgumentIndexError, _mexpand
+from sympy.core.logic import fuzzy_or, fuzzy_not
 from sympy.core.power import Pow
-from sympy.core.compatibility import range
 from sympy.functions.combinatorial.factorials import factorial
 from sympy.functions.elementary.trigonometric import sin, cos, csc, cot
+from sympy.functions.elementary.integers import ceiling
 from sympy.functions.elementary.complexes import Abs
+from sympy.functions.elementary.exponential import exp, log
 from sympy.functions.elementary.miscellaneous import sqrt, root
 from sympy.functions.elementary.complexes import re, im
-from sympy.functions.special.gamma_functions import gamma
+from sympy.functions.special.gamma_functions import gamma, digamma
 from sympy.functions.special.hyper import hyper
 from sympy.polys.orthopolys import spherical_bessel_fn as fn
 
-
 # TODO
 # o Scorer functions G1 and G2
 # o Asymptotic expansions
 #   These are possible, e.g. for fixed order, but since the bessel type
 #   functions are oscillatory they are not actually tractable at
 #   infinity, so this is not particularly useful right now.
-# o Series Expansions for functions of the second kind about zero
 # o Nicer series expansions.
 # o More rewriting.
 # o Add solvers to ode.py (or rather add solvers for the hypergeometric equation).
@@ -30,27 +28,28 @@ from sympy.polys.orthopolys import spherical_bessel_fn as fn
 
 class BesselBase(Function):
     """
-    Abstract base class for bessel-type functions.
+    Abstract base class for Bessel-type functions.
 
     This class is meant to reduce code duplication.
-    All Bessel type functions can 1) be differentiated, and the derivatives
-    expressed in terms of similar functions and 2) be rewritten in terms
-    of other bessel-type functions.
+    All Bessel-type functions can 1) be differentiated, with the derivatives
+    expressed in terms of similar functions, and 2) be rewritten in terms
+    of other Bessel-type functions.
 
-    Here "bessel-type functions" are assumed to have one complex parameter.
+    Here, Bessel-type functions are assumed to have one complex parameter.
 
     To use this base class, define class attributes ``_a`` and ``_b`` such that
     ``2*F_n' = -_a*F_{n+1} + b*F_{n-1}``.
+
     """
 
     @property
     def order(self):
-        """ The order of the bessel-type function. """
+        """ The order of the Bessel-type function. """
         return self.args[0]
 
     @property
     def argument(self):
-        """ The argument of the bessel-type function. """
+        """ The argument of the Bessel-type function. """
         return self.args[1]
 
     @classmethod
@@ -68,13 +67,26 @@ class BesselBase(Function):
         if z.is_extended_negative is False:
             return self.__class__(self.order.conjugate(), z.conjugate())
 
+    def _eval_is_meromorphic(self, x, a):
+        nu, z = self.order, self.argument
+
+        if nu.has(x):
+            return False
+        if not z._eval_is_meromorphic(x, a):
+            return None
+        z0 = z.subs(x, a)
+        if nu.is_integer:
+            if isinstance(self, (besselj, besseli, hn1, hn2, jn, yn)) or not nu.is_zero:
+                return fuzzy_not(z0.is_infinite)
+        return fuzzy_not(fuzzy_or([z0.is_zero, z0.is_infinite]))
+
     def _eval_expand_func(self, **hints):
         nu, z, f = self.order, self.argument, self.__class__
-        if nu.is_extended_real:
-            if (nu - 1).is_extended_positive:
+        if nu.is_real:
+            if (nu - 1).is_positive:
                 return (-self._a*self._b*f(nu - 2, z)._eval_expand_func() +
                         2*self._a*(nu - 1)*f(nu - 1, z)._eval_expand_func()/z)
-            elif (nu + 1).is_extended_negative:
+            elif (nu + 1).is_negative:
                 return (2*self._b*(nu + 1)*f(nu + 1, z)._eval_expand_func()/z -
                         self._a*self._b*f(nu + 2, z)._eval_expand_func())
         return self
@@ -88,6 +100,9 @@ class besselj(BesselBase):
     r"""
     Bessel function of the first kind.
 
+    Explanation
+    ===========
+
     The Bessel $J$ function of order $\nu$ is defined to be the function
     satisfying Bessel's differential equation
 
@@ -147,6 +162,7 @@ class besselj(BesselBase):
            Approximations, Volume 1
     .. [3] https://en.wikipedia.org/wiki/Bessel_function
     .. [4] http://functions.wolfram.com/Bessel-TypeFunctions/BesselJ/
+
     """
 
     _a = S.One
@@ -176,7 +192,7 @@ class besselj(BesselBase):
                 return I**(nu)*besseli(nu, newz)
 
         # branch handling:
-        from sympy import unpolarify, exp
+        from sympy import unpolarify
         if nu.is_integer:
             newz = unpolarify(z)
             if newz != z:
@@ -190,7 +206,7 @@ class besselj(BesselBase):
             return besselj(nnu, z)
 
     def _eval_rewrite_as_besseli(self, nu, z, **kwargs):
-        from sympy import polar_lift, exp
+        from sympy import polar_lift
         return exp(I*pi*nu/2)*besseli(nu, polar_lift(-I)*z)
 
     def _eval_rewrite_as_bessely(self, nu, z, **kwargs):
@@ -200,20 +216,56 @@ class besselj(BesselBase):
     def _eval_rewrite_as_jn(self, nu, z, **kwargs):
         return sqrt(2*z/pi)*jn(nu - S.Half, self.argument)
 
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        nu, z = self.args
+        arg = z.as_leading_term(x)
+        if x in arg.free_symbols:
+            return arg**nu/(2**nu*gamma(nu + 1))
+        else:
+            return self.func(nu, z.subs(x, 0))
+
     def _eval_is_extended_real(self):
         nu, z = self.args
         if nu.is_integer and z.is_extended_real:
             return True
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.bessel_J(self.args[0]._sage_(), self.args[1]._sage_())
+    def _eval_nseries(self, x, n, logx, cdir=0):
+        from sympy.series.order import Order
+        nu, z = self.args
+
+        # In case of powers less than 1, number of terms need to be computed
+        # separately to avoid repeated callings of _eval_nseries with wrong n
+        try:
+            _, exp = z.leadterm(x)
+        except (ValueError, NotImplementedError):
+            return self
+
+        if exp.is_positive:
+            newn = ceiling(n/exp)
+            o = Order(x**n, x)
+            r = (z/2)._eval_nseries(x, n, logx, cdir).removeO()
+            if r is S.Zero:
+                return o
+            t = (_mexpand(r**2) + o).removeO()
+
+            term = r**nu/gamma(nu + 1)
+            s = [term]
+            for k in range(1, (newn + 1)//2):
+                term *= -t/(k*(nu + k))
+                term = (_mexpand(term) + o).removeO()
+                s.append(term)
+            return Add(*s) + o
+
+        return super(besselj, self)._eval_nseries(x, n, logx, cdir)
 
 
 class bessely(BesselBase):
     r"""
     Bessel function of the second kind.
 
+    Explanation
+    ===========
+
     The Bessel $Y$ function of order $\nu$ is defined as
 
     .. math ::
@@ -279,21 +331,74 @@ class bessely(BesselBase):
     def _eval_rewrite_as_yn(self, nu, z, **kwargs):
         return sqrt(2*z/pi) * yn(nu - S.Half, self.argument)
 
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        nu, z = self.args
+        term_one = ((2/pi)*log(z/2)*besselj(nu, z))
+        term_two = (z/2)**(-nu)*factorial(nu - 1)/pi if (nu - 1).is_positive else S.Zero
+        term_three = (z/2)**nu/(pi*factorial(nu))*(digamma(nu + 1) - S.EulerGamma)
+        arg = Add(*[term_one, term_two, term_three]).as_leading_term(x)
+        if x in arg.free_symbols:
+            return arg
+        else:
+            return self.func(nu, z.subs(x, 0).cancel())
+
     def _eval_is_extended_real(self):
         nu, z = self.args
         if nu.is_integer and z.is_positive:
             return True
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.bessel_Y(self.args[0]._sage_(), self.args[1]._sage_())
+    def _eval_nseries(self, x, n, logx, cdir=0):
+        from sympy.series.order import Order
+        nu, z = self.args
+
+        # In case of powers less than 1, number of terms need to be computed
+        # separately to avoid repeated callings of _eval_nseries with wrong n
+        try:
+            _, exp = z.leadterm(x)
+        except (ValueError, NotImplementedError):
+            return self
+
+        if exp.is_positive and nu.is_integer:
+            newn = ceiling(n/exp)
+            bn = besselj(nu, z)
+            a = ((2/pi)*log(z/2)*bn)._eval_nseries(x, n, logx, cdir)
+
+            b, c = [], []
+            o = Order(x**n, x)
+            r = (z/2)._eval_nseries(x, n, logx, cdir).removeO()
+            if r is S.Zero:
+                return o
+            t = (_mexpand(r**2) + o).removeO()
+
+            if nu > S.One:
+                term = r**(-nu)*factorial(nu - 1)/pi
+                b.append(term)
+                for k in range(1, nu - 1):
+                    term *= t*(nu - k - 1)/k
+                    term = (_mexpand(term) + o).removeO()
+                    b.append(term)
+
+            p = r**nu/(pi*factorial(nu))
+            term = p*(digamma(nu + 1) - S.EulerGamma)
+            c.append(term)
+            for k in range(1, (newn + 1)//2):
+                p *= -t/(k*(k + nu))
+                p = (_mexpand(p) + o).removeO()
+                term = p*(digamma(k + nu + 1) + digamma(k + 1))
+                c.append(term)
+            return a - Add(*b) - Add(*c) # Order term comes from a
+
+        return super(bessely, self)._eval_nseries(x, n, logx, cdir)
 
 
 class besseli(BesselBase):
     r"""
     Modified Bessel function of the first kind.
 
-    The Bessel I function is a solution to the modified Bessel equation
+    Explanation
+    ===========
+
+    The Bessel $I$ function is a solution to the modified Bessel equation
 
     .. math ::
         z^2 \frac{\mathrm{d}^2 w}{\mathrm{d}z^2}
@@ -340,9 +445,8 @@ class besseli(BesselBase):
                 return S.ComplexInfinity
             elif nu.is_imaginary:
                 return S.NaN
-        if z.is_imaginary:
-            if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
-                return S.Zero
+        if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
+            return S.Zero
 
         if z.could_extract_minus_sign():
             return (z)**nu*(-z)**(-nu)*besseli(nu, -z)
@@ -354,7 +458,7 @@ class besseli(BesselBase):
                 return I**(-nu)*besselj(nu, -newz)
 
         # branch handling:
-        from sympy import unpolarify, exp
+        from sympy import unpolarify
         if nu.is_integer:
             newz = unpolarify(z)
             if newz != z:
@@ -368,7 +472,7 @@ class besseli(BesselBase):
             return besseli(nnu, z)
 
     def _eval_rewrite_as_besselj(self, nu, z, **kwargs):
-        from sympy import polar_lift, exp
+        from sympy import polar_lift
         return exp(-I*pi*nu/2)*besselj(nu, polar_lift(I)*z)
 
     def _eval_rewrite_as_bessely(self, nu, z, **kwargs):
@@ -384,16 +488,15 @@ class besseli(BesselBase):
         if nu.is_integer and z.is_extended_real:
             return True
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.bessel_I(self.args[0]._sage_(), self.args[1]._sage_())
-
 
 class besselk(BesselBase):
     r"""
     Modified Bessel function of the second kind.
 
-    The Bessel K function of order $\nu$ is defined as
+    Explanation
+    ===========
+
+    The Bessel $K$ function of order $\nu$ is defined as
 
     .. math ::
         K_\nu(z) = \lim_{\mu \to \nu} \frac{\pi}{2}
@@ -436,9 +539,8 @@ class besselk(BesselBase):
                 return S.ComplexInfinity
             elif re(nu).is_zero:
                 return S.NaN
-        if z.is_imaginary:
-            if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
-                return S.Zero
+        if z in (S.Infinity, I*S.Infinity, I*S.NegativeInfinity):
+            return S.Zero
 
         if nu.is_integer:
             if nu.could_extract_minus_sign():
@@ -468,15 +570,14 @@ class besselk(BesselBase):
         if nu.is_integer and z.is_positive:
             return True
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.bessel_K(self.args[0]._sage_(), self.args[1]._sage_())
-
 
 class hankel1(BesselBase):
     r"""
     Hankel function of the first kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math ::
@@ -520,6 +621,9 @@ class hankel2(BesselBase):
     r"""
     Hankel function of the second kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math ::
@@ -576,26 +680,19 @@ class SphericalBesselBase(BesselBase):
     since spherical Bessel functions differ from the ordinary
     ones just by a slight change in order.
 
-    To use this class, define the ``_rewrite`` and ``_expand`` methods.
+    To use this class, define the ``_eval_evalf()`` and ``_expand()`` methods.
+
     """
 
     def _expand(self, **hints):
         """ Expand self into a polynomial. Nu is guaranteed to be Integer. """
         raise NotImplementedError('expansion')
 
-    def _rewrite(self):
-        """ Rewrite self in terms of ordinary Bessel functions. """
-        raise NotImplementedError('rewriting')
-
     def _eval_expand_func(self, **hints):
         if self.order.is_Integer:
             return self._expand(**hints)
         return self
 
-    def _eval_evalf(self, prec):
-        if self.order.is_Integer:
-            return self._rewrite()._eval_evalf(prec)
-
     def fdiff(self, argindex=2):
         if argindex != 2:
             raise ArgumentIndexError(self, argindex)
@@ -616,6 +713,9 @@ class jn(SphericalBesselBase):
     r"""
     Spherical Bessel function of the first kind.
 
+    Explanation
+    ===========
+
     This function is a solution to the spherical Bessel equation
 
     .. math ::
@@ -641,7 +741,6 @@ class jn(SphericalBesselBase):
     ========
 
     >>> from sympy import Symbol, jn, sin, cos, expand_func, besselj, bessely
-    >>> from sympy import simplify
     >>> z = Symbol("z")
     >>> nu = Symbol("nu", integer=True)
     >>> print(expand_func(jn(0, z)))
@@ -681,9 +780,6 @@ class jn(SphericalBesselBase):
         if z in (S.NegativeInfinity, S.Infinity):
             return S.Zero
 
-    def _rewrite(self):
-        return self._eval_rewrite_as_besselj(self.order, self.argument)
-
     def _eval_rewrite_as_besselj(self, nu, z, **kwargs):
         return sqrt(pi/(2*z)) * besselj(nu + S.Half, z)
 
@@ -696,11 +792,18 @@ class jn(SphericalBesselBase):
     def _expand(self, **hints):
         return _jn(self.order, self.argument)
 
+    def _eval_evalf(self, prec):
+        if self.order.is_Integer:
+            return self.rewrite(besselj)._eval_evalf(prec)
+
 
 class yn(SphericalBesselBase):
     r"""
     Spherical Bessel function of the second kind.
 
+    Explanation
+    ===========
+
     This function is another solution to the spherical Bessel equation, and
     linearly independent from $j_n$. It can be defined as
 
@@ -741,10 +844,6 @@ class yn(SphericalBesselBase):
     .. [1] http://dlmf.nist.gov/10.47
 
     """
-
-    def _rewrite(self):
-        return self._eval_rewrite_as_bessely(self.order, self.argument)
-
     @assume_integer_order
     def _eval_rewrite_as_besselj(self, nu, z, **kwargs):
         return (-1)**(nu+1) * sqrt(pi/(2*z)) * besselj(-nu - S.Half, z)
@@ -759,12 +858,13 @@ class yn(SphericalBesselBase):
     def _expand(self, **hints):
         return _yn(self.order, self.argument)
 
+    def _eval_evalf(self, prec):
+        if self.order.is_Integer:
+            return self.rewrite(bessely)._eval_evalf(prec)
+
 
 class SphericalHankelBase(SphericalBesselBase):
 
-    def _rewrite(self):
-        return self._eval_rewrite_as_besselj(self.order, self.argument)
-
     @assume_integer_order
     def _eval_rewrite_as_besselj(self, nu, z, **kwargs):
         # jn +- I*yn
@@ -815,11 +915,18 @@ class SphericalHankelBase(SphericalBesselBase):
 
         return (_jn(n, z) + hks*I*_yn(n, z)).expand()
 
+    def _eval_evalf(self, prec):
+        if self.order.is_Integer:
+            return self.rewrite(besselj)._eval_evalf(prec)
+
 
 class hn1(SphericalHankelBase):
     r"""
     Spherical Hankel function of the first kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math:: h_\nu^(1)(z) = j_\nu(z) + i y_\nu(z),
@@ -873,6 +980,9 @@ class hn2(SphericalHankelBase):
     r"""
     Spherical Hankel function of the second kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math:: h_\nu^(2)(z) = j_\nu(z) - i y_\nu(z),
@@ -926,7 +1036,10 @@ def jn_zeros(n, k, method="sympy", dps=15):
     """
     Zeros of the spherical Bessel function of the first kind.
 
-    This returns an array of zeros of jn up to the k-th zero.
+    Explanation
+    ===========
+
+    This returns an array of zeros of $jn$ up to the $k$-th zero.
 
     * method = "sympy": uses `mpmath.besseljzero
       <http://mpmath.org/doc/current/functions/bessel.html#mpmath.besseljzero>`_
@@ -937,9 +1050,9 @@ def jn_zeros(n, k, method="sympy", dps=15):
       to find all
       roots, which is faster than computing the zeros using a general
       numerical solver, but it requires SciPy and only works with low
-      precision floating point numbers.  [The function used with
-      method="sympy" is a recent addition to mpmath, before that a general
-      solver was used.]
+      precision floating point numbers. (The function used with
+      method="sympy" is a recent addition to mpmath; before that a general
+      solver was used.)
 
     Examples
     ========
@@ -952,6 +1065,17 @@ def jn_zeros(n, k, method="sympy", dps=15):
     ========
 
     jn, yn, besselj, besselk, bessely
+
+    Parameters
+    ==========
+
+    n : integer
+        order of Bessel function
+
+    k : integer
+        number of zeros to return
+
+
     """
     from math import pi
 
@@ -998,6 +1122,7 @@ class AiryBase(Function):
     Abstract base class for Airy functions.
 
     This class is meant to reduce code duplication.
+
     """
 
     def _eval_conjugate(self):
@@ -1023,6 +1148,9 @@ class airyai(AiryBase):
     r"""
     The Airy function $\operatorname{Ai}$ of the first kind.
 
+    Explanation
+    ===========
+
     The Airy function $\operatorname{Ai}(z)$ is defined to be the function
     satisfying Airy's differential equation
 
@@ -1062,7 +1190,7 @@ class airyai(AiryBase):
     >>> conjugate(airyai(z))
     airyai(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airyai(z), z)
@@ -1082,7 +1210,7 @@ class airyai(AiryBase):
     >>> airyai(-2).evalf(50)
     0.22740742820168557599192443603787379946077222541710
 
-    Rewrite Ai(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Ai}(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airyai(z).rewrite(hyper)
@@ -1102,6 +1230,7 @@ class airyai(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1191,6 +1320,9 @@ class airybi(AiryBase):
     r"""
     The Airy function $\operatorname{Bi}$ of the second kind.
 
+    Explanation
+    ===========
+
     The Airy function $\operatorname{Bi}(z)$ is defined to be the function
     satisfying Airy's differential equation
 
@@ -1232,7 +1364,7 @@ class airybi(AiryBase):
     >>> conjugate(airybi(z))
     airybi(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airybi(z), z)
@@ -1252,7 +1384,7 @@ class airybi(AiryBase):
     >>> airybi(-2).evalf(50)
     -0.41230258795639848808323405461146104203453483447240
 
-    Rewrite Bi(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Bi}(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airybi(z).rewrite(hyper)
@@ -1272,6 +1404,7 @@ class airybi(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1362,9 +1495,14 @@ class airybi(AiryBase):
 
 class airyaiprime(AiryBase):
     r"""
-    The derivative $\operatorname{Ai}^\prime$ of the Airy function of the first kind.
+    The derivative $\operatorname{Ai}^\prime$ of the Airy function of the first
+    kind.
 
-    The Airy function $\operatorname{Ai}^\prime(z)$ is defined to be the function
+    Explanation
+    ===========
+
+    The Airy function $\operatorname{Ai}^\prime(z)$ is defined to be the
+    function
 
     .. math::
         \operatorname{Ai}^\prime(z) := \frac{\mathrm{d} \operatorname{Ai}(z)}{\mathrm{d} z}.
@@ -1394,7 +1532,7 @@ class airyaiprime(AiryBase):
     >>> conjugate(airyaiprime(z))
     airyaiprime(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airyaiprime(z), z)
@@ -1414,7 +1552,7 @@ class airyaiprime(AiryBase):
     >>> airyaiprime(-2).evalf(50)
     0.61825902074169104140626429133247528291577794512415
 
-    Rewrite Ai'(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Ai}^\prime(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airyaiprime(z).rewrite(hyper)
@@ -1434,6 +1572,7 @@ class airyaiprime(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1515,9 +1654,14 @@ class airyaiprime(AiryBase):
 
 class airybiprime(AiryBase):
     r"""
-    The derivative $\operatorname{Bi}^\prime$ of the Airy function of the first kind.
+    The derivative $\operatorname{Bi}^\prime$ of the Airy function of the first
+    kind.
 
-    The Airy function $\operatorname{Bi}^\prime(z)$ is defined to be the function
+    Explanation
+    ===========
+
+    The Airy function $\operatorname{Bi}^\prime(z)$ is defined to be the
+    function
 
     .. math::
         \operatorname{Bi}^\prime(z) := \frac{\mathrm{d} \operatorname{Bi}(z)}{\mathrm{d} z}.
@@ -1549,7 +1693,7 @@ class airybiprime(AiryBase):
     >>> conjugate(airybiprime(z))
     airybiprime(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airybiprime(z), z)
@@ -1569,7 +1713,7 @@ class airybiprime(AiryBase):
     >>> airybiprime(-2).evalf(50)
     0.27879516692116952268509756941098324140300059345163
 
-    Rewrite Bi'(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Bi}^\prime(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airybiprime(z).rewrite(hyper)
@@ -1589,6 +1733,7 @@ class airybiprime(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1675,9 +1820,12 @@ class airybiprime(AiryBase):
 
 class marcumq(Function):
     r"""
-    The Marcum Q-function
+    The Marcum Q-function.
 
-    It is defined by the meromorphic continuation of
+    Explanation
+    ===========
+
+    The Marcum Q-function is defined by the meromorphic continuation of
 
     .. math::
         Q_m(a, b) = a^{- m + 1} \int_{b}^{\infty} x^{m} e^{- \frac{a^{2}}{2} - \frac{x^{2}}{2}} I_{m - 1}\left(a x\right)\, dx
@@ -1686,7 +1834,7 @@ class marcumq(Function):
     ========
 
     >>> from sympy import marcumq
-    >>> from sympy.abc import m, a, b, x
+    >>> from sympy.abc import m, a, b
     >>> marcumq(m, a, b)
     marcumq(m, a, b)
 
@@ -1703,7 +1851,7 @@ class marcumq(Function):
     >>> marcumq(2, a, a)
     1/2 + exp(-a**2)*besseli(0, a**2)/2 + exp(-a**2)*besseli(1, a**2)
 
-    Differentiation with respect to a and b is supported:
+    Differentiation with respect to $a$ and $b$ is supported:
 
     >>> from sympy import diff
     >>> diff(marcumq(m, a, b), a)
@@ -1716,6 +1864,7 @@ class marcumq(Function):
 
     .. [1] https://en.wikipedia.org/wiki/Marcum_Q-function
     .. [2] http://mathworld.wolfram.com/MarcumQ-Function.html
+
     """
 
     @classmethod
