@@ -197,11 +197,8 @@ class gamma(Function):
         t = self.args[0] - x0
         return (self.func(t + 1)/rf(self.args[0], -x0 + 1))._eval_nseries(x, n, logx)
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.gamma(self.args[0]._sage_())
-
-    def _eval_as_leading_term(self, x, cdir=0):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        from sympy import PoleError
         arg = self.args[0]
         x0 = arg.subs(x, 0)
 
@@ -209,18 +206,9 @@ class gamma(Function):
             n = -x0
             res = (-1)**n/self.func(n + 1)
             return res/(arg + n).as_leading_term(x)
-        elif x0.is_finite:
+        elif not x0.is_infinite:
             return self.func(x0)
-        ####################################################
-        # The correct result here should be 'None'.        #
-        # Indeed arg in not bounded as x tends to 0.       #
-        # Consequently the series expansion does not admit #
-        # the leading term.                                #
-        # For compatibility reasons, the return value here #
-        # is the original function, i.e. gamma(arg),       #
-        # instead of None.                                 #
-        ####################################################
-        return self.func(arg)
+        raise PoleError()
 
 
 ###############################################################################
@@ -563,10 +551,6 @@ class uppergamma(Function):
         from sympy import expint
         return expint(1 - s, x)*x**s
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.gamma(self.args[0]._sage_(), self.args[1]._sage_())
-
 
 ###############################################################################
 ###################### POLYGAMMA and LOGGAMMA FUNCTIONS #######################
@@ -859,7 +843,7 @@ class polygamma(Function):
             else:
                 return S.NegativeOne**(n+1) * factorial(n) * (zeta(n+1) - harmonic(z-1, n+1))
 
-    def _eval_as_leading_term(self, x, cdir=0):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
         from sympy import Order
         n, z = [a.as_leading_term(x) for a in self.args]
         o = Order(z, x)
@@ -1065,10 +1049,6 @@ class loggamma(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.log_gamma(self.args[0]._sage_())
-
 
 class digamma(Function):
     r"""
@@ -1157,7 +1137,7 @@ class digamma(Function):
     def _eval_rewrite_as_polygamma(self, z, **kwargs):
         return polygamma(0, z)
 
-    def _eval_as_leading_term(self, x, cdir=0):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
         z = self.args[0]
         return polygamma(0, z).as_leading_term(x)
 
@@ -1253,7 +1233,7 @@ class trigamma(Function):
     def _eval_rewrite_as_harmonic(self, z, **kwargs):
         return -harmonic(z - 1, 2) + S.Pi**2 / 6
 
-    def _eval_as_leading_term(self, x, cdir=0):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
         z = self.args[0]
         return polygamma(1, z).as_leading_term(x)
 
