@@ -1,10 +1,10 @@
 """ Riemann zeta and related function. """
-from __future__ import print_function, division
 
+from sympy import Add
 from sympy.core import Function, S, sympify, pi, I
-from sympy.core.compatibility import range
 from sympy.core.function import ArgumentIndexError
 from sympy.functions.combinatorial.numbers import bernoulli, factorial, harmonic
+from sympy.functions.elementary.complexes import re
 from sympy.functions.elementary.exponential import log, exp_polar
 from sympy.functions.elementary.miscellaneous import sqrt
 
@@ -17,12 +17,15 @@ class lerchphi(Function):
     r"""
     Lerch transcendent (Lerch phi function).
 
-    For :math:`\operatorname{Re}(a) > 0`, `|z| < 1` and `s \in \mathbb{C}`, the
+    Explanation
+    ===========
+
+    For $\operatorname{Re}(a) > 0$, $|z| < 1$ and $s \in \mathbb{C}$, the
     Lerch transcendent is defined as
 
     .. math :: \Phi(z, s, a) = \sum_{n=0}^\infty \frac{z^n}{(n + a)^s},
 
-    where the standard branch of the argument is used for :math:`n + a`,
+    where the standard branch of the argument is used for $n + a$,
     and by analytic continuation for other values of the parameters.
 
     A commonly used related function is the Lerch zeta function, defined by
@@ -35,27 +38,71 @@ class lerchphi(Function):
 
     .. math:: \Phi(z, s, a) = z\Phi(z, s, a+1) + a^{-s}.
 
-    This provides the analytic continuation to `\operatorname{Re}(a) \le 0`.
+    This provides the analytic continuation to $\operatorname{Re}(a) \le 0$.
 
-    Assume now `\operatorname{Re}(a) > 0`. The integral representation
+    Assume now $\operatorname{Re}(a) > 0$. The integral representation
 
     .. math:: \Phi_0(z, s, a) = \int_0^\infty \frac{t^{s-1} e^{-at}}{1 - ze^{-t}}
                                 \frac{\mathrm{d}t}{\Gamma(s)}
 
-    provides an analytic continuation to :math:`\mathbb{C} - [1, \infty)`.
-    Finally, for :math:`x \in (1, \infty)` we find
+    provides an analytic continuation to $\mathbb{C} - [1, \infty)$.
+    Finally, for $x \in (1, \infty)$ we find
 
     .. math:: \lim_{\epsilon \to 0^+} \Phi_0(x + i\epsilon, s, a)
              -\lim_{\epsilon \to 0^+} \Phi_0(x - i\epsilon, s, a)
              = \frac{2\pi i \log^{s-1}{x}}{x^a \Gamma(s)},
 
-    using the standard branch for both :math:`\log{x}` and
-    :math:`\log{\log{x}}` (a branch of :math:`\log{\log{x}}` is needed to
-    evaluate :math:`\log{x}^{s-1}`).
+    using the standard branch for both $\log{x}$ and
+    $\log{\log{x}}$ (a branch of $\log{\log{x}}$ is needed to
+    evaluate $\log{x}^{s-1}$).
     This concludes the analytic continuation. The Lerch transcendent is thus
-    branched at :math:`z \in \{0, 1, \infty\}` and
-    :math:`a \in \mathbb{Z}_{\le 0}`. For fixed :math:`z, a` outside these
-    branch points, it is an entire function of :math:`s`.
+    branched at $z \in \{0, 1, \infty\}$ and
+    $a \in \mathbb{Z}_{\le 0}$. For fixed $z, a$ outside these
+    branch points, it is an entire function of $s$.
+
+    Examples
+    ========
+
+    The Lerch transcendent is a fairly general function, for this reason it does
+    not automatically evaluate to simpler functions. Use ``expand_func()`` to
+    achieve this.
+
+    If $z=1$, the Lerch transcendent reduces to the Hurwitz zeta function:
+
+    >>> from sympy import lerchphi, expand_func
+    >>> from sympy.abc import z, s, a
+    >>> expand_func(lerchphi(1, s, a))
+    zeta(s, a)
+
+    More generally, if $z$ is a root of unity, the Lerch transcendent
+    reduces to a sum of Hurwitz zeta functions:
+
+    >>> expand_func(lerchphi(-1, s, a))
+    zeta(s, a/2)/2**s - zeta(s, a/2 + 1/2)/2**s
+
+    If $a=1$, the Lerch transcendent reduces to the polylogarithm:
+
+    >>> expand_func(lerchphi(z, s, 1))
+    polylog(s, z)/z
+
+    More generally, if $a$ is rational, the Lerch transcendent reduces
+    to a sum of polylogarithms:
+
+    >>> from sympy import S
+    >>> expand_func(lerchphi(z, s, S(1)/2))
+    2**(s - 1)*(polylog(s, sqrt(z))/sqrt(z) -
+                polylog(s, sqrt(z)*exp_polar(I*pi))/sqrt(z))
+    >>> expand_func(lerchphi(z, s, S(3)/2))
+    -2**s/z + 2**(s - 1)*(polylog(s, sqrt(z))/sqrt(z) -
+                          polylog(s, sqrt(z)*exp_polar(I*pi))/sqrt(z))/z
+
+    The derivatives with respect to $z$ and $a$ can be computed in
+    closed form:
+
+    >>> lerchphi(z, s, a).diff(z)
+    (-a*lerchphi(z, s, a) + lerchphi(z, s - 1, a))/z
+    >>> lerchphi(z, s, a).diff(a)
+    -s*lerchphi(z, s + 1, a)
 
     See Also
     ========
@@ -70,49 +117,6 @@ class lerchphi(Function):
     .. [2] http://dlmf.nist.gov/25.14
     .. [3] https://en.wikipedia.org/wiki/Lerch_transcendent
 
-    Examples
-    ========
-
-    The Lerch transcendent is a fairly general function, for this reason it does
-    not automatically evaluate to simpler functions. Use expand_func() to
-    achieve this.
-
-    If :math:`z=1`, the Lerch transcendent reduces to the Hurwitz zeta function:
-
-    >>> from sympy import lerchphi, expand_func
-    >>> from sympy.abc import z, s, a
-    >>> expand_func(lerchphi(1, s, a))
-    zeta(s, a)
-
-    More generally, if :math:`z` is a root of unity, the Lerch transcendent
-    reduces to a sum of Hurwitz zeta functions:
-
-    >>> expand_func(lerchphi(-1, s, a))
-    2**(-s)*zeta(s, a/2) - 2**(-s)*zeta(s, a/2 + 1/2)
-
-    If :math:`a=1`, the Lerch transcendent reduces to the polylogarithm:
-
-    >>> expand_func(lerchphi(z, s, 1))
-    polylog(s, z)/z
-
-    More generally, if :math:`a` is rational, the Lerch transcendent reduces
-    to a sum of polylogarithms:
-
-    >>> from sympy import S
-    >>> expand_func(lerchphi(z, s, S(1)/2))
-    2**(s - 1)*(polylog(s, sqrt(z))/sqrt(z) -
-                polylog(s, sqrt(z)*exp_polar(I*pi))/sqrt(z))
-    >>> expand_func(lerchphi(z, s, S(3)/2))
-    -2**s/z + 2**(s - 1)*(polylog(s, sqrt(z))/sqrt(z) -
-                          polylog(s, sqrt(z)*exp_polar(I*pi))/sqrt(z))/z
-
-    The derivatives with respect to :math:`z` and :math:`a` can be computed in
-    closed form:
-
-    >>> lerchphi(z, s, a).diff(z)
-    (-a*lerchphi(z, s, a) + lerchphi(z, s - 1, a))/z
-    >>> lerchphi(z, s, a).diff(a)
-    -s*lerchphi(z, s + 1, a)
     """
 
     def _eval_expand_func(self, **hints):
@@ -207,16 +211,19 @@ class polylog(Function):
     r"""
     Polylogarithm function.
 
-    For :math:`|z| < 1` and :math:`s \in \mathbb{C}`, the polylogarithm is
+    Explanation
+    ===========
+
+    For $|z| < 1$ and $s \in \mathbb{C}$, the polylogarithm is
     defined by
 
     .. math:: \operatorname{Li}_s(z) = \sum_{n=1}^\infty \frac{z^n}{n^s},
 
-    where the standard branch of the argument is used for :math:`n`. It admits
-    an analytic continuation which is branched at :math:`z=1` (notably not on the
-    sheet of initial definition), :math:`z=0` and :math:`z=\infty`.
+    where the standard branch of the argument is used for $n$. It admits
+    an analytic continuation which is branched at $z=1$ (notably not on the
+    sheet of initial definition), $z=0$ and $z=\infty$.
 
-    The name polylogarithm comes from the fact that for :math:`s=1`, the
+    The name polylogarithm comes from the fact that for $s=1$, the
     polylogarithm is related to the ordinary logarithm (see examples), and that
 
     .. math:: \operatorname{Li}_{s+1}(z) =
@@ -224,17 +231,12 @@ class polylog(Function):
 
     The polylogarithm is a special case of the Lerch transcendent:
 
-    .. math:: \operatorname{Li}_{s}(z) = z \Phi(z, s, 1)
-
-    See Also
-    ========
-
-    zeta, lerchphi
+    .. math:: \operatorname{Li}_{s}(z) = z \Phi(z, s, 1).
 
     Examples
     ========
 
-    For :math:`z \in \{0, 1, -1\}`, the polylogarithm is automatically expressed
+    For $z \in \{0, 1, -1\}$, the polylogarithm is automatically expressed
     using other functions:
 
     >>> from sympy import polylog
@@ -246,9 +248,9 @@ class polylog(Function):
     >>> polylog(s, -1)
     -dirichlet_eta(s)
 
-    If :math:`s` is a negative integer, :math:`0` or :math:`1`, the
-    polylogarithm can be expressed using elementary functions. This can be
-    done using expand_func():
+    If $s$ is a negative integer, $0$ or $1$, the polylogarithm can be
+    expressed using elementary functions. This can be done using
+    ``expand_func()``:
 
     >>> from sympy import expand_func
     >>> from sympy.abc import z
@@ -257,7 +259,7 @@ class polylog(Function):
     >>> expand_func(polylog(0, z))
     z/(1 - z)
 
-    The derivative with respect to :math:`z` can be computed in closed form:
+    The derivative with respect to $z$ can be computed in closed form:
 
     >>> polylog(s, z).diff(z)
     polylog(s - 1, z)/z
@@ -267,6 +269,12 @@ class polylog(Function):
     >>> from sympy import lerchphi
     >>> polylog(s, z).rewrite(lerchphi)
     z*lerchphi(z, s, 1)
+
+    See Also
+    ========
+
+    zeta, lerchphi
+
     """
 
     @classmethod
@@ -346,6 +354,38 @@ class polylog(Function):
         if z.is_zero:
             return True
 
+    def _eval_nseries(self, x, n, logx, cdir=0):
+        from sympy import ceiling, Order
+        nu, z = self.args
+
+        z0 = z.subs(x, 0)
+        if z0 is S.NaN:
+            z0 = z.limit(x, 0, dir='-' if re(cdir).is_negative else '+')
+
+        if z0.is_zero:
+            # In case of powers less than 1, number of terms need to be computed
+            # separately to avoid repeated callings of _eval_nseries with wrong n
+            try:
+                _, exp = z.leadterm(x)
+            except (ValueError, NotImplementedError):
+                return self
+
+            if exp.is_positive:
+                newn = ceiling(n/exp)
+                o = Order(x**n, x)
+                r = z._eval_nseries(x, n, logx, cdir).removeO()
+                if r is S.Zero:
+                    return o
+
+                term = r
+                s = [term]
+                for k in range(2, newn):
+                    term *= r
+                    s.append(term/k**nu)
+                return Add(*s) + o
+
+        return super(polylog, self)._eval_nseries(x, n, logx, cdir)
+
 ###############################################################################
 ###################### HURWITZ GENERALIZED ZETA FUNCTION ######################
 ###############################################################################
@@ -355,16 +395,20 @@ class zeta(Function):
     r"""
     Hurwitz zeta function (or Riemann zeta function).
 
-    For `\operatorname{Re}(a) > 0` and `\operatorname{Re}(s) > 1`, this function is defined as
+    Explanation
+    ===========
+
+    For $\operatorname{Re}(a) > 0$ and $\operatorname{Re}(s) > 1$, this
+    function is defined as
 
     .. math:: \zeta(s, a) = \sum_{n=0}^\infty \frac{1}{(n + a)^s},
 
-    where the standard choice of argument for :math:`n + a` is used. For fixed
-    :math:`a` with `\operatorname{Re}(a) > 0` the Hurwitz zeta function admits a
-    meromorphic continuation to all of :math:`\mathbb{C}`, it is an unbranched
-    function with a simple pole at :math:`s = 1`.
+    where the standard choice of argument for $n + a$ is used. For fixed
+    $a$ with $\operatorname{Re}(a) > 0$ the Hurwitz zeta function admits a
+    meromorphic continuation to all of $\mathbb{C}$, it is an unbranched
+    function with a simple pole at $s = 1$.
 
-    Analytic continuation to other :math:`a` is possible under some circumstances,
+    Analytic continuation to other $a$ is possible under some circumstances,
     but this is not typically done.
 
     The Hurwitz zeta function is a special case of the Lerch transcendent:
@@ -372,27 +416,16 @@ class zeta(Function):
     .. math:: \zeta(s, a) = \Phi(1, s, a).
 
     This formula defines an analytic continuation for all possible values of
-    :math:`s` and :math:`a` (also `\operatorname{Re}(a) < 0`), see the documentation of
+    $s$ and $a$ (also $\operatorname{Re}(a) < 0$), see the documentation of
     :class:`lerchphi` for a description of the branching behavior.
 
-    If no value is passed for :math:`a`, by this function assumes a default value
-    of :math:`a = 1`, yielding the Riemann zeta function.
-
-    See Also
-    ========
-
-    dirichlet_eta, lerchphi, polylog
-
-    References
-    ==========
-
-    .. [1] http://dlmf.nist.gov/25.11
-    .. [2] https://en.wikipedia.org/wiki/Hurwitz_zeta_function
+    If no value is passed for $a$, by this function assumes a default value
+    of $a = 1$, yielding the Riemann zeta function.
 
     Examples
     ========
 
-    For :math:`a = 1` the Hurwitz zeta function reduces to the famous Riemann
+    For $a = 1$ the Hurwitz zeta function reduces to the famous Riemann
     zeta function:
 
     .. math:: \zeta(s, 1) = \zeta(s) = \sum_{n=1}^\infty \frac{1}{n^s}.
@@ -437,26 +470,36 @@ class zeta(Function):
     >>> zeta(3).n()
     1.20205690315959
 
-    The derivative of :math:`\zeta(s, a)` with respect to :math:`a` is easily
-    computed:
+    The derivative of $\zeta(s, a)$ with respect to $a$ can be computed:
 
     >>> from sympy.abc import a
     >>> zeta(s, a).diff(a)
     -s*zeta(s + 1, a)
 
-    However the derivative with respect to :math:`s` has no useful closed form
+    However the derivative with respect to $s$ has no useful closed form
     expression:
 
     >>> zeta(s, a).diff(s)
     Derivative(zeta(s, a), s)
 
-    The Hurwitz zeta function can be expressed in terms of the Lerch transcendent,
-    :class:`~.lerchphi`:
+    The Hurwitz zeta function can be expressed in terms of the Lerch
+    transcendent, :class:`~.lerchphi`:
 
     >>> from sympy import lerchphi
     >>> zeta(s, a).rewrite(lerchphi)
     lerchphi(1, s, a)
 
+    See Also
+    ========
+
+    dirichlet_eta, lerchphi, polylog
+
+    References
+    ==========
+
+    .. [1] http://dlmf.nist.gov/25.11
+    .. [2] https://en.wikipedia.org/wiki/Hurwitz_zeta_function
+
     """
 
     @classmethod
@@ -528,23 +571,16 @@ class dirichlet_eta(Function):
     r"""
     Dirichlet eta function.
 
-    For `\operatorname{Re}(s) > 0`, this function is defined as
+    Explanation
+    ===========
+
+    For $\operatorname{Re}(s) > 0$, this function is defined as
 
     .. math:: \eta(s) = \sum_{n=1}^\infty \frac{(-1)^{n-1}}{n^s}.
 
-    It admits a unique analytic continuation to all of :math:`\mathbb{C}`.
+    It admits a unique analytic continuation to all of $\mathbb{C}$.
     It is an entire, unbranched function.
 
-    See Also
-    ========
-
-    zeta
-
-    References
-    ==========
-
-    .. [1] https://en.wikipedia.org/wiki/Dirichlet_eta_function
-
     Examples
     ========
 
@@ -555,6 +591,16 @@ class dirichlet_eta(Function):
     >>> dirichlet_eta(s).rewrite(zeta)
     (1 - 2**(1 - s))*zeta(s)
 
+    See Also
+    ========
+
+    zeta
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Dirichlet_eta_function
+
     """
 
     @classmethod
@@ -569,8 +615,47 @@ class dirichlet_eta(Function):
         return (1 - 2**(1 - s)) * zeta(s)
 
 
+class riemann_xi(Function):
+    r"""
+    Riemann Xi function.
+
+    Examples
+    ========
+
+    The Riemann Xi function is closely related to the Riemann zeta function.
+    The zeros of Riemann Xi function are precisely the non-trivial zeros
+    of the zeta function.
+
+    >>> from sympy import riemann_xi, zeta
+    >>> from sympy.abc import s
+    >>> riemann_xi(s).rewrite(zeta)
+    s*(s - 1)*gamma(s/2)*zeta(s)/(2*pi**(s/2))
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Riemann_Xi_function
+
+    """
+
+
+    @classmethod
+    def eval(cls, s):
+        from sympy import gamma
+        z = zeta(s)
+        if s is S.Zero or s is S.One:
+            return S.Half
+
+        if not isinstance(z, zeta):
+            return s*(s - 1)*gamma(s/2)*z/(2*pi**(s/2))
+
+    def _eval_rewrite_as_zeta(self, s, **kwargs):
+        from sympy import gamma
+        return s*(s - 1)*gamma(s/2)*zeta(s)/(2*pi**(s/2))
+
 class stieltjes(Function):
-    r"""Represents Stieltjes constants, :math:`\gamma_{k}` that occur in
+    r"""
+    Represents Stieltjes constants, $\gamma_{k}$ that occur in
     Laurent Series expansion of the Riemann zeta function.
 
     Examples
@@ -581,19 +666,19 @@ class stieltjes(Function):
     >>> stieltjes(n)
     stieltjes(n)
 
-    zero'th stieltjes constant
+    The zero'th stieltjes constant:
 
     >>> stieltjes(0)
     EulerGamma
     >>> stieltjes(0, 1)
     EulerGamma
 
-    For generalized stieltjes constants
+    For generalized stieltjes constants:
 
     >>> stieltjes(n, m)
     stieltjes(n, m)
 
-    Constants are only defined for integers >= 0
+    Constants are only defined for integers >= 0:
 
     >>> stieltjes(-1)
     zoo
@@ -602,6 +687,7 @@ class stieltjes(Function):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Stieltjes_constants
+
     """
 
     @classmethod
