@@ -3,38 +3,48 @@ This module implements Holonomic Functions and
 various operations on them.
 """
 
-from __future__ import print_function, division
-
-from sympy import (Symbol, diff, S, Dummy, Order, rf, meijerint, I,
+from sympy import (Symbol, S, Dummy, Order, rf, I,
     solve, limit, Float, nsimplify, gamma)
-from sympy.printing import sstr
-from sympy.core.compatibility import range, ordered
-from sympy.functions.combinatorial.factorials import binomial, factorial
-from sympy.core.sympify import sympify
-from sympy.simplify.hyperexpand import hyperexpand
-from sympy.functions.special.hyper import hyper, meijerg
+from sympy.core.compatibility import ordered
 from sympy.core.numbers import NaN, Infinity, NegativeInfinity
-from sympy.matrices import Matrix
+from sympy.core.sympify import sympify
+from sympy.functions.combinatorial.factorials import binomial, factorial
 from sympy.functions.elementary.exponential import exp_polar, exp
+from sympy.functions.special.hyper import hyper, meijerg
+from sympy.integrals import meijerint
+from sympy.matrices import Matrix
+from sympy.polys.rings import PolyElement
+from sympy.polys.fields import FracElement
+from sympy.polys.domains import QQ, RR
+from sympy.polys.polyclasses import DMF
+from sympy.polys.polyroots import roots
+from sympy.polys.polytools import Poly
+from sympy.polys.matrices import DomainMatrix
+from sympy.printing import sstr
+from sympy.simplify.hyperexpand import hyperexpand
 
-from .linearsolver import NewMatrix
 from .recurrence import HolonomicSequence, RecurrenceOperator, RecurrenceOperators
 from .holonomicerrors import (NotPowerSeriesError, NotHyperSeriesError,
     SingularityError, NotHolonomicError)
 
-from sympy.polys.rings import PolyElement
-from sympy.polys.fields import FracElement
-from sympy.polys.domains import QQ, ZZ, RR
-from sympy.polys.domains.pythonrational import PythonRational
-from sympy.polys.polyclasses import DMF
-from sympy.polys.polyroots import roots
-from sympy.polys.polytools import Poly
+
+def _find_nonzero_solution(r, homosys):
+    ones = lambda shape: DomainMatrix.ones(shape, r.domain)
+    particular, nullspace = r._solve(homosys)
+    nullity = nullspace.shape[0]
+    nullpart = ones((1, nullity)) * nullspace
+    sol = (particular + nullpart).transpose()
+    return sol
+
 
 
 def DifferentialOperators(base, generator):
     r"""
     This function is used to create annihilators using ``Dx``.
 
+    Explanation
+    ===========
+
     Returns an Algebra of Differential Operators also called Weyl Algebra
     and the operator for differentiation i.e. the ``Dx`` operator.
 
@@ -66,18 +76,19 @@ def DifferentialOperators(base, generator):
     return (ring, ring.derivative_operator)
 
 
-class DifferentialOperatorAlgebra(object):
+class DifferentialOperatorAlgebra:
     r"""
     An Ore Algebra is a set of noncommutative polynomials in the
     intermediate ``Dx`` and coefficients in a base polynomial ring :math:`A`.
     It follows the commutation rule:
+
     .. math ::
-        Dxa = \sigma(a)Dx + \delta(a)
+       Dxa = \sigma(a)Dx + \delta(a)
 
     for :math:`a \subset A`.
 
-    Where :math:`\sigma: A --> A` is an endomorphism and :math:`\delta: A --> A`
-    is a skew-derivation i.e. :math:`\delta(ab) = \delta(a) * b + \sigma(a) * \delta(b)`.
+    Where :math:`\sigma: A \Rightarrow A` is an endomorphism and :math:`\delta: A \rightarrow A`
+    is a skew-derivation i.e. :math:`\delta(ab) = \delta(a) b + \sigma(a) \delta(b)`.
 
     If one takes the sigma as identity map and delta as the standard derivation
     then it becomes the algebra of Differential Operators also called
@@ -135,12 +146,15 @@ class DifferentialOperatorAlgebra(object):
             return False
 
 
-class DifferentialOperator(object):
+class DifferentialOperator:
     """
     Differential Operators are elements of Weyl Algebra. The Operators
     are defined by a list of polynomials in the base ring and the
     parent ring of the Operator i.e. the algebra it belongs to.
 
+    Explanation
+    ===========
+
     Takes a list of polynomials for each power of ``Dx`` and the
     parent ring which must be an instance of DifferentialOperatorAlgebra.
 
@@ -151,7 +165,7 @@ class DifferentialOperator(object):
     ========
 
     >>> from sympy.holonomic.holonomic import DifferentialOperator, DifferentialOperators
-    >>> from sympy.polys.domains import ZZ, QQ
+    >>> from sympy.polys.domains import ZZ
     >>> from sympy import symbols
     >>> x = symbols('x')
     >>> R, Dx = DifferentialOperators(ZZ.old_poly_ring(x),'Dx')
@@ -294,11 +308,8 @@ class DifferentialOperator(object):
     def __neg__(self):
         return -1 * self
 
-    def __div__(self, other):
-        return self * (S.One / other)
-
     def __truediv__(self, other):
-        return self.__div__(other)
+        return self * (S.One / other)
 
     def __pow__(self, n):
         if n == 1:
@@ -356,7 +367,7 @@ class DifferentialOperator(object):
                 return False
         else:
             if self.listofpoly[0] == other:
-                for i in listofpoly[1:]:
+                for i in self.listofpoly[1:]:
                     if i is not self.parent.base.zero:
                         return False
                 return True
@@ -372,7 +383,7 @@ class DifferentialOperator(object):
         return x0 in roots(base.to_sympy(self.listofpoly[-1]), self.x)
 
 
-class HolonomicFunction(object):
+class HolonomicFunction:
     r"""
     A Holonomic Function is a solution to a linear homogeneous ordinary
     differential equation with polynomial coefficients. This differential
@@ -380,6 +391,9 @@ class HolonomicFunction(object):
     Operator ``L`` such that :math:`L.f = 0`. For uniqueness of these functions,
     initial conditions can also be provided along with the annihilator.
 
+    Explanation
+    ===========
+
     Holonomic functions have closure properties and thus forms a ring.
     Given two Holonomic Functions f and g, their sum, product,
     integral and derivative is also a Holonomic Function.
@@ -398,7 +412,7 @@ class HolonomicFunction(object):
     ========
 
     >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-    >>> from sympy.polys.domains import ZZ, QQ
+    >>> from sympy.polys.domains import QQ
     >>> from sympy import symbols, S
     >>> x = symbols('x')
     >>> R, Dx = DifferentialOperators(QQ.old_poly_ring(x),'Dx')
@@ -542,7 +556,7 @@ class HolonomicFunction(object):
             y0 = []
             a = int(a)
             for i in range(a):
-                y0.append(S(0))
+                y0.append(S.Zero)
             y0 += [j * factorial(a + i) for i, j in enumerate(b)]
 
             return HolonomicFunction(self.annihilator, self.x, self.x0, y0)
@@ -581,23 +595,19 @@ class HolonomicFunction(object):
             p = []
             for i in range(dim + 1):
                 if i >= len(expr.listofpoly):
-                    p.append(0)
+                    p.append(K.zero)
                 else:
                     p.append(K.new(expr.listofpoly[i].rep))
             r.append(p)
 
-        r = NewMatrix(r).transpose()
-
-        homosys = [[S(0) for q in range(dim + 1)]]
-        homosys = NewMatrix(homosys).transpose()
-
         # solving the linear system using gauss jordan solver
-        solcomp = r.gauss_jordan_solve(homosys)
-        sol = solcomp[0]
+        r = DomainMatrix(r, (len(row), dim+1), K).transpose()
+        homosys = DomainMatrix.zeros((dim+1, 1), K)
+        sol = _find_nonzero_solution(r, homosys)
 
         # if a solution is not obtained then increasing the order by 1 in each
         # iteration
-        while sol.is_zero:
+        while sol.is_zero_matrix:
             dim += 1
 
             diff1 = (gen * rowsself[-1])
@@ -613,24 +623,20 @@ class HolonomicFunction(object):
                 p = []
                 for i in range(dim + 1):
                     if i >= len(expr.listofpoly):
-                        p.append(S(0))
+                        p.append(K.zero)
                     else:
-
                         p.append(K.new(expr.listofpoly[i].rep))
                 r.append(p)
 
-            r = NewMatrix(r).transpose()
-
-            homosys = [[S(0) for q in range(dim + 1)]]
-            homosys = NewMatrix(homosys).transpose()
-
-            solcomp = r.gauss_jordan_solve(homosys)
-            sol = solcomp[0]
+            # solving the linear system using gauss jordan solver
+            r = DomainMatrix(r, (len(row), dim+1), K).transpose()
+            homosys = DomainMatrix.zeros((dim+1, 1), K)
+            sol = _find_nonzero_solution(r, homosys)
 
         # taking only the coefficients needed to multiply with `self`
         # can be also be done the other way by taking R.H.S and multiplying with
         # `other`
-        sol = sol[:dim + 1 - deg1]
+        sol = sol.flat()[:dim + 1 - deg1]
         sol1 = _normalize(sol, self.annihilator.parent)
         # annihilator of the solution
         sol = sol1 * (self.annihilator)
@@ -682,12 +688,12 @@ class HolonomicFunction(object):
         if self.is_singularics() == False and other.is_singularics() == True:
             # convert the ordinary initial condition to singular.
             _y0 = [j / factorial(i) for i, j in enumerate(self.y0)]
-            y1 = {S(0): _y0}
+            y1 = {S.Zero: _y0}
             y2 = other.y0
         elif self.is_singularics() == True and other.is_singularics() == False:
             _y0 = [j / factorial(i) for i, j in enumerate(other.y0)]
             y1 = self.y0
-            y2 = {S(0): _y0}
+            y2 = {S.Zero: _y0}
         elif self.is_singularics() == True and other.is_singularics() == True:
             y1 = self.y0
             y2 = other.y0
@@ -715,7 +721,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import QQ
         >>> from sympy import symbols
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(QQ.old_poly_ring(x),'Dx')
@@ -743,7 +749,7 @@ class HolonomicFunction(object):
                 c2 = []
                 for j in range(len(c)):
                     if c[j] == 0:
-                        c2.append(S(0))
+                        c2.append(S.Zero)
 
                     # if power on `x` is -1, the integration becomes log(x)
                     # TODO: Implement this case
@@ -761,7 +767,7 @@ class HolonomicFunction(object):
         # if no initial conditions are available for the function
         if not self._have_init_cond():
             if initcond:
-                return HolonomicFunction(self.annihilator * D, self.x, self.x0, [S(0)])
+                return HolonomicFunction(self.annihilator * D, self.x, self.x0, [S.Zero])
             return HolonomicFunction(self.annihilator * D, self.x)
 
         # definite integral
@@ -778,7 +784,7 @@ class HolonomicFunction(object):
         else:
             definite = False
 
-        y0 = [S(0)]
+        y0 = [S.Zero]
         y0 += self.y0
 
         indefinite_integral = HolonomicFunction(self.annihilator * D, self.x, self.x0, y0)
@@ -834,7 +840,7 @@ class HolonomicFunction(object):
 
         return HolonomicFunction(self.annihilator * D, self.x)
 
-    def diff(self, *args):
+    def diff(self, *args, **kwargs):
         r"""
         Differentiation of the given Holonomic function.
 
@@ -842,7 +848,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import ZZ
         >>> from sympy import symbols
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(ZZ.old_poly_ring(x),'Dx')
@@ -856,10 +862,10 @@ class HolonomicFunction(object):
 
         .integrate()
         """
-
+        kwargs.setdefault('evaluate', True)
         if args:
             if args[0] != self.x:
-                return S(0)
+                return S.Zero
             elif len(args) == 2:
                 sol = self
                 for i in range(args[1]):
@@ -867,11 +873,10 @@ class HolonomicFunction(object):
                 return sol
 
         ann = self.annihilator
-        dx = ann.parent.derivative_operator
 
         # if the function is constant.
         if ann.listofpoly[0] == ann.parent.base.zero and ann.order == 1:
-            return S(0)
+            return S.Zero
 
         # if the coefficient of y in the differential equation is zero.
         # a shifting is done to compute the answer in this case.
@@ -974,19 +979,19 @@ class HolonomicFunction(object):
         other_red = [-list_other[i] / list_other[b] for i in range(b)]
 
         # coeff_mull[i][j] is the coefficient of Dx^i(f).Dx^j(g)
-        coeff_mul = [[S(0) for i in range(b + 1)] for j in range(a + 1)]
-        coeff_mul[0][0] = S(1)
+        coeff_mul = [[K.zero for i in range(b + 1)] for j in range(a + 1)]
+        coeff_mul[0][0] = K.one
 
         # making the ansatz
-        lin_sys = [[coeff_mul[i][j] for i in range(a) for j in range(b)]]
+        lin_sys_elements = [[coeff_mul[i][j] for i in range(a) for j in range(b)]]
+        lin_sys = DomainMatrix(lin_sys_elements, (1, a*b), K).transpose()
 
-        homo_sys = [[S(0) for q in range(a * b)]]
-        homo_sys = NewMatrix(homo_sys).transpose()
+        homo_sys = DomainMatrix.zeros((a*b, 1), K)
 
-        sol = (NewMatrix(lin_sys).transpose()).gauss_jordan_solve(homo_sys)
+        sol = _find_nonzero_solution(lin_sys, homo_sys)
 
         # until a non trivial solution is found
-        while sol[0].is_zero:
+        while sol.is_zero_matrix:
 
             # updating the coefficients Dx^i(f).Dx^j(g) for next degree
             for i in range(a - 1, -1, -1):
@@ -1000,11 +1005,11 @@ class HolonomicFunction(object):
 
             # reduce the terms to lower power using annihilators of f, g
             for i in range(a + 1):
-                if not coeff_mul[i][b] == S(0):
+                if not coeff_mul[i][b].is_zero:
                     for j in range(b):
                         coeff_mul[i][j] += other_red[j] * \
                             coeff_mul[i][b]
-                    coeff_mul[i][b] = S(0)
+                    coeff_mul[i][b] = K.zero
 
             # not d2 + 1, as that is already covered in previous loop
             for j in range(b):
@@ -1012,15 +1017,14 @@ class HolonomicFunction(object):
                     for i in range(a):
                         coeff_mul[i][j] += self_red[i] * \
                             coeff_mul[a][j]
-                    coeff_mul[a][j] = S(0)
+                    coeff_mul[a][j] = K.zero
 
-            lin_sys.append([coeff_mul[i][j] for i in range(a)
-                            for j in range(b)])
+            lin_sys_elements.append([coeff_mul[i][j] for i in range(a) for j in range(b)])
+            lin_sys = DomainMatrix(lin_sys_elements, (len(lin_sys_elements), a*b), K).transpose()
 
-            sol = (NewMatrix(lin_sys).transpose()).gauss_jordan_solve(homo_sys)
+            sol = _find_nonzero_solution(lin_sys, homo_sys)
 
-
-        sol_ann = _normalize(sol[0][0:], self.annihilator.parent, negative=False)
+        sol_ann = _normalize(sol.flat(), self.annihilator.parent, negative=False)
 
         if not (self._have_init_cond() and other._have_init_cond()):
             return HolonomicFunction(sol_ann, self.x)
@@ -1084,12 +1088,12 @@ class HolonomicFunction(object):
 
         if self.is_singularics() == False and other.is_singularics() == True:
             _y0 = [j / factorial(i) for i, j in enumerate(self.y0)]
-            y1 = {S(0): _y0}
+            y1 = {S.Zero: _y0}
             y2 = other.y0
         elif self.is_singularics() == True and other.is_singularics() == False:
             _y0 = [j / factorial(i) for i, j in enumerate(other.y0)]
             y1 = self.y0
-            y2 = {S(0): _y0}
+            y2 = {S.Zero: _y0}
         elif self.is_singularics() == True and other.is_singularics() == True:
             y1 = self.y0
             y2 = other.y0
@@ -1101,7 +1105,7 @@ class HolonomicFunction(object):
                 k = min(len(y1[i]), len(y2[j]))
                 c = []
                 for a in range(k):
-                    s = S(0)
+                    s = S.Zero
                     for b in range(a + 1):
                         s += y1[i][b] * y2[j][a - b]
                     c.append(s)
@@ -1122,11 +1126,8 @@ class HolonomicFunction(object):
     def __neg__(self):
         return -1 * self
 
-    def __div__(self, other):
-        return self * (S.One / other)
-
     def __truediv__(self, other):
-        return self.__div__(other)
+        return self * (S.One / other)
 
     def __pow__(self, n):
         if self.annihilator.order <= 1:
@@ -1150,7 +1151,7 @@ class HolonomicFunction(object):
             raise NotHolonomicError("Negative Power on a Holonomic Function")
         if n == 0:
             Dx = self.annihilator.parent.derivative_operator
-            return HolonomicFunction(Dx, self.x, S(0), [S(1)])
+            return HolonomicFunction(Dx, self.x, S.Zero, [S.One])
         if n == 1:
             return self
         else:
@@ -1179,7 +1180,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import QQ
         >>> from sympy import symbols
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(QQ.old_poly_ring(x),'Dx')
@@ -1205,25 +1206,23 @@ class HolonomicFunction(object):
 
         r = listofpoly[a].subs({self.x:expr})
         subs = [-listofpoly[i].subs({self.x:expr}) / r for i in range (a)]
-        coeffs = [S(0) for i in range(a)]  # coeffs[i] == coeff of (D^i f)(a) in D^k (f(a))
-        coeffs[0] = S(1)
+        coeffs = [S.Zero for i in range(a)]  # coeffs[i] == coeff of (D^i f)(a) in D^k (f(a))
+        coeffs[0] = S.One
         system = [coeffs]
-        homogeneous = Matrix([[S(0) for i in range(a)]]).transpose()
-        sol = S(0)
-
-        while sol.is_zero:
+        homogeneous = Matrix([[S.Zero for i in range(a)]]).transpose()
+        while True:
             coeffs_next = [p.diff(self.x) for p in coeffs]
             for i in range(a - 1):
                 coeffs_next[i + 1] += (coeffs[i] * diff)
-
             for i in range(a):
                 coeffs_next[i] += (coeffs[-1] * subs[i] * diff)
             coeffs = coeffs_next
-
             # check for linear relations
             system.append(coeffs)
             sol, taus = (Matrix(system).transpose()
                 ).gauss_jordan_solve(homogeneous)
+            if sol.is_zero_matrix is not True:
+                break
 
         tau = list(taus)[0]
         sol = sol.subs(tau, 1)
@@ -1240,6 +1239,9 @@ class HolonomicFunction(object):
         of the function about :math:`x_0`, where :math:`x_0` is the point at
         which the initial condition is stored.
 
+        Explanation
+        ===========
+
         If the point :math:`x_0` is ordinary, solution of the form :math:`[(R, n_0)]`
         is returned. Where :math:`R` is the recurrence relation and :math:`n_0` is the
         smallest ``n`` for which the recurrence holds true.
@@ -1259,7 +1261,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import QQ
         >>> from sympy import symbols, S
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(QQ.old_poly_ring(x),'Dx')
@@ -1312,7 +1314,6 @@ class HolonomicFunction(object):
 
                 if (i - k, k) in dict1:
                     dict1[(i - k, k)] += (dom.to_sympy(coeff) * rf(n - k + 1, i))
-
                 else:
                     dict1[(i - k, k)] = (dom.to_sympy(coeff) * rf(n - k + 1, i))
 
@@ -1333,13 +1334,13 @@ class HolonomicFunction(object):
         # an appropriate shift of the recurrence
         for j in range(lower, upper + 1):
             if j in keylist:
-                temp = S(0)
+                temp = S.Zero
                 for k in dict1.keys():
                     if k[0] == j:
                         temp += dict1[k].subs(n, n - lower)
                 sol.append(temp)
             else:
-                sol.append(S(0))
+                sol.append(S.Zero)
 
         # the recurrence relation
         sol = RecurrenceOperator(sol, R)
@@ -1368,12 +1369,12 @@ class HolonomicFunction(object):
         if len(u0) < order:
 
             for i in range(degree):
-                eq = S(0)
+                eq = S.Zero
 
                 for j in dict1:
 
                     if i + j[0] < 0:
-                        dummys[i + j[0]] = S(0)
+                        dummys[i + j[0]] = S.Zero
 
                     elif i + j[0] < len(u0):
                         dummys[i + j[0]] = u0[i + j[0]]
@@ -1443,8 +1444,6 @@ class HolonomicFunction(object):
         compl.sort(key=lambda x : x[2])
         reals.sort()
 
-        x = self.x
-
         # grouping the roots, roots differ by an integer are put in the same group.
         grp = []
 
@@ -1524,7 +1523,6 @@ class HolonomicFunction(object):
 
                     if (i - k, k - i) in dict1:
                         dict1[(i - k, k - i)] += (dom.to_sympy(coeff) * rf(n - k + 1 + p, i))
-
                     else:
                         dict1[(i - k, k - i)] = (dom.to_sympy(coeff) * rf(n - k + 1 + p, i))
 
@@ -1542,13 +1540,13 @@ class HolonomicFunction(object):
 
             for j in range(lower, upper + 1):
                 if j in keylist:
-                    temp = S(0)
+                    temp = S.Zero
                     for k in dict1.keys():
                         if k[0] == j:
                             temp += dict1[k].subs(n, n - lower)
                     sol.append(temp)
                 else:
-                    sol.append(S(0))
+                    sol.append(S.Zero)
 
             # the recurrence relation
             sol = RecurrenceOperator(sol, R)
@@ -1578,11 +1576,11 @@ class HolonomicFunction(object):
             if len(u0) < order:
 
                 for i in range(degree2, degree):
-                    eq = S(0)
+                    eq = S.Zero
 
                     for j in dict1:
                         if i + j[0] < 0:
-                            dummys[i + j[0]] = S(0)
+                            dummys[i + j[0]] = S.Zero
 
                         elif i + j[0] < len(u0):
                             dummys[i + j[0]] = u0[i + j[0]]
@@ -1646,6 +1644,9 @@ class HolonomicFunction(object):
         r"""
         Finds the power series expansion of given holonomic function about :math:`x_0`.
 
+        Explanation
+        ===========
+
         A list of series might be returned if :math:`x_0` is a regular point with
         multiple roots of the indicial equation.
 
@@ -1653,7 +1654,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import QQ
         >>> from sympy import symbols
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(QQ.old_poly_ring(x),'Dx')
@@ -1668,7 +1669,7 @@ class HolonomicFunction(object):
         HolonomicFunction.to_sequence()
         """
 
-        if _recur == None:
+        if _recur is None:
             recurrence = self.to_sequence()
         else:
             recurrence = _recur
@@ -1713,7 +1714,7 @@ class HolonomicFunction(object):
         else:
             # use the initial conditions to find the next term
             for i in range(l + 1 - k, n - k):
-                coeff = S(0)
+                coeff = S.Zero
                 for j in range(k):
                     if i + j >= 0:
                         coeff += DMFsubs(sub[j], i) * sol[i + j]
@@ -1722,7 +1723,7 @@ class HolonomicFunction(object):
         if coefficient:
             return sol
 
-        ser = S(0)
+        ser = S.Zero
         for i, j in enumerate(sol):
             ser += x**(i + constantpower) * j
         if order:
@@ -1777,6 +1778,9 @@ class HolonomicFunction(object):
         (RK4 by default). A set of points (real or complex) must be provided
         which will be the path for the numerical integration.
 
+        Explanation
+        ===========
+
         The path should be given as a list :math:`[x_1, x_2, ... x_n]`. The numerical
         values will be computed at each point in this order
         :math:`x_1 --> x_2 --> x_3 ... --> x_n`.
@@ -1787,7 +1791,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import QQ
         >>> from sympy import symbols
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(QQ.old_poly_ring(x),'Dx')
@@ -1879,6 +1883,9 @@ class HolonomicFunction(object):
         Returns a hypergeometric function (or linear combination of them)
         representing the given holonomic function.
 
+        Explanation
+        ===========
+
         Returns an answer of the form:
         `a_1 \cdot x^{b_1} \cdot{hyper()} + a_2 \cdot x^{b_2} \cdot{hyper()} ...`
 
@@ -1889,7 +1896,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import ZZ
         >>> from sympy import symbols
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(ZZ.old_poly_ring(x),'Dx')
@@ -1906,7 +1913,7 @@ class HolonomicFunction(object):
         from_hyper, from_meijerg
         """
 
-        if _recur == None:
+        if _recur is None:
             recurrence = self.to_sequence()
         else:
             recurrence = _recur
@@ -1945,7 +1952,7 @@ class HolonomicFunction(object):
         if m == 0:
             nonzeroterms = roots(r.parent.base.to_sympy(r.listofpoly[0]), recurrence.n, filter='R')
 
-            sol = S(0)
+            sol = S.Zero
             for j, i in enumerate(nonzeroterms):
 
                 if i < 0 or int(i) != i:
@@ -2000,7 +2007,7 @@ class HolonomicFunction(object):
         arg1 = roots(r.parent.base.to_sympy(a), recurrence.n)
         arg2 = roots(r.parent.base.to_sympy(b), recurrence.n)
 
-        # iterate thorugh the initial conditions to find
+        # iterate through the initial conditions to find
         # the hypergeometric representation of the given
         # function.
         # The answer will be a linear combination
@@ -2062,7 +2069,7 @@ class HolonomicFunction(object):
         ========
 
         >>> from sympy.holonomic.holonomic import HolonomicFunction, DifferentialOperators
-        >>> from sympy.polys.domains import ZZ, QQ
+        >>> from sympy.polys.domains import ZZ
         >>> from sympy import symbols, S
         >>> x = symbols('x')
         >>> R, Dx = DifferentialOperators(ZZ.old_poly_ring(x),'Dx')
@@ -2077,13 +2084,13 @@ class HolonomicFunction(object):
 
     def change_ics(self, b, lenics=None):
         """
-        Changes the point `x0` to `b` for initial conditions.
+        Changes the point `x0` to ``b`` for initial conditions.
 
         Examples
         ========
 
         >>> from sympy.holonomic import expr_to_holonomic
-        >>> from sympy import symbols, sin, cos, exp
+        >>> from sympy import symbols, sin, exp
         >>> x = symbols('x')
 
         >>> expr_to_holonomic(sin(x)).change_ics(1)
@@ -2095,7 +2102,7 @@ class HolonomicFunction(object):
 
         symbolic = True
 
-        if lenics == None and len(self.y0) > self.annihilator.order:
+        if lenics is None and len(self.y0) > self.annihilator.order:
             lenics = len(self.y0)
         dom = self.annihilator.parent.base.domain
 
@@ -2133,7 +2140,7 @@ class HolonomicFunction(object):
 
         # convert to hypergeometric first
         rep = self.to_hyper(as_list=True)
-        sol = S(0)
+        sol = S.Zero
 
         for i in rep:
             if len(i) == 1:
@@ -2154,7 +2161,7 @@ def from_hyper(func, x0=0, evalf=False):
     Examples
     ========
 
-    >>> from sympy.holonomic.holonomic import from_hyper, DifferentialOperators
+    >>> from sympy.holonomic.holonomic import from_hyper
     >>> from sympy import symbols, hyper, S
     >>> x = symbols('x')
     >>> from_hyper(hyper([], [S(3)/2], x**2/4))
@@ -2189,7 +2196,7 @@ def from_hyper(func, x0=0, evalf=False):
             else:
                 val = simp.subs(x, x0)
             # return None if it is Infinite or NaN
-            if (val.is_finite is not None and not val.is_finite) or isinstance(val, NaN):
+            if val.is_finite is False or isinstance(val, NaN):
                 return None
             y0.append(val)
             simp = simp.diff(x)
@@ -2209,7 +2216,7 @@ def from_hyper(func, x0=0, evalf=False):
 
     if isinstance(simp, hyper):
         x0 = 1
-        # use evalf if the function can't be simpified
+        # use evalf if the function can't be simplified
         y0 = _find_conditions(simp, x, x0, sol.order, evalf)
         while not y0:
             x0 += 1
@@ -2228,7 +2235,7 @@ def from_meijerg(func, x0=0, evalf=False, initcond=True, domain=QQ):
     Examples
     ========
 
-    >>> from sympy.holonomic.holonomic import from_meijerg, DifferentialOperators
+    >>> from sympy.holonomic.holonomic import from_meijerg
     >>> from sympy import symbols, meijerg, S
     >>> x = symbols('x')
     >>> from_meijerg(meijerg(([], []), ([S(1)/2], [0]), x**2/4))
@@ -2274,7 +2281,7 @@ def from_meijerg(func, x0=0, evalf=False, initcond=True, domain=QQ):
                 val = simp.subs(x, x0).evalf()
             else:
                 val = simp.subs(x, x0)
-            if (val.is_finite is not None and not val.is_finite) or isinstance(val, NaN):
+            if val.is_finite is False or isinstance(val, NaN):
                 return None
             y0.append(val)
             simp = simp.diff(x)
@@ -2327,7 +2334,7 @@ def expr_to_holonomic(func, x=None, x0=0, y0=None, lenics=None, domain=None, ini
         Number of terms in the initial condition. By default it is
         equal to the order of the annihilator.
     domain:
-        Ground domain for the polynomials in `x` appearing as coefficients
+        Ground domain for the polynomials in ``x`` appearing as coefficients
         in the annihilator.
     initcond:
         Set it false if you don't want the initial conditions to be computed.
@@ -2346,7 +2353,7 @@ def expr_to_holonomic(func, x=None, x0=0, y0=None, lenics=None, domain=None, ini
     See Also
     ========
 
-    meijerint._rewrite1, _convert_poly_rat_alg, _create_table
+    sympy.integrals.meijerint._rewrite1, _convert_poly_rat_alg, _create_table
     """
     func = sympify(func)
     syms = func.free_symbols
@@ -2361,7 +2368,7 @@ def expr_to_holonomic(func, x=None, x0=0, y0=None, lenics=None, domain=None, ini
 
     extra_syms = list(syms)
 
-    if domain == None:
+    if domain is None:
         if func.has(Float):
             domain = RR
         else:
@@ -2454,7 +2461,7 @@ def expr_to_holonomic(func, x=None, x0=0, y0=None, lenics=None, domain=None, ini
     if sol.annihilator.is_singular(x0):
         r = sol._indicial()
         l = list(r)
-        if len(r) == 1 and r[l[0]] == S(1):
+        if len(r) == 1 and r[l[0]] == S.One:
             r = l[0]
             g = func / (x - x0)**r
             singular_ics = _find_conditions(g, x, x0, lenics)
@@ -2481,7 +2488,7 @@ def _normalize(list_of, parent, negative=True):
     denom = []
     base = parent.base
     K = base.get_field()
-    lcm_denom = base.from_sympy(S(1))
+    lcm_denom = base.from_sympy(S.One)
     list_of_coeff = []
 
     # convert polynomials to the elements of associated
@@ -2556,9 +2563,6 @@ def _hyper_to_meijerg(func):
     ap = func.ap
     bq = func.bq
 
-    p = len(ap)
-    q = len(bq)
-
     ispoly = any(i <= 0 and int(i) == i for i in ap)
     if ispoly:
         return hyperexpand(func)
@@ -2568,10 +2572,10 @@ def _hyper_to_meijerg(func):
     # parameters of the `meijerg` function.
     an = (1 - i for i in ap)
     anp = ()
-    bm = (S(0), )
+    bm = (S.Zero, )
     bmq = (1 - i for i in bq)
 
-    k = S(1)
+    k = S.One
 
     for i in bq:
         k = k * gamma(i)
@@ -2604,7 +2608,6 @@ def _extend_y0(Holonomic, n):
 
     annihilator = Holonomic.annihilator
     a = annihilator.order
-    x = Holonomic.x
 
     listofpoly = []
 
@@ -2629,11 +2632,8 @@ def _extend_y0(Holonomic, n):
             sol = 0
             for a, b in zip(y1, list_red):
                 r = DMFsubs(b, Holonomic.x0)
-                try:
-                    if not r.is_finite:
-                        return y0
-                except AttributeError:
-                    pass
+                if not getattr(r, 'is_finite', True):
+                    return y0
                 if isinstance(r, (PolyElement, FracElement)):
                     r = r.as_expr()
                 sol += a * r
@@ -2663,8 +2663,8 @@ def DMFsubs(frac, x0, mpm=False):
 
     p = frac.num
     q = frac.den
-    sol_p = S(0)
-    sol_q = S(0)
+    sol_p = S.Zero
+    sol_q = S.Zero
 
     if mpm:
         from mpmath import mp
@@ -2727,7 +2727,7 @@ def _convert_poly_rat_alg(func, x, x0=0, y0=None, lenics=None, domain=QQ, initco
         is_singular = sol.is_singular(x0)
 
         # try to compute the conditions for singular points
-        if y0 == None and x0 == 0 and is_singular:
+        if y0 is None and x0 == 0 and is_singular:
             rep = R.from_sympy(func).rep
             for i, j in enumerate(reversed(rep)):
                 if j == 0:
@@ -2742,7 +2742,6 @@ def _convert_poly_rat_alg(func, x, x0=0, y0=None, lenics=None, domain=QQ, initco
             y0 = {indicial: S(coeff)}
 
     elif israt:
-        order = 1
         p, q = func.as_numer_denom()
         # differential equation satisfied by rational
         sol = p * q * Dx + p * q.diff(x) - q * p.diff(x)
@@ -2754,8 +2753,8 @@ def _convert_poly_rat_alg(func, x, x0=0, y0=None, lenics=None, domain=QQ, initco
         is_singular = sol.is_singular(x0)
 
         # try to compute the conditions for singular points
-        if y0 == None and x0 == 0 and is_singular and \
-            (lenics == None or lenics <= 1):
+        if y0 is None and x0 == 0 and is_singular and \
+            (lenics is None or lenics <= 1):
             rep = R.from_sympy(basepoly).rep
             for i, j in enumerate(reversed(rep)):
                 if j == 0:
@@ -2779,7 +2778,7 @@ def _convert_poly_rat_alg(func, x, x0=0, y0=None, lenics=None, domain=QQ, initco
     if sol.is_singular(x0):
         r = HolonomicFunction(sol, x, x0)._indicial()
         l = list(r)
-        if len(r) == 1 and r[l[0]] == S(1):
+        if len(r) == 1 and r[l[0]] == S.One:
             r = l[0]
             g = func / (x - x0)**r
             singular_ics = _find_conditions(g, x, x0, lenics)
@@ -2806,7 +2805,7 @@ def _convert_meijerint(func, x, initcond=True, domain=QQ):
     # lists for sum of meijerg functions
     fac_list = [fac * i[0] for i in g]
     t = po.as_base_exp()
-    s = t[1] if t[0] is x else S(0)
+    s = t[1] if t[0] == x else S.Zero
     po_list = [s + i[1] for i in g]
     G_list = [i[2] for i in g]
 
@@ -2821,7 +2820,7 @@ def _convert_meijerint(func, x, initcond=True, domain=QQ):
         a = d[b]
 
         t = b.as_base_exp()
-        b = t[1] if t[0] is x else S(0)
+        b = t[1] if t[0] == x else S.Zero
         r = s / b
         an = (i + r for i in func.args[0][0])
         ap = (i + r for i in func.args[0][1])
@@ -2887,7 +2886,7 @@ def _find_conditions(func, x, x0, order):
         val = func.subs(x, x0)
         if isinstance(val, NaN):
             val = limit(func, x, x0)
-        if (val.is_finite is not None and not val.is_finite) or isinstance(val, NaN):
+        if val.is_finite is False or isinstance(val, NaN):
             return None
         y0.append(val)
         func = func.diff(x)
