@@ -9,10 +9,16 @@ def ratint(f, x, **flags):
     """
     Performs indefinite integration of rational functions.
 
+    Explanation
+    ===========
+
     Given a field :math:`K` and a rational function :math:`f = p/q`,
     where :math:`p` and :math:`q` are polynomials in :math:`K[x]`,
     returns a function :math:`g` such that :math:`f = g'`.
 
+    Examples
+    ========
+
     >>> from sympy.integrals.rationaltools import ratint
     >>> from sympy.abc import x
 
@@ -22,7 +28,7 @@ def ratint(f, x, **flags):
     References
     ==========
 
-    .. [Bro05] M. Bronstein, Symbolic Integration I: Transcendental
+    .. [1] M. Bronstein, Symbolic Integration I: Transcendental
        Functions, Second Edition, Springer-Verlag, 2005, pp. 35-70
 
     See Also
@@ -113,6 +119,9 @@ def ratint_ratpart(f, g, x):
     """
     Horowitz-Ostrogradsky algorithm.
 
+    Explanation
+    ===========
+
     Given a field K and polynomials f and g in K[x], such that f and g
     are coprime and deg(f) < deg(g), returns fractions A and B in K(x),
     such that f/g = A' + B and B has square-free denominator.
@@ -173,6 +182,9 @@ def ratint_logpart(f, g, x, t=None):
     r"""
     Lazard-Rioboo-Trager algorithm.
 
+    Explanation
+    ===========
+
     Given a field K and polynomials f and g in K[x], such that f and g
     are coprime, deg(f) < deg(g) and g is square-free, returns a list
     of tuples (s_i, q_i) of polynomials, for i = 1..n, such that s_i
@@ -261,6 +273,9 @@ def log_to_atan(f, g):
     """
     Convert complex logarithms to real arctangents.
 
+    Explanation
+    ===========
+
     Given a real field K and polynomials f and g in K[x], with g != 0,
     returns a sum h of arctangents of polynomials in K[x], such that:
 
@@ -307,6 +322,9 @@ def log_to_real(h, q, x, t):
     r"""
     Convert complex logarithms to real functions.
 
+    Explanation
+    ===========
+
     Given real field K and polynomials h in K[t,x] and q in K[t],
     returns real function f such that:
                           ___
