@@ -69,7 +69,7 @@ def test_sparse_matrix():
         (2, 0),
         (5, 0)
     ))
-    assert a._smat == {(0, 0): 2, (1, 0): 5}
+    assert a.todok() == {(0, 0): 2, (1, 0): 5}
 
     # test_multiplication
     a = SparseMatrix((
@@ -157,6 +157,11 @@ def test_sparse_matrix():
         [0, 1, 0],
         [1, 0, 0],
         [2, 0, 1]])
+    S.row_swap(0, 1)
+    assert S == SparseMatrix([
+        [1, 0, 0],
+        [0, 1, 0],
+        [2, 0, 1]])
 
     a = SparseMatrix(1, 2, [1, 2])
     b = a.copy()
@@ -278,7 +283,7 @@ def test_sparse_matrix():
                               (0, 0, 0, 0),
                               (0, 0, 1, 0),
                               (0, 0, 0, 1)])
-    assert len(m._smat) == 2
+    assert len(m.todok()) == 2
     m[:2, :2] = sparse_eye(2)
     assert m == sparse_eye(4)
     m[:, 0] = SparseMatrix(4, 1, (1, 2, 3, 4))
@@ -408,9 +413,42 @@ def test_sparse_matrix():
         [0,     1]
     ])
 
+    # row insert
+    assert a.row_insert(2, sparse_eye(2)) == SparseMatrix([
+        [1, 2 + I],
+        [3,     4],
+        [1,     0],
+        [0,     1]
+    ])
+
+    # col insert
+    assert a.col_insert(2, SparseMatrix.zeros(2, 1)) == SparseMatrix([
+        [1, 2 + I, 0],
+        [3,     4, 0],
+    ])
+
     # symmetric
     assert not a.is_symmetric(simplify=False)
 
+    # col op
+    M = SparseMatrix.eye(3)*2
+    M[1, 0] = -1
+    M.col_op(1, lambda v, i: v + 2*M[i, 0])
+    assert M == SparseMatrix([
+        [ 2, 4, 0],
+        [-1, 0, 0],
+        [ 0, 0, 2]
+    ])
+
+    # fill
+    M = SparseMatrix.eye(3)
+    M.fill(2)
+    assert M == SparseMatrix([
+        [2, 2, 2],
+        [2, 2, 2],
+        [2, 2, 2],
+    ])
+
     # test_cofactor
     assert sparse_eye(3) == sparse_eye(3).cofactor_matrix()
     test = SparseMatrix([[1, 3, 2], [2, 6, 3], [2, 3, 6]])
@@ -506,6 +544,10 @@ def test_sparse_matrix():
     assert SparseMatrix.eye(2).nnz() == 2
 
 
+def test_scalar_multiply():
+    assert SparseMatrix([[1, 2]]).scalar_multiply(3) == SparseMatrix([[3, 6]])
+
+
 def test_transpose():
     assert SparseMatrix(((1, 2), (3, 4))).transpose() == \
         SparseMatrix(((1, 3), (2, 4)))
@@ -528,7 +570,7 @@ def test_add():
         SparseMatrix(((1, 1), (1, 1)))
     a = SparseMatrix(100, 100, lambda i, j: int(j != 0 and i % j == 0))
     b = SparseMatrix(100, 100, lambda i, j: int(i != 0 and j % i == 0))
-    assert (len(a._smat) + len(b._smat) - len((a + b)._smat) > 0)
+    assert (len(a.todok()) + len(b.todok()) - len((a + b).todok()) > 0)
 
 
 def test_errors():
@@ -555,9 +597,9 @@ def test_len():
 
 def test_sparse_zeros_sparse_eye():
     assert SparseMatrix.eye(3) == eye(3, cls=SparseMatrix)
-    assert len(SparseMatrix.eye(3)._smat) == 3
+    assert len(SparseMatrix.eye(3).todok()) == 3
     assert SparseMatrix.zeros(3) == zeros(3, cls=SparseMatrix)
-    assert len(SparseMatrix.zeros(3)._smat) == 0
+    assert len(SparseMatrix.zeros(3).todok()) == 0
 
 
 def test_copyin():
