@@ -105,7 +105,7 @@ class Sieve:
         newsieve = _arange(begin, n + 1)
 
         # Now eliminate all multiples of primes in [2, sqrt(n)]
-        for p in self.primerange(2, maxbase):
+        for p in self.primerange(maxbase):
             # Start counting at a multiple of p, offsetting
             # the index to account for the new sieve's base index
             startindex = (-begin) % p
@@ -142,22 +142,40 @@ class Sieve:
         while len(self._list) < i:
             self.extend(int(self._list[-1] * 1.5))
 
-    def primerange(self, a, b):
-        """Generate all prime numbers in the range [a, b).
+    def primerange(self, a, b=None):
+        """Generate all prime numbers in the range [2, a) or [a, b).
 
         Examples
         ========
 
-        >>> from sympy import sieve
-        >>> print([i for i in sieve.primerange(7, 18)])
+        >>> from sympy import sieve, prime
+
+        All primes less than 19:
+
+        >>> print([i for i in sieve.primerange(19)])
+        [2, 3, 5, 7, 11, 13, 17]
+
+        All primes greater than or equal to 7 and less than 19:
+
+        >>> print([i for i in sieve.primerange(7, 19)])
         [7, 11, 13, 17]
+
+        All primes through the 10th prime
+
+        >>> list(sieve.primerange(prime(10) + 1))
+        [2, 3, 5, 7, 11, 13, 17, 19, 23, 29]
+
         """
         from sympy.functions.elementary.integers import ceiling
 
         # wrapping ceiling in as_int will raise an error if there was a problem
         # determining whether the expression was exactly an integer or not
-        a = max(2, as_int(ceiling(a)))
-        b = as_int(ceiling(b))
+        if b is None:
+            b = as_int(ceiling(a))
+            a = 2
+        else:
+            a = max(2, as_int(ceiling(a)))
+            b = as_int(ceiling(b))
         if a >= b:
             return
         self.extend(b)
@@ -462,10 +480,21 @@ class primepi(Function):
         Examples
         ========
 
-        >>> from sympy import primepi
+        >>> from sympy import primepi, prime, prevprime, isprime
         >>> primepi(25)
         9
 
+        So there are 9 primes less than or equal to 25. Is 25 prime?
+
+        >>> isprime(25)
+        False
+
+        It isn't. So the first prime less than 25 must be the
+        9th prime:
+
+        >>> prevprime(25) == prime(9)
+        True
+
         See Also
         ========
 
@@ -639,8 +668,9 @@ def prevprime(n):
         n -= 4
 
 
-def primerange(a, b):
-    """ Generate a list of all prime numbers in the range [a, b).
+def primerange(a, b=None):
+    """ Generate a list of all prime numbers in the range [2, a),
+        or [a, b).
 
         If the range exists in the default sieve, the values will
         be returned from there; otherwise values will be returned
@@ -649,14 +679,28 @@ def primerange(a, b):
         Examples
         ========
 
-        >>> from sympy import primerange, sieve
-        >>> print([i for i in primerange(1, 30)])
+        >>> from sympy import primerange, prime
+
+        All primes less than 19:
+
+        >>> list(primerange(19))
+        [2, 3, 5, 7, 11, 13, 17]
+
+        All primes greater than or equal to 7 and less than 19:
+
+        >>> list(primerange(7, 19))
+        [7, 11, 13, 17]
+
+        All primes through the 10th prime
+
+        >>> list(primerange(prime(10) + 1))
         [2, 3, 5, 7, 11, 13, 17, 19, 23, 29]
 
         The Sieve method, primerange, is generally faster but it will
         occupy more memory as the sieve stores values. The default
         instance of Sieve, named sieve, can be used:
 
+        >>> from sympy import sieve
         >>> list(sieve.primerange(1, 30))
         [2, 3, 5, 7, 11, 13, 17, 19, 23, 29]
 
@@ -684,6 +728,7 @@ def primerange(a, b):
         See Also
         ========
 
+        prime : Return the nth prime
         nextprime : Return the ith prime greater than n
         prevprime : Return the largest prime smaller than n
         randprime : Returns a random prime in a given range
@@ -700,6 +745,8 @@ def primerange(a, b):
     """
     from sympy.functions.elementary.integers import ceiling
 
+    if b is None:
+        a, b = 2, a
     if a >= b:
         return
     # if we already have the range, return it
@@ -766,7 +813,7 @@ def primorial(n, nth=True):
     Examples
     ========
 
-    >>> from sympy.ntheory.generate import primorial, randprime, primerange
+    >>> from sympy.ntheory.generate import primorial, primerange
     >>> from sympy import factorint, Mul, primefactors, sqrt
     >>> primorial(4) # the first 4 primes are 2, 3, 5, 7
     210
