@@ -1,25 +1,25 @@
-from __future__ import print_function, division
-
 from sympy.core.backend import zeros, Matrix, diff, eye
 from sympy import solve_linear_system_LU
-from sympy.core.compatibility import range
 from sympy.utilities import default_sort_key
 from sympy.physics.vector import (ReferenceFrame, dynamicsymbols,
                                   partial_velocity)
+from sympy.physics.mechanics.method import _Methods
 from sympy.physics.mechanics.particle import Particle
 from sympy.physics.mechanics.rigidbody import RigidBody
 from sympy.physics.mechanics.functions import (msubs, find_dynamicsymbols,
                                                _f_list_parser)
 from sympy.physics.mechanics.linearize import Linearizer
-from sympy.utilities.exceptions import SymPyDeprecationWarning
 from sympy.utilities.iterables import iterable
 
 __all__ = ['KanesMethod']
 
 
-class KanesMethod(object):
+class KanesMethod(_Methods):
     """Kane's method object.
 
+    Explanation
+    ===========
+
     This object is used to do the "book-keeping" as you go through and form
     equations of motion in the way Kane presents in:
     Kane, T., Levinson, D. Dynamics Theory and Applications. 1985 McGraw-Hill
@@ -31,9 +31,9 @@ class KanesMethod(object):
 
     q, u : Matrix
         Matrices of the generalized coordinates and speeds
-    bodylist : iterable
+    bodies : iterable
         Iterable of Point and RigidBody objects in the system.
-    forcelist : iterable
+    loads : iterable
         Iterable of (Point, vector) or (ReferenceFrame, vector) tuples
         describing the forces on the system.
     auxiliary : Matrix
@@ -118,7 +118,7 @@ class KanesMethod(object):
     def __init__(self, frame, q_ind, u_ind, kd_eqs=None, q_dependent=None,
             configuration_constraints=None, u_dependent=None,
             velocity_constraints=None, acceleration_constraints=None,
-            u_auxiliary=None):
+            u_auxiliary=None, bodies=None, forcelist=None):
 
         """Please read the online documentation. """
         if not q_ind:
@@ -132,8 +132,8 @@ class KanesMethod(object):
         self._fr = None
         self._frstar = None
 
-        self._forcelist = None
-        self._bodylist = None
+        self._forcelist = forcelist
+        self._bodylist = bodies
 
         self._initialize_vectors(q_ind, q_dependent, u_ind, u_dependent,
                 u_auxiliary)
@@ -195,8 +195,8 @@ class KanesMethod(object):
             raise ValueError('There must be an equal number of dependent '
                              'speeds and acceleration constraints.')
         if vel:
-            u_zero = dict((i, 0) for i in self.u)
-            udot_zero = dict((i, 0) for i in self._udot)
+            u_zero = {i: 0 for i in self.u}
+            udot_zero = {i: 0 for i in self._udot}
 
             # When calling kanes_equations, another class instance will be
             # created if auxiliary u's are present. In this case, the
@@ -210,8 +210,11 @@ class KanesMethod(object):
             self._k_nh = (vel - self._f_nh).jacobian(self.u)
             # If no acceleration constraints given, calculate them.
             if not acc:
-                self._f_dnh = (self._k_nh.diff(dynamicsymbols._t) * self.u +
-                               self._f_nh.diff(dynamicsymbols._t))
+                _f_dnh = (self._k_nh.diff(dynamicsymbols._t) * self.u +
+                          self._f_nh.diff(dynamicsymbols._t))
+                if self._qdot_u_map is not None:
+                    _f_dnh = msubs(_f_dnh, self._qdot_u_map)
+                self._f_dnh = _f_dnh
                 self._k_dnh = self._k_nh
             else:
                 if self._qdot_u_map is not None:
@@ -245,9 +248,9 @@ class KanesMethod(object):
             u = self.u
             qdot = self._qdot
             # Dictionaries setting things to zero
-            u_zero = dict((i, 0) for i in u)
-            uaux_zero = dict((i, 0) for i in self._uaux)
-            qdot_zero = dict((i, 0) for i in qdot)
+            u_zero = {i: 0 for i in u}
+            uaux_zero = {i: 0 for i in self._uaux}
+            qdot_zero = {i: 0 for i in qdot}
 
             f_k = msubs(kdeqs, u_zero, qdot_zero)
             k_ku = (msubs(kdeqs, qdot_zero) - f_k).jacobian(u)
@@ -310,13 +313,13 @@ class KanesMethod(object):
         t = dynamicsymbols._t
         N = self._inertial
         # Dicts setting things to zero
-        udot_zero = dict((i, 0) for i in self._udot)
-        uaux_zero = dict((i, 0) for i in self._uaux)
+        udot_zero = {i: 0 for i in self._udot}
+        uaux_zero = {i: 0 for i in self._uaux}
         uauxdot = [diff(i, t) for i in self._uaux]
-        uauxdot_zero = dict((i, 0) for i in uauxdot)
+        uauxdot_zero = {i: 0 for i in uauxdot}
         # Dictionary of q' and q'' to u and u'
-        q_ddot_u_map = dict((k.diff(t), v.diff(t)) for (k, v) in
-                self._qdot_u_map.items())
+        q_ddot_u_map = {k.diff(t): v.diff(t) for (k, v) in
+                self._qdot_u_map.items()}
         q_ddot_u_map.update(self._qdot_u_map)
 
         # Fill up the list of partials: format is a list with num elements
@@ -419,10 +422,10 @@ class KanesMethod(object):
         else:
             f_a = Matrix()
         # Dicts to sub to zero, for splitting up expressions
-        u_zero = dict((i, 0) for i in self.u)
-        ud_zero = dict((i, 0) for i in self._udot)
-        qd_zero = dict((i, 0) for i in self._qdot)
-        qd_u_zero = dict((i, 0) for i in Matrix([self._qdot, self.u]))
+        u_zero = {i: 0 for i in self.u}
+        ud_zero = {i: 0 for i in self._udot}
+        qd_zero = {i: 0 for i in self._qdot}
+        qd_u_zero = {i: 0 for i in Matrix([self._qdot, self.u])}
         # Break the kinematic differential eqs apart into f_0 and f_1
         f_0 = msubs(self._f_k, u_zero) + self._k_kqdot*Matrix(self._qdot)
         f_1 = msubs(self._f_k, qd_zero) + self._k_ku*Matrix(self.u)
@@ -448,7 +451,7 @@ class KanesMethod(object):
         # Form dictionary to set auxiliary speeds & their derivatives to 0.
         uaux = self._uaux
         uauxdot = uaux.diff(dynamicsymbols._t)
-        uaux_zero = dict((i, 0) for i in Matrix([uaux, uauxdot]))
+        uaux_zero = {i: 0 for i in Matrix([uaux, uauxdot])}
 
         # Checking for dynamic symbols outside the dynamic differential
         # equations; throws error if there is.
@@ -471,9 +474,13 @@ class KanesMethod(object):
         return Linearizer(f_0, f_1, f_2, f_3, f_4, f_c, f_v, f_a, q, u, q_i,
                 q_d, u_i, u_d, r)
 
-    def linearize(self, **kwargs):
+    # TODO : Remove `new_method` after 1.1 has been released.
+    def linearize(self, *, new_method=None, **kwargs):
         """ Linearize the equations of motion about a symbolic operating point.
 
+        Explanation
+        ===========
+
         If kwarg A_and_B is False (default), returns M, A, B, r for the
         linearized form, M*[q', u']^T = A*[q_ind, u_ind]^T + B*r.
 
@@ -495,17 +502,16 @@ class KanesMethod(object):
         you can specify beforehand, the faster this computation will run.
 
         For more documentation, please see the ``Linearizer`` class."""
-
-        # TODO : Remove this after 1.1 has been released.
-        _ = kwargs.pop('new_method', None)
-
         linearizer = self.to_linearizer()
         result = linearizer.linearize(**kwargs)
         return result + (linearizer.r,)
 
-    def kanes_equations(self, bodies, loads=None):
+    def kanes_equations(self, bodies=None, loads=None):
         """ Method to form Kane's equations, Fr + Fr* = 0.
 
+        Explanation
+        ===========
+
         Returns (Fr, Fr*). In the case where auxiliary generalized speeds are
         present (say, s auxiliary speeds, o generalized speeds, and m motion
         constraints) the length of the returned vectors will be o - m + s in
@@ -525,17 +531,12 @@ class KanesMethod(object):
             Must be either a non-empty iterable of tuples or None which corresponds
             to a system with no constraints.
         """
-        if (bodies is None and loads is not None) or isinstance(bodies[0], tuple):
-            # This switches the order if they use the old way.
-            bodies, loads = loads, bodies
-            SymPyDeprecationWarning(value='The API for kanes_equations() has changed such '
-                    'that the loads (forces and torques) are now the second argument '
-                    'and is optional with None being the default.',
-                    feature='The kanes_equation() argument order',
-                    useinstead='switched argument order to update your code, For example: '
-                    'kanes_equations(loads, bodies) > kanes_equations(bodies, loads).',
-                    issue=10945, deprecated_since_version="1.1").warn()
-
+        if bodies is None:
+            bodies = self.bodies
+        if  loads is None and self._forcelist is not None:
+            loads = self._forcelist
+        if loads == []:
+            loads = None
         if not self._k_kqdot:
             raise AttributeError('Create an instance of KanesMethod with '
                     'kinematic differential equations to use this method.')
@@ -559,6 +560,10 @@ class KanesMethod(object):
             self._frstar = frstar.col_join(frstaraux)
         return (self._fr, self._frstar)
 
+    def _form_eoms(self):
+        fr, frstar = self.kanes_equations(self.bodylist, self.forcelist)
+        return fr + frstar
+
     def rhs(self, inv_method=None):
         """Returns the system's equations of motion in first order form. The
         output is the right hand side of::
@@ -571,6 +576,7 @@ class KanesMethod(object):
 
         Parameters
         ==========
+
         inv_method : str
             The specific sympy inverse matrix calculation method to use. For a
             list of valid methods, see
@@ -656,3 +662,11 @@ class KanesMethod(object):
     @property
     def forcelist(self):
         return self._forcelist
+
+    @property
+    def bodies(self):
+        return self._bodylist
+
+    @property
+    def loads(self):
+        return self._forcelist
