@@ -6,13 +6,12 @@ Todo:
 * Update tests.
 """
 
-from __future__ import print_function, division
 
 import math
 
 from sympy import Integer, log, Mul, Add, Pow, conjugate
 from sympy.core.basic import sympify
-from sympy.core.compatibility import string_types, range, SYMPY_INTS
+from sympy.core.compatibility import SYMPY_INTS
 from sympy.matrices import Matrix, zeros
 from sympy.printing.pretty.stringpict import prettyForm
 
@@ -60,7 +59,7 @@ class QubitState(State):
             return args[0].qubit_values
 
         # Turn strings into tuple of strings
-        if len(args) == 1 and isinstance(args[0], string_types):
+        if len(args) == 1 and isinstance(args[0], str):
             args = tuple(args[0])
 
         args = sympify(args)
@@ -191,7 +190,7 @@ class Qubit(QubitState, Ket):
     def _represent_ZGate(self, basis, **options):
         """Represent this qubits in the computational basis (ZGate).
         """
-        format = options.get('format', 'sympy')
+        _format = options.get('format', 'sympy')
         n = 1
         definite_state = 0
         for it in reversed(self.qubit_values):
@@ -199,12 +198,12 @@ class Qubit(QubitState, Ket):
             n = n*2
         result = [0]*(2**self.dimension)
         result[int(definite_state)] = 1
-        if format == 'sympy':
+        if _format == 'sympy':
             return Matrix(result)
-        elif format == 'numpy':
+        elif _format == 'numpy':
             import numpy as np
             return np.matrix(result, dtype='complex').transpose()
-        elif format == 'scipy.sparse':
+        elif _format == 'scipy.sparse':
             from scipy import sparse
             return sparse.csr_matrix(result, dtype='complex').transpose()
 
@@ -287,7 +286,7 @@ class IntQubitState(QubitState):
         if len(args) == 1 and isinstance(args[0], QubitState):
             return QubitState._eval_args(args)
         # otherwise, args should be integer
-        elif not all((isinstance(a, (int, Integer)) for a in args)):
+        elif not all(isinstance(a, (int, Integer)) for a in args):
             raise ValueError('values must be integers, got (%s)' % (tuple(type(a) for a in args),))
         # use nqubits if specified
         if nqubits is not None:
@@ -446,7 +445,6 @@ def matrix_to_qubit(matrix):
     Represent a state and then go back to its qubit form:
 
         >>> from sympy.physics.quantum.qubit import matrix_to_qubit, Qubit
-        >>> from sympy.physics.quantum.gate import Z
         >>> from sympy.physics.quantum.represent import represent
         >>> q = Qubit('01')
         >>> matrix_to_qubit(represent(q))
@@ -556,7 +554,7 @@ def measure_all(qubit, format='sympy', normalize=True):
     ========
 
         >>> from sympy.physics.quantum.qubit import Qubit, measure_all
-        >>> from sympy.physics.quantum.gate import H, X, Y, Z
+        >>> from sympy.physics.quantum.gate import H
         >>> from sympy.physics.quantum.qapply import qapply
 
         >>> c = H(0)*H(1)*Qubit('00')
@@ -614,7 +612,7 @@ def measure_partial(qubit, bits, format='sympy', normalize=True):
     ========
 
         >>> from sympy.physics.quantum.qubit import Qubit, measure_partial
-        >>> from sympy.physics.quantum.gate import H, X, Y, Z
+        >>> from sympy.physics.quantum.gate import H
         >>> from sympy.physics.quantum.qapply import qapply
 
         >>> c = H(0)*H(1)*Qubit('00')
