@@ -1,9 +1,6 @@
 """Simple Harmonic Oscillator 1-Dimension"""
 
-from __future__ import print_function, division
-
 from sympy import sqrt, I, Symbol, Integer, S
-from sympy.core.compatibility import range
 from sympy.physics.quantum.constants import hbar
 from sympy.physics.quantum.operator import Operator
 from sympy.physics.quantum.state import Bra, Ket, State
@@ -103,7 +100,7 @@ class RaisingOp(SHOOp):
 
     """
 
-    def _eval_rewrite_as_xp(self, *args):
+    def _eval_rewrite_as_xp(self, *args, **kwargs):
         return (Integer(1)/sqrt(Integer(2)*hbar*m*omega))*(
             Integer(-1)*I*Px + m*omega*X)
 
@@ -134,7 +131,6 @@ class RaisingOp(SHOOp):
     def _represent_NumberOp(self, basis, **options):
         ndim_info = options.get('ndim', 4)
         format = options.get('format','sympy')
-        spmatrix = options.get('spmatrix', 'csr')
         matrix = matrix_zeros(ndim_info, ndim_info, **options)
         for i in range(ndim_info - 1):
             value = sqrt(i + 1)
@@ -156,7 +152,7 @@ class RaisingOp(SHOOp):
     def _print_contents_pretty(self, printer, *args):
         from sympy.printing.pretty.stringpict import prettyForm
         pform = printer._print(self.args[0], *args)
-        pform = pform**prettyForm(u'\N{DAGGER}')
+        pform = pform**prettyForm('\N{DAGGER}')
         return pform
 
     def _print_contents_latex(self, printer, *args):
@@ -242,7 +238,7 @@ class LoweringOp(SHOOp):
 
     """
 
-    def _eval_rewrite_as_xp(self, *args):
+    def _eval_rewrite_as_xp(self, *args, **kwargs):
         return (Integer(1)/sqrt(Integer(2)*hbar*m*omega))*(
             I*Px + m*omega*X)
 
@@ -276,7 +272,6 @@ class LoweringOp(SHOOp):
     def _represent_NumberOp(self, basis, **options):
         ndim_info = options.get('ndim', 4)
         format = options.get('format', 'sympy')
-        spmatrix = options.get('spmatrix', 'csr')
         matrix = matrix_zeros(ndim_info, ndim_info, **options)
         for i in range(ndim_info - 1):
             value = sqrt(i + 1)
@@ -361,14 +356,14 @@ class NumberOp(SHOOp):
 
     """
 
-    def _eval_rewrite_as_a(self, *args):
+    def _eval_rewrite_as_a(self, *args, **kwargs):
         return ad*a
 
-    def _eval_rewrite_as_xp(self, *args):
+    def _eval_rewrite_as_xp(self, *args, **kwargs):
         return (Integer(1)/(Integer(2)*m*hbar*omega))*(Px**2 + (
             m*omega*X)**2) - Integer(1)/Integer(2)
 
-    def _eval_rewrite_as_H(self, *args):
+    def _eval_rewrite_as_H(self, *args, **kwargs):
         return H/(hbar*omega) - Integer(1)/Integer(2)
 
     def _apply_operator_SHOKet(self, ket):
@@ -397,7 +392,6 @@ class NumberOp(SHOOp):
     def _represent_NumberOp(self, basis, **options):
         ndim_info = options.get('ndim', 4)
         format = options.get('format', 'sympy')
-        spmatrix = options.get('spmatrix', 'csr')
         matrix = matrix_zeros(ndim_info, ndim_info, **options)
         for i in range(ndim_info):
             value = i
@@ -475,13 +469,13 @@ class Hamiltonian(SHOOp):
 
     """
 
-    def _eval_rewrite_as_a(self, *args):
+    def _eval_rewrite_as_a(self, *args, **kwargs):
         return hbar*omega*(ad*a + Integer(1)/Integer(2))
 
-    def _eval_rewrite_as_xp(self, *args):
+    def _eval_rewrite_as_xp(self, *args, **kwargs):
         return (Integer(1)/(Integer(2)*m))*(Px**2 + (m*omega*X)**2)
 
-    def _eval_rewrite_as_N(self, *args):
+    def _eval_rewrite_as_N(self, *args, **kwargs):
         return hbar*omega*(N + Integer(1)/Integer(2))
 
     def _apply_operator_SHOKet(self, ket):
@@ -504,7 +498,6 @@ class Hamiltonian(SHOOp):
     def _represent_NumberOp(self, basis, **options):
         ndim_info = options.get('ndim', 4)
         format = options.get('format', 'sympy')
-        spmatrix = options.get('spmatrix', 'csr')
         matrix = matrix_zeros(ndim_info, ndim_info, **options)
         for i in range(ndim_info):
             value = i + Integer(1)/Integer(2)
@@ -599,7 +592,6 @@ class SHOKet(SHOState, Ket):
         if isinstance(self.n, Integer):
             if self.n >= ndim_info:
                 return ValueError("N-Dimension too small")
-            value = Integer(1)
             if format == 'scipy.sparse':
                 vector[int(self.n), 0] = 1.0
                 vector = vector.tocsr()
