@@ -1,11 +1,13 @@
-from __future__ import print_function, division
+from typing import Any, Dict
 
-from sympy.core.compatibility import range, is_sequence
+from sympy.core.compatibility import is_sequence
 from sympy.external import import_module
 from sympy.printing.printer import Printer
 import sympy
 from functools import partial
 
+from sympy.utilities.decorator import doctest_depends_on
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 theano = import_module('theano')
 
@@ -54,7 +56,7 @@ if theano:
             sympy.Max: tt.maximum,  # Sympy accept >2 inputs, Theano only 2
             sympy.Min: tt.minimum,  # Sympy accept >2 inputs, Theano only 2
             sympy.conjugate: tt.conj,
-            sympy.numbers.ImaginaryUnit: lambda:tt.complex(0,1),
+            sympy.core.numbers.ImaginaryUnit: lambda:tt.complex(0,1),
             # Matrices
             sympy.MatAdd: tt.Elemwise(ts.add),
             sympy.HadamardProduct: tt.Elemwise(ts.mul),
@@ -96,7 +98,7 @@ class TheanoPrinter(Printer):
 
     def __init__(self, *args, **kwargs):
         self.cache = kwargs.pop('cache', dict())
-        super(TheanoPrinter, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _get_key(self, s, name=None, dtype=None, broadcastable=None):
         """ Get the cache key for a Sympy object.
@@ -300,7 +302,7 @@ class TheanoPrinter(Printer):
         return self._print(expr, dtypes=dtypes, broadcastables=broadcastables)
 
 
-global_cache = {}
+global_cache = {}  # type: Dict[Any, Any]
 
 
 def theano_code(expr, cache=None, **kwargs):
@@ -331,6 +333,12 @@ def theano_code(expr, cache=None, **kwargs):
         expression graph.
 
     """
+    SymPyDeprecationWarning(
+        feature="sympy.printing.theanocode",
+        useinstead="Theano is deprecated; use Aesara and sympy.printing.aesaracode",
+        issue=21150,
+        deprecated_since_version="1.8").warn()
+
     if not theano:
         raise ImportError("theano is required for theano_code")
 
@@ -387,7 +395,9 @@ def dim_handling(inputs, dim=None, dims=None, broadcastables=None):
     return {}
 
 
-def theano_function(inputs, outputs, scalar=False, **kwargs):
+@doctest_depends_on(modules=('theano',))
+def theano_function(inputs, outputs, scalar=False, *,
+        dim=None, dims=None, broadcastables=None, **kwargs):
     """
     Create a Theano function from SymPy expressions.
 
@@ -475,6 +485,12 @@ def theano_function(inputs, outputs, scalar=False, **kwargs):
     dim_handling
 
     """
+    SymPyDeprecationWarning(
+        feature="sympy.printing.theanocode",
+        useinstead="Theano is deprecated; use Aesara and sympy.printing.aesaracode",
+        issue=21150,
+        deprecated_since_version="1.8").warn()
+
     if not theano:
         raise ImportError("theano is required for theano_function")
 
@@ -483,10 +499,7 @@ def theano_function(inputs, outputs, scalar=False, **kwargs):
     dtypes = kwargs.pop('dtypes', {})
 
     broadcastables = dim_handling(
-        inputs,
-        dim=kwargs.pop('dim', None),
-        dims=kwargs.pop('dims', None),
-        broadcastables=kwargs.pop('broadcastables', None),
+        inputs, dim=dim, dims=dims, broadcastables=broadcastables,
     )
 
     # Print inputs/outputs
