@@ -1,5 +1,3 @@
-from __future__ import print_function, division
-
 from sympy.core import Function, S, Mul, Pow, Add
 from sympy.core.compatibility import ordered, default_sort_key
 from sympy.core.function import count_ops, expand_func
@@ -14,6 +12,9 @@ def gammasimp(expr):
     r"""
     Simplify expressions with gamma functions.
 
+    Explanation
+    ===========
+
     This function takes as input an expression containing gamma
     functions or functions that can be rewritten in terms of gamma
     functions and tries to minimize the number of those functions and
@@ -39,8 +40,8 @@ def gammasimp(expr):
 
     All transformation rules can be found (or was derived from) here:
 
-    1. http://functions.wolfram.com/GammaBetaErf/Pochhammer/17/01/02/
-    2. http://functions.wolfram.com/GammaBetaErf/Pochhammer/27/01/0005/
+    .. [1] http://functions.wolfram.com/GammaBetaErf/Pochhammer/17/01/02/
+    .. [2] http://functions.wolfram.com/GammaBetaErf/Pochhammer/27/01/0005/
 
     Examples
     ========
@@ -65,6 +66,9 @@ def _gammasimp(expr, as_comb):
     """
     Helper function for gammasimp and combsimp.
 
+    Explanation
+    ===========
+
     Simplifies expressions written in terms of gamma function. If
     as_comb is True, it tries to preserve integer arguments. See
     docstring of gammasimp for more information. This was part of
@@ -417,7 +421,7 @@ def _gammasimp(expr, as_comb):
                 if expr in inv:
                     return inv[expr]
                 return (expr.free_symbols, expr.atoms(Function).union(
-                        set(e.exp for e in expr.atoms(Pow))))
+                        {e.exp for e in expr.atoms(Pow)}))
 
             def update_ST(expr):
                 inv[expr] = compute_ST(expr)
