@@ -1,5 +1,3 @@
-from __future__ import print_function, division
-
 from collections import defaultdict
 
 from sympy import SYMPY_DEBUG
@@ -11,7 +9,7 @@ from sympy.core.parameters import global_parameters
 from sympy.core.exprtools import Factors, gcd_terms
 from sympy.core.function import _mexpand
 from sympy.core.mul import _keep_coeff, _unevaluated_Mul
-from sympy.core.numbers import Rational
+from sympy.core.numbers import Rational, zoo, nan
 from sympy.functions import exp, sqrt, log
 from sympy.functions.elementary.complexes import Abs
 from sympy.polys import gcd
@@ -24,6 +22,9 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
     """
     Collect additive terms of an expression.
 
+    Explanation
+    ===========
+
     This function collects additive terms of an expression with respect
     to a list of expression up to powers with rational exponents. By the
     term symbol here are meant arbitrary expressions, which can contain
@@ -31,7 +32,7 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
     will be searched for in the expression's terms.
 
     The input expression is not expanded by :func:`collect`, so user is
-    expected to provide an expression is an appropriate form. This makes
+    expected to provide an expression in an appropriate form. This makes
     :func:`collect` more predictable as there is no magic happening behind the
     scenes. However, it is important to note, that powers of products are
     converted to products of powers using the :func:`~.expand_power_base`
@@ -240,8 +241,8 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
          - sym_expo is the symbolic exponent that sexpr is raised to
          - deriv contains the derivatives the the expression
 
-         for example, the output of x would be (x, 1, None, None)
-         the output of 2**x would be (2, 1, x, None)
+         For example, the output of x would be (x, 1, None, None)
+         the output of 2**x would be (2, 1, x, None).
         """
         rat_expo, sym_expo = S.One, None
         sexpr, deriv = expr, None
@@ -252,7 +253,15 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
             else:
                 sexpr = expr.base
 
-            if expr.exp.is_Number:
+            if expr.base == S.Exp1:
+                arg = expr.exp
+                if arg.is_Rational:
+                    sexpr, rat_expo = S.Exp1, arg
+                elif arg.is_Mul:
+                    coeff, tail = arg.as_coeff_Mul(rational=True)
+                    sexpr, rat_expo = exp(tail), coeff
+
+            elif expr.exp.is_Number:
                 rat_expo = expr.exp
             else:
                 coeff, tail = expr.exp.as_coeff_Mul()
@@ -262,7 +271,7 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
                 else:
                     sym_expo = expr.exp
         elif isinstance(expr, exp):
-            arg = expr.args[0]
+            arg = expr.exp
             if arg.is_Rational:
                 sexpr, rat_expo = S.Exp1, arg
             elif arg.is_Mul:
@@ -275,8 +284,8 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
 
     def parse_expression(terms, pattern):
         """Parse terms searching for a pattern.
-        terms is a list of tuples as returned by parse_terms;
-        pattern is an expression treated as a product of factors
+        Terms is a list of tuples as returned by parse_terms;
+        Pattern is an expression treated as a product of factors.
         """
         pattern = Mul.make_args(pattern)
 
@@ -433,8 +442,8 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
             collected[key] = val + order_term
 
     if func is not None:
-        collected = dict(
-            [(key, func(val)) for key, val in collected.items()])
+        collected = {
+            key: func(val) for key, val in collected.items()}
 
     if evaluate:
         return Add(*[key*val for key, val in collected.items()])
@@ -742,7 +751,10 @@ def radsimp(expr, symbolic=True, max_terms=4):
     r"""
     Rationalize the denominator by removing square roots.
 
-    Note: the expression returned from radsimp must be used with caution
+    Explanation
+    ===========
+
+    The expression returned from radsimp must be used with caution
     since if the denominator contains symbols, it will be possible to make
     substitutions that violate the assumptions of the simplification process:
     that for a denominator matching a + b*sqrt(c), a != +/-b*sqrt(c). (If
@@ -804,7 +816,7 @@ def radsimp(expr, symbolic=True, max_terms=4):
     >>> radsimp(eq).subs(a, b*sqrt(c))
     nan
 
-    If symbolic=False, symbolic denominators will not be transformed (but
+    If ``symbolic=False``, symbolic denominators will not be transformed (but
     numeric denominators will still be processed):
 
     >>> radsimp(eq, symbolic=False)
@@ -954,6 +966,8 @@ def radsimp(expr, symbolic=True, max_terms=4):
             n *= num
             d *= num
             d = powdenest(_mexpand(d), force=symbolic)
+            if d.has(S.Zero, nan, zoo):
+                return expr
             if d.is_Atom:
                 break
 
@@ -988,7 +1002,7 @@ def radsimp(expr, symbolic=True, max_terms=4):
 
 def rad_rationalize(num, den):
     """
-    Rationalize num/den by removing square roots in the denominator;
+    Rationalize ``num/den`` by removing square roots in the denominator;
     num and den are sum of terms whose squares are positive rationals.
 
     Examples
@@ -1041,7 +1055,7 @@ def fraction(expr, exact=False):
        >>> fraction(x * y**k)
        (x, y**(-k))
 
-       If we know nothing about sign of some exponent and 'exact'
+       If we know nothing about sign of some exponent and ``exact``
        flag is unset, then structure this exponent's structure will
        be analyzed and pretty fraction will be returned:
 
@@ -1055,7 +1069,7 @@ def fraction(expr, exact=False):
        >>> fraction(exp(-x), exact=True)
        (exp(-x), 1)
 
-       The `exact` flag will also keep any unevaluated Muls from
+       The ``exact`` flag will also keep any unevaluated Muls from
        being evaluated:
 
        >>> u = Mul(2, x + 1, evaluate=False)
@@ -1169,9 +1183,9 @@ def split_surds(expr):
 
 def _split_gcd(*a):
     """
-    split the list of integers ``a`` into a list of integers, ``a1`` having
+    Split the list of integers ``a`` into a list of integers, ``a1`` having
     ``g = gcd(a1)``, and a list ``a2`` whose elements are not divisible by
-    ``g``.  Returns ``g, a1, a2``
+    ``g``.  Returns ``g, a1, a2``.
 
     Examples
     ========
