@@ -11,7 +11,7 @@ sympy.stats.drv
 
 
 from sympy import (Basic, Lambda, sympify, Indexed, Symbol, ProductSet, S,
-                   Dummy)
+                   Dummy, prod)
 from sympy.concrete.products import Product
 from sympy.concrete.summations import Sum, summation
 from sympy.core.compatibility import iterable
@@ -20,8 +20,9 @@ from sympy.integrals.integrals import Integral, integrate
 from sympy.matrices import ImmutableMatrix, matrix2numpy, list2numpy
 from sympy.stats.crv import SingleContinuousDistribution, SingleContinuousPSpace
 from sympy.stats.drv import SingleDiscreteDistribution, SingleDiscretePSpace
-from sympy.stats.rv import (ProductPSpace, NamedArgsMixin,
-                            ProductDomain, RandomSymbol, random_symbols, SingleDomain, _symbol_converter)
+from sympy.stats.rv import (ProductPSpace, NamedArgsMixin, Distribution,
+                            ProductDomain, RandomSymbol, random_symbols,
+                            SingleDomain, _symbol_converter)
 from sympy.utilities.misc import filldedent
 from sympy.external import import_module
 
@@ -149,15 +150,26 @@ class SampleJointScipy:
     def _sample_scipy(cls, dist, size, seed):
         """Sample from SciPy."""
 
+        import numpy
+        if seed is None or isinstance(seed, int):
+            rand_state = numpy.random.default_rng(seed=seed)
+        else:
+            rand_state = seed
         from scipy import stats as scipy_stats
         scipy_rv_map = {
             'MultivariateNormalDistribution': lambda dist, size: scipy_stats.multivariate_normal.rvs(
                 mean=matrix2numpy(dist.mu).flatten(),
-                cov=matrix2numpy(dist.sigma), size=size, random_state=seed),
+                cov=matrix2numpy(dist.sigma), size=size, random_state=rand_state),
             'MultivariateBetaDistribution': lambda dist, size: scipy_stats.dirichlet.rvs(
-                alpha=list2numpy(dist.alpha, float).flatten(), size=size, random_state=seed),
+                alpha=list2numpy(dist.alpha, float).flatten(), size=size, random_state=rand_state),
             'MultinomialDistribution': lambda dist, size: scipy_stats.multinomial.rvs(
-                n=int(dist.n), p=list2numpy(dist.p, float).flatten(), size=size, random_state=seed)
+                n=int(dist.n), p=list2numpy(dist.p, float).flatten(), size=size, random_state=rand_state)
+        }
+
+        sample_shape = {
+            'MultivariateNormalDistribution': lambda dist: matrix2numpy(dist.mu).flatten().shape,
+            'MultivariateBetaDistribution': lambda dist: list2numpy(dist.alpha).flatten().shape,
+            'MultinomialDistribution': lambda dist: list2numpy(dist.p).flatten().shape
         }
 
         dist_list = scipy_rv_map.keys()
@@ -166,9 +178,7 @@ class SampleJointScipy:
             return None
 
         samples = scipy_rv_map[dist.__class__.__name__](dist, size)
-        if samples.shape[0:len(size)] != size:
-            samples = samples.reshape((size[0],) + samples.shape)
-        return samples
+        return samples.reshape(size + sample_shape[dist.__class__.__name__](dist))
 
 class SampleJointNumpy:
     """Returns the sample from numpy of the given distribution"""
@@ -195,15 +205,19 @@ class SampleJointNumpy:
                 n=int(dist.n), pvals=list2numpy(dist.p, float).flatten(), size=size)
         }
 
+        sample_shape = {
+            'MultivariateNormalDistribution': lambda dist: matrix2numpy(dist.mu).flatten().shape,
+            'MultivariateBetaDistribution': lambda dist: list2numpy(dist.alpha).flatten().shape,
+            'MultinomialDistribution': lambda dist: list2numpy(dist.p).flatten().shape
+        }
+
         dist_list = numpy_rv_map.keys()
 
         if dist.__class__.__name__ not in dist_list:
             return None
 
-        samples = numpy_rv_map[dist.__class__.__name__](dist, size)
-        if samples.shape[0:len(size)] != size:
-            samples = samples.reshape((size[0],) + samples.shape)
-        return samples
+        samples = numpy_rv_map[dist.__class__.__name__](dist, prod(size))
+        return samples.reshape(size + sample_shape[dist.__class__.__name__](dist))
 
 class SampleJointPymc:
     """Returns the sample from pymc3 of the given distribution"""
@@ -227,17 +241,23 @@ class SampleJointPymc:
                 p=list2numpy(dist.p, float).flatten(), shape=(1, len(dist.p)))
         }
 
+        sample_shape = {
+            'MultivariateNormalDistribution': lambda dist: matrix2numpy(dist.mu).flatten().shape,
+            'MultivariateBetaDistribution': lambda dist: list2numpy(dist.alpha).flatten().shape,
+            'MultinomialDistribution': lambda dist: list2numpy(dist.p).flatten().shape
+        }
+
         dist_list = pymc3_rv_map.keys()
 
         if dist.__class__.__name__ not in dist_list:
             return None
 
+        import logging
+        logging.getLogger("pymc3").setLevel(logging.ERROR)
         with pymc3.Model():
             pymc3_rv_map[dist.__class__.__name__](dist)
-            samples = pymc3.sample(size, chains=1, progressbar=False, random_seed=seed)[:]['X']
-            if samples.shape[0:len(size)] != size:
-                samples = samples.reshape((size[0],) + samples.shape)
-            return samples
+            samples = pymc3.sample(draws=prod(size), chains=1, progressbar=False, random_seed=seed, return_inferencedata=False, compute_convergence_checks=False)[:]['X']
+        return samples.reshape(size + sample_shape[dist.__class__.__name__](dist))
 
 
 _get_sample_class_jrv = {
@@ -246,7 +266,7 @@ _get_sample_class_jrv = {
     'numpy': SampleJointNumpy
 }
 
-class JointDistribution(Basic, NamedArgsMixin):
+class JointDistribution(Distribution, NamedArgsMixin):
     """
     Represented by the random variables part of the joint distribution.
     Contains methods for PDF, CDF, sampling, marginal densities, etc.
@@ -320,7 +340,7 @@ class JointRandomSymbol(RandomSymbol):
 
 
 
-class MarginalDistribution(Basic):
+class MarginalDistribution(Distribution):
     """
     Represents the marginal distribution of a joint probability space.
 
