@@ -4,11 +4,13 @@ from sympy import (sympify, S, pi, sqrt, exp, Lambda, Indexed, besselk, gamma, I
                    Intersection, Matrix, symbols, Product, IndexedBase)
 from sympy.matrices import ImmutableMatrix, MatrixSymbol
 from sympy.matrices.expressions.determinant import det
-from sympy.stats.joint_rv import (JointDistribution, JointPSpace,
-    JointDistributionHandmade, MarginalDistribution)
+from sympy.matrices.expressions.matexpr import MatrixElement
+from sympy.stats.joint_rv import JointDistribution, JointPSpace, MarginalDistribution
 from sympy.stats.rv import _value_check, random_symbols
 
 __all__ = ['JointRV',
+'MultivariateNormal',
+'MultivariateLaplace',
 'Dirichlet',
 'GeneralizedMultivariateLogGamma',
 'GeneralizedMultivariateLogGammaOmega',
@@ -27,6 +29,55 @@ def multivariate_rv(cls, sym, *args):
     dist.check(*args)
     return JointPSpace(sym, dist).value
 
+
+def marginal_distribution(rv, *indices):
+    """
+    Marginal distribution function of a joint random variable.
+
+    Parameters
+    ==========
+
+    rv: A random variable with a joint probability distribution.
+    indices: component indices or the indexed random symbol
+        for whom the joint distribution is to be calculated
+
+    Returns
+    =======
+
+    A Lambda expression in `sym`.
+
+    Examples
+    ========
+
+    >>> from sympy.stats import MultivariateNormal, marginal_distribution
+    >>> m = MultivariateNormal('X', [1, 2], [[2, 1], [1, 2]])
+    >>> marginal_distribution(m, m[0])(1)
+    1/(2*sqrt(pi))
+
+    """
+    indices = list(indices)
+    for i in range(len(indices)):
+        if isinstance(indices[i], Indexed):
+            indices[i] = indices[i].args[1]
+    prob_space = rv.pspace
+    if not indices:
+        raise ValueError(
+            "At least one component for marginal density is needed.")
+    if hasattr(prob_space.distribution, '_marginal_distribution'):
+        return prob_space.distribution._marginal_distribution(indices, rv.symbol)
+    return prob_space.marginal_distribution(*indices)
+
+
+class JointDistributionHandmade(JointDistribution):
+
+    _argnames = ('pdf',)
+    is_Continuous = True
+
+    @property
+    def set(self):
+        return self.args[1]
+
+
 def JointRV(symbol, pdf, _set=None):
     """
     Create a Joint Random Variable where each of its component is conitinuous,
@@ -39,21 +90,22 @@ def JointRV(symbol, pdf, _set=None):
     NOTE: As of now, the set for each component for a `JointRV` is
     equal to the set of all integers, which can not be changed.
 
-    Returns a RandomSymbol.
-
     Examples
     ========
 
-    >>> from sympy import symbols, exp, pi, Indexed, S
-    >>> from sympy.stats import density
-    >>> from sympy.stats.joint_rv_types import JointRV
-
+    >>> from sympy import exp, pi, Indexed, S
+    >>> from sympy.stats import density, JointRV
     >>> x1, x2 = (Indexed('x', i) for i in (1, 2))
     >>> pdf = exp(-x1**2/2 + x1 - x2**2/2 - S(1)/2)/(2*pi)
-
     >>> N1 = JointRV('x', pdf) #Multivariate Normal distribution
     >>> density(N1)(1, 2)
     exp(-2)/(2*pi)
+
+    Returns
+    =======
+
+    RandomSymbol
+
     """
     #TODO: Add support for sets provided by the user
     symbol = sympify(symbol)
@@ -71,10 +123,10 @@ def JointRV(symbol, pdf, _set=None):
     return jrv
 
 #-------------------------------------------------------------------------------
-# Multivariate Normal distribution ---------------------------------------------------------
+# Multivariate Normal distribution ---------------------------------------------
 
 class MultivariateNormalDistribution(JointDistribution):
-    _argnames = ['mu', 'sigma']
+    _argnames = ('mu', 'sigma')
 
     is_Continuous=True
 
@@ -87,21 +139,24 @@ class MultivariateNormalDistribution(JointDistribution):
     def check(mu, sigma):
         _value_check(mu.shape[0] == sigma.shape[0],
             "Size of the mean vector and covariance matrix are incorrect.")
-        #check if covariance matrix is positive definite or not.
+        #check if covariance matrix is positive semi definite or not.
         if not isinstance(sigma, MatrixSymbol):
-            _value_check(sigma.is_positive_definite,
-            "The covariance matrix must be positive definite. ")
+            _value_check(sigma.is_positive_semidefinite,
+            "The covariance matrix must be positive semi definite. ")
 
     def pdf(self, *args):
         mu, sigma = self.mu, self.sigma
         k = mu.shape[0]
-        args = ImmutableMatrix(args)
+        if len(args) == 1 and args[0].is_Matrix:
+            args = args[0]
+        else:
+            args = ImmutableMatrix(args)
         x = args - mu
-        return  S.One/sqrt((2*pi)**(k)*det(sigma))*exp(
-            Rational(-1, 2)*x.transpose()*(sigma.inv()*\
-                x))[0]
+        density = S.One/sqrt((2*pi)**(k)*det(sigma))*exp(
+            Rational(-1, 2)*x.transpose()*(sigma.inv()*x))
+        return MatrixElement(density, 0, 0)
 
-    def marginal_distribution(self, indices, sym):
+    def _marginal_distribution(self, indices, sym):
         sym = ImmutableMatrix([Indexed(sym, i) for i in indices])
         _mu, _sigma = self.mu, self.sigma
         k = self.mu.shape[0]
@@ -114,11 +169,70 @@ class MultivariateNormalDistribution(JointDistribution):
             Rational(-1, 2)*(_mu - sym).transpose()*(_sigma.inv()*\
                 (_mu - sym)))[0])
 
+def MultivariateNormal(name, mu, sigma):
+    """
+    Creates a continuous random variable with Multivariate Normal
+    Distribution.
+
+    The density of the multivariate normal distribution can be found at [1].
+
+    Parameters
+    ==========
+
+    mu : List representing the mean or the mean vector
+    sigma : Positive semidefinite square matrix
+        Represents covariance Matrix
+        If `sigma` is noninvertible then only sampling is supported currently
+
+    Returns
+    =======
+
+    RandomSymbol
+
+    Examples
+    ========
+
+    >>> from sympy.stats import MultivariateNormal, density, marginal_distribution
+    >>> from sympy import symbols, MatrixSymbol
+    >>> X = MultivariateNormal('X', [3, 4], [[2, 1], [1, 2]])
+    >>> y, z = symbols('y z')
+    >>> density(X)(y, z)
+    sqrt(3)*exp(-y**2/3 + y*z/3 + 2*y/3 - z**2/3 + 5*z/3 - 13/3)/(6*pi)
+    >>> density(X)(1, 2)
+    sqrt(3)*exp(-4/3)/(6*pi)
+    >>> marginal_distribution(X, X[1])(y)
+    exp(-(y - 4)**2/4)/(2*sqrt(pi))
+    >>> marginal_distribution(X, X[0])(y)
+    exp(-(y - 3)**2/4)/(2*sqrt(pi))
+
+    The example below shows that it is also possible to use
+    symbolic parameters to define the MultivariateNormal class.
+
+    >>> n = symbols('n', natural=True)
+    >>> Sg = MatrixSymbol('Sg', n, n)
+    >>> mu = MatrixSymbol('mu', n, 1)
+    >>> obs = MatrixSymbol('obs', n, 1)
+    >>> X = MultivariateNormal('X', mu, Sg)
+
+    The density of a multivariate normal can be
+    calculated using a matrix argument, as shown below.
+
+    >>> density(X)(obs)
+    (exp(((1/2)*mu.T - (1/2)*obs.T)*Sg**(-1)*(-mu + obs))/sqrt((2*pi)**n*Determinant(Sg)))[0, 0]
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Multivariate_normal_distribution
+
+    """
+    return multivariate_rv(MultivariateNormalDistribution, name, mu, sigma)
+
 #-------------------------------------------------------------------------------
-# Multivariate Laplace distribution ---------------------------------------------------------
+# Multivariate Laplace distribution --------------------------------------------
 
 class MultivariateLaplaceDistribution(JointDistribution):
-    _argnames = ['mu', 'sigma']
+    _argnames = ('mu', 'sigma')
     is_Continuous=True
 
     @property
@@ -149,12 +263,50 @@ class MultivariateLaplaceDistribution(JointDistribution):
         *(y/(2 + x))**(S(v)/2)*besselk(v, sqrt((2 + x)*(y)))\
         *exp((args_T*sigma_inv*mu)[0])
 
+def MultivariateLaplace(name, mu, sigma):
+    """
+    Creates a continuous random variable with Multivariate Laplace
+    Distribution.
+
+    The density of the multivariate Laplace distribution can be found at [1].
+
+    Parameters
+    ==========
+
+    mu : List representing the mean or the mean vector
+    sigma : Positive definite square matrix
+        Represents covariance Matrix
+
+    Returns
+    =======
+
+    RandomSymbol
+
+    Examples
+    ========
+
+    >>> from sympy.stats import MultivariateLaplace, density
+    >>> from sympy import symbols
+    >>> y, z = symbols('y z')
+    >>> X = MultivariateLaplace('X', [2, 4], [[3, 1], [1, 3]])
+    >>> density(X)(y, z)
+    sqrt(2)*exp(y/4 + 5*z/4)*besselk(0, sqrt(15*y*(3*y/8 - z/8)/2 + 15*z*(-y/8 + 3*z/8)/2))/(4*pi)
+    >>> density(X)(1, 2)
+    sqrt(2)*exp(11/4)*besselk(0, sqrt(165)/4)/(4*pi)
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Multivariate_Laplace_distribution
+
+    """
+    return multivariate_rv(MultivariateLaplaceDistribution, name, mu, sigma)
 
 #-------------------------------------------------------------------------------
-# Multivariate StudentT distribution ---------------------------------------------------------
+# Multivariate StudentT distribution -------------------------------------------
 
 class MultivariateTDistribution(JointDistribution):
-    _argnames = ['mu', 'shape_mat', 'dof']
+    _argnames = ('mu', 'shape_mat', 'dof')
     is_Continuous=True
 
     @property
@@ -188,25 +340,39 @@ def MultivariateT(syms, mu, sigma, v):
     Parameters
     ==========
 
-    syms: list/tuple/set of symbols for identifying each component
-    mu: A list/tuple/set consisting of k means,represents a k
-        dimensional location vector
+    syms: A symbol/str
+        For identifying the random variable.
+    mu: A list/matrix
+        Representing the location vector
     sigma: The shape matrix for the distribution
 
+    Examples
+    ========
+
+    >>> from sympy.stats import density, MultivariateT
+    >>> from sympy import Symbol
+
+    >>> x = Symbol("x")
+    >>> X = MultivariateT("x", [1, 1], [[1, 0], [0, 1]], 2)
+
+    >>> density(X)(1, 2)
+    2/(9*pi)
+
     Returns
     =======
 
-    A random symbol
+    RandomSymbol
+
     """
     return multivariate_rv(MultivariateTDistribution, syms, mu, sigma, v)
 
 
 #-------------------------------------------------------------------------------
-# Multivariate Normal Gamma distribution ---------------------------------------------------------
+# Multivariate Normal Gamma distribution ---------------------------------------
 
 class NormalGammaDistribution(JointDistribution):
 
-    _argnames = ['mu', 'lamda', 'alpha', 'beta']
+    _argnames = ('mu', 'lamda', 'alpha', 'beta')
     is_Continuous=True
 
     @staticmethod
@@ -228,7 +394,7 @@ class NormalGammaDistribution(JointDistribution):
         tau**(alpha - S.Half)*exp(-1*beta*tau)*\
         exp(-1*(lamda*tau*(x - mu)**2)/S(2))
 
-    def marginal_distribution(self, indices, *sym):
+    def _marginal_distribution(self, indices, *sym):
         if len(indices) == 2:
             return self.pdf(*sym)
         if indices[0] == 0:
@@ -243,7 +409,7 @@ class NormalGammaDistribution(JointDistribution):
         from sympy.stats.crv_types import GammaDistribution
         return Lambda(sym, GammaDistribution(self.alpha, self.beta)(sym[0]))
 
-def NormalGamma(syms, mu, lamda, alpha, beta):
+def NormalGamma(sym, mu, lamda, alpha, beta):
     """
     Creates a bivariate joint random variable with multivariate Normal gamma
     distribution.
@@ -251,25 +417,48 @@ def NormalGamma(syms, mu, lamda, alpha, beta):
     Parameters
     ==========
 
-    syms: list/tuple/set of two symbols for identifying each component
-    mu: A real number, as the mean of the normal distribution
-    alpha: a positive integer
-    beta: a positive integer
-    lamda: a positive integer
+    sym: A symbol/str
+        For identifying the random variable.
+    mu: A real number
+        The mean of the normal distribution
+    lamda: A positive integer
+        Parameter of joint distribution
+    alpha: A positive integer
+        Parameter of joint distribution
+    beta: A positive integer
+        Parameter of joint distribution
 
     Returns
     =======
 
-    A random symbol
+    RandomSymbol
+
+    Examples
+    ========
+
+    >>> from sympy.stats import density, NormalGamma
+    >>> from sympy import symbols
+
+    >>> X = NormalGamma('x', 0, 1, 2, 3)
+    >>> y, z = symbols('y z')
+
+    >>> density(X)(y, z)
+    9*sqrt(2)*z**(3/2)*exp(-3*z)*exp(-y**2*z/2)/(2*sqrt(pi))
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Normal-gamma_distribution
+
     """
-    return multivariate_rv(NormalGammaDistribution, syms, mu, lamda, alpha, beta)
+    return multivariate_rv(NormalGammaDistribution, sym, mu, lamda, alpha, beta)
 
 #-------------------------------------------------------------------------------
-# Multivariate Beta/Dirichlet distribution ---------------------------------------------------------
+# Multivariate Beta/Dirichlet distribution -------------------------------------
 
 class MultivariateBetaDistribution(JointDistribution):
 
-    _argnames = ['alpha']
+    _argnames = ('alpha',)
     is_Continuous = True
 
     @staticmethod
@@ -287,7 +476,7 @@ class MultivariateBetaDistribution(JointDistribution):
     def pdf(self, *syms):
         alpha = self.alpha
         B = Mul.fromiter(map(gamma, alpha))/gamma(Add(*alpha))
-        return Mul.fromiter([sym**(a_k - 1) for a_k, sym in zip(alpha, syms)])/B
+        return Mul.fromiter(sym**(a_k - 1) for a_k, sym in zip(alpha, syms))/B
 
 def MultivariateBeta(syms, *alpha):
     """
@@ -299,19 +488,18 @@ def MultivariateBeta(syms, *alpha):
     Parameters
     ==========
 
-    alpha: positive real numbers signifying concentration numbers.
+    alpha: Positive real numbers
+        Signifies concentration numbers.
 
     Returns
     =======
 
-    A RandomSymbol.
+    RandomSymbol
 
     Examples
     ========
 
-    >>> from sympy.stats import density
-    >>> from sympy.stats.joint_rv import marginal_distribution
-    >>> from sympy.stats.joint_rv_types import MultivariateBeta
+    >>> from sympy.stats import density, MultivariateBeta, marginal_distribution
     >>> from sympy import Symbol
     >>> a1 = Symbol('a1', positive=True)
     >>> a2 = Symbol('a2', positive=True)
@@ -338,11 +526,11 @@ def MultivariateBeta(syms, *alpha):
 Dirichlet = MultivariateBeta
 
 #-------------------------------------------------------------------------------
-# Multivariate Ewens distribution ---------------------------------------------------------
+# Multivariate Ewens distribution ----------------------------------------------
 
 class MultivariateEwensDistribution(JointDistribution):
 
-    _argnames = ['n', 'theta']
+    _argnames = ('n', 'theta')
     is_Discrete = True
     is_Continuous = False
 
@@ -371,8 +559,8 @@ class MultivariateEwensDistribution(JointDistribution):
                                 "the dimension is symbolic")
         term_1 = factorial(n)/rf(theta, n)
         if condi:
-            term_2 = Mul.fromiter([theta**syms[j]/((j+1)**syms[j]*factorial(syms[j]))
-                                    for j in range(n)])
+            term_2 = Mul.fromiter(theta**syms[j]/((j+1)**syms[j]*factorial(syms[j]))
+                                    for j in range(n))
             cond = Eq(sum([(k + 1)*syms[k] for k in range(n)]), n)
             return Piecewise((term_1 * term_2, cond), (0, True))
         syms = syms[0]
@@ -393,27 +581,26 @@ def MultivariateEwens(syms, n, theta):
     Parameters
     ==========
 
-    n: positive integer of class Integer,
-            size of the sample or the integer whose partitions are considered
-    theta: mutation rate, must be positive real number.
+    n: Positive integer
+        Size of the sample or the integer whose partitions are considered
+    theta: Positive real number
+        Denotes Mutation rate
 
     Returns
     =======
 
-    A RandomSymbol.
+    RandomSymbol
 
     Examples
     ========
 
-    >>> from sympy.stats import density
-    >>> from sympy.stats.joint_rv import marginal_distribution
-    >>> from sympy.stats.joint_rv_types import MultivariateEwens
+    >>> from sympy.stats import density, marginal_distribution, MultivariateEwens
     >>> from sympy import Symbol
     >>> a1 = Symbol('a1', positive=True)
     >>> a2 = Symbol('a2', positive=True)
     >>> ed = MultivariateEwens('E', 2, 1)
     >>> density(ed)(a1, a2)
-    Piecewise((2**(-a2)/(factorial(a1)*factorial(a2)), Eq(a1 + 2*a2, 2)), (0, True))
+    Piecewise((1/(2**a2*factorial(a1)*factorial(a2)), Eq(a1 + 2*a2, 2)), (0, True))
     >>> marginal_distribution(ed, ed[0])(a1)
     Piecewise((1/factorial(a1), Eq(a1, 2)), (0, True))
 
@@ -427,11 +614,11 @@ def MultivariateEwens(syms, n, theta):
     return multivariate_rv(MultivariateEwensDistribution, syms, n, theta)
 
 #-------------------------------------------------------------------------------
-# Generalized Multivariate Log Gamma distribution ---------------------------------------------------------
+# Generalized Multivariate Log Gamma distribution ------------------------------
 
 class GeneralizedMultivariateLogGammaDistribution(JointDistribution):
 
-    _argnames = ['delta', 'v', 'lamda', 'mu']
+    _argnames = ('delta', 'v', 'lamda', 'mu')
     is_Continuous=True
 
     def check(self, delta, v, l, mu):
@@ -455,7 +642,7 @@ class GeneralizedMultivariateLogGammaDistribution(JointDistribution):
         k = len(l)
         sterm1 = Pow((1 - d), n)/\
                 ((gamma(v + n)**(k - 1))*gamma(v)*gamma(n + 1))
-        sterm2 = Mul.fromiter([mui*li**(-v - n) for mui, li in zip(mu, l)])
+        sterm2 = Mul.fromiter(mui*li**(-v - n) for mui, li in zip(mu, l))
         term1 = sterm1 * sterm2
         sterm3 = (v + n) * sum([mui * yi for mui, yi in zip(mu, y)])
         sterm4 = sum([exp(mui * yi)/li for (mui, yi, li) in zip(mu, y, l)])
@@ -474,20 +661,19 @@ def GeneralizedMultivariateLogGamma(syms, delta, v, lamda, mu):
 
     syms: list/tuple/set of symbols for identifying each component
     delta: A constant in range [0, 1]
-    v: positive real
-    lamda: a list of positive reals
-    mu: a list of positive reals
+    v: Positive real number
+    lamda: List of positive real numbers
+    mu: List of positive real numbers
 
     Returns
     =======
 
-    A Random Symbol
+    RandomSymbol
 
     Examples
     ========
 
     >>> from sympy.stats import density
-    >>> from sympy.stats.joint_rv import marginal_distribution
     >>> from sympy.stats.joint_rv_types import GeneralizedMultivariateLogGamma
     >>> from sympy import symbols, S
     >>> v = 1
@@ -496,8 +682,8 @@ def GeneralizedMultivariateLogGamma(syms, delta, v, lamda, mu):
     >>> y = symbols('y_1:4', positive=True)
     >>> Gd = GeneralizedMultivariateLogGamma('G', d, v, l, mu)
     >>> density(Gd)(y[0], y[1], y[2])
-    Sum(2**(-n)*exp((n + 1)*(y_1 + y_2 + y_3) - exp(y_1) - exp(y_2) -
-    exp(y_3))/gamma(n + 1)**3, (n, 0, oo))/2
+    Sum(exp((n + 1)*(y_1 + y_2 + y_3) - exp(y_1) - exp(y_2) -
+    exp(y_3))/(2**n*gamma(n + 1)**3), (n, 0, oo))/2
 
     References
     ==========
@@ -524,24 +710,24 @@ def GeneralizedMultivariateLogGammaOmega(syms, omega, v, lamda, mu):
     Parameters
     ==========
 
-    syms: list/tuple/set of symbols for identifying each component
+    syms: list/tuple/set of symbols
+        For identifying each component
     omega: A square matrix
            Every element of square matrix must be absolute value of
            square root of correlation coefficient
-    v: positive real
-    lamda: a list of positive reals
-    mu: a list of positive reals
+    v: Positive real number
+    lamda: List of positive real numbers
+    mu: List of positive real numbers
 
     Returns
     =======
 
-    A Random Symbol
+    RandomSymbol
 
     Examples
     ========
 
     >>> from sympy.stats import density
-    >>> from sympy.stats.joint_rv import marginal_distribution
     >>> from sympy.stats.joint_rv_types import GeneralizedMultivariateLogGammaOmega
     >>> from sympy import Matrix, symbols, S
     >>> omega = Matrix([[1, S.Half, S.Half], [S.Half, 1, S.Half], [S.Half, S.Half, 1]])
@@ -556,13 +742,15 @@ def GeneralizedMultivariateLogGammaOmega(syms, omega, v, lamda, mu):
     References
     ==========
 
-    See references of GeneralizedMultivariateLogGamma.
+    .. [1] https://en.wikipedia.org/wiki/Generalized_multivariate_log-gamma_distribution
+    .. [2] https://www.researchgate.net/publication/234137346_On_a_multivariate_log-gamma_distribution_and_the_use_of_the_distribution_in_the_Bayesian_analysis
 
     Notes
     =====
 
     If the GeneralizedMultivariateLogGammaOmega is too long to type use,
     `from sympy.stats.joint_rv_types import GeneralizedMultivariateLogGammaOmega as GMVLGO`
+
     """
     _value_check((omega.is_square, isinstance(omega, Matrix)), "omega must be a"
                                                             " square matrix")
@@ -581,11 +769,11 @@ def GeneralizedMultivariateLogGammaOmega(syms, omega, v, lamda, mu):
 
 
 #-------------------------------------------------------------------------------
-# Multinomial distribution ---------------------------------------------------------
+# Multinomial distribution -----------------------------------------------------
 
 class MultinomialDistribution(JointDistribution):
 
-    _argnames = ['n', 'p']
+    _argnames = ('n', 'p')
     is_Continuous=False
     is_Discrete = True
 
@@ -605,8 +793,8 @@ class MultinomialDistribution(JointDistribution):
 
     def pdf(self, *x):
         n, p = self.n, self.p
-        term_1 = factorial(n)/Mul.fromiter([factorial(x_k) for x_k in x])
-        term_2 = Mul.fromiter([p_k**x_k for p_k, x_k in zip(p, x)])
+        term_1 = factorial(n)/Mul.fromiter(factorial(x_k) for x_k in x)
+        term_2 = Mul.fromiter(p_k**x_k for p_k, x_k in zip(p, x))
         return Piecewise((term_1 * term_2, Eq(sum(x), n)), (0, True))
 
 def Multinomial(syms, n, *p):
@@ -617,19 +805,21 @@ def Multinomial(syms, n, *p):
 
     Parameters
     ==========
-    n: positive integer of class Integer,
-       number of trials
-    p: event probabilites, >= 0 and <= 1
+
+    n: Positive integer
+        Represents number of trials
+    p: List of event probabilites
+        Must be in the range of [0, 1]
 
     Returns
     =======
-    A RandomSymbol.
+
+    RandomSymbol
 
     Examples
     ========
-    >>> from sympy.stats import density
-    >>> from sympy.stats.joint_rv import marginal_distribution
-    >>> from sympy.stats.joint_rv_types import Multinomial
+
+    >>> from sympy.stats import density,  Multinomial, marginal_distribution
     >>> from sympy import symbols
     >>> x1, x2, x3 = symbols('x1, x2, x3', nonnegative=True, integer=True)
     >>> p1, p2, p3 = symbols('p1, p2, p3', positive=True)
@@ -642,19 +832,21 @@ def Multinomial(syms, n, *p):
 
     References
     ==========
+
     .. [1] https://en.wikipedia.org/wiki/Multinomial_distribution
     .. [2] http://mathworld.wolfram.com/MultinomialDistribution.html
+
     """
     if not isinstance(p[0], list):
         p = (list(p), )
     return multivariate_rv(MultinomialDistribution, syms, n, p[0])
 
 #-------------------------------------------------------------------------------
-# Negative Multinomial Distribution ---------------------------------------------------------
+# Negative Multinomial Distribution --------------------------------------------
 
 class NegativeMultinomialDistribution(JointDistribution):
 
-    _argnames = ['k0', 'p']
+    _argnames = ('k0', 'p')
     is_Continuous=False
     is_Discrete = True
 
@@ -675,7 +867,7 @@ class NegativeMultinomialDistribution(JointDistribution):
     def pdf(self, *k):
         k0, p = self.k0, self.p
         term_1 = (gamma(k0 + sum(k))*(1 - sum(p))**k0)/gamma(k0)
-        term_2 = Mul.fromiter([pi**ki/factorial(ki) for pi, ki in zip(p, k)])
+        term_2 = Mul.fromiter(pi**ki/factorial(ki) for pi, ki in zip(p, k))
         return term_1 * term_2
 
 def NegativeMultinomial(syms, k0, *p):
@@ -686,19 +878,21 @@ def NegativeMultinomial(syms, k0, *p):
 
     Parameters
     ==========
-    k0: positive integer of class Integer,
-        number of failures before the experiment is stopped
-    p: event probabilites, >= 0 and <= 1
+
+    k0: positive integer
+        Represents number of failures before the experiment is stopped
+    p: List of event probabilites
+        Must be in the range of [0, 1]
 
     Returns
     =======
-    A RandomSymbol.
+
+    RandomSymbol
 
     Examples
     ========
-    >>> from sympy.stats import density
-    >>> from sympy.stats.joint_rv import marginal_distribution
-    >>> from sympy.stats.joint_rv_types import NegativeMultinomial
+
+    >>> from sympy.stats import density, NegativeMultinomial, marginal_distribution
     >>> from sympy import symbols
     >>> x1, x2, x3 = symbols('x1, x2, x3', nonnegative=True, integer=True)
     >>> p1, p2, p3 = symbols('p1, p2, p3', positive=True)
@@ -713,8 +907,10 @@ def NegativeMultinomial(syms, k0, *p):
 
     References
     ==========
+
     .. [1] https://en.wikipedia.org/wiki/Negative_multinomial_distribution
     .. [2] http://mathworld.wolfram.com/NegativeBinomialDistribution.html
+
     """
     if not isinstance(p[0], list):
         p = (list(p), )
