@@ -19,13 +19,15 @@ from typing import Tuple as tTuple
 
 from sympy import (Basic, S, Expr, Symbol, Tuple, And, Add, Eq, lambdify, Or,
                    Equality, Lambda, sympify, Dummy, Ne, KroneckerDelta,
-                   DiracDelta, Mul, Indexed, MatrixSymbol, Function)
+                   DiracDelta, Mul, Indexed, MatrixSymbol, Function, prod)
 from sympy.core.relational import Relational
 from sympy.core.sympify import _sympify
 from sympy.sets.sets import FiniteSet, ProductSet, Intersection
 from sympy.solvers.solveset import solveset
 from sympy.external import import_module
 from sympy.utilities.misc import filldedent
+from sympy.utilities.decorator import doctest_depends_on
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 import warnings
 
 
@@ -42,7 +44,7 @@ def _(x):
 
 class RandomDomain(Basic):
     """
-    Represents a set of variables and the values which they can take
+    Represents a set of variables and the values which they can take.
 
     See Also
     ========
@@ -77,7 +79,7 @@ class RandomDomain(Basic):
 
 class SingleDomain(RandomDomain):
     """
-    A single variable and its domain
+    A single variable and its domain.
 
     See Also
     ========
@@ -106,7 +108,7 @@ class SingleDomain(RandomDomain):
 
 class MatrixDomain(RandomDomain):
     """
-    A Random Matrix variable and its domain
+    A Random Matrix variable and its domain.
 
     """
     def __new__(cls, symbol, set):
@@ -124,7 +126,7 @@ class MatrixDomain(RandomDomain):
 
 class ConditionalDomain(RandomDomain):
     """
-    A RandomDomain with an attached condition
+    A RandomDomain with an attached condition.
 
     See Also
     ========
@@ -159,7 +161,10 @@ class ConditionalDomain(RandomDomain):
 
 class PSpace(Basic):
     """
-    A Probability Space
+    A Probability Space.
+
+    Explanation
+    ===========
 
     Probability Spaces encode processes that equal different values
     probabilistically. These underly Random Symbols which occur in SymPy
@@ -199,7 +204,7 @@ class PSpace(Basic):
     def compute_density(self, expr):
         raise NotImplementedError()
 
-    def sample(self):
+    def sample(self, size=(), library='scipy', seed=None):
         raise NotImplementedError()
 
     def probability(self, condition):
@@ -237,11 +242,14 @@ class SinglePSpace(PSpace):
 
 class RandomSymbol(Expr):
     """
-    Random Symbols represent ProbabilitySpaces in SymPy Expressions
+    Random Symbols represent ProbabilitySpaces in SymPy Expressions.
     In principle they can take on any value that their symbol can take on
     within the associated PSpace with probability determined by the PSpace
     Density.
 
+    Explanation
+    ===========
+
     Random Symbols contain pspace and symbol properties.
     The pspace property points to the represented Probability Space
     The symbol is a standard SymPy Symbol that is used in that probability space
@@ -328,6 +336,10 @@ class RandomIndexedSymbol(RandomSymbol):
             return free_syms
         return {self}
 
+    @property
+    def pspace(self):
+        return self.args[1]
+
 class RandomMatrixSymbol(RandomSymbol, MatrixSymbol): # type: ignore
     def __new__(cls, symbol, n, m, pspace=None):
         n, m = _sympify(n), _sympify(m)
@@ -340,6 +352,7 @@ class RandomMatrixSymbol(RandomSymbol, MatrixSymbol): # type: ignore
     symbol = property(lambda self: self.args[0])
     pspace = property(lambda self: self.args[3])
 
+
 class ProductPSpace(PSpace):
     """
     Abstract class for representing probability spaces with multiple random
@@ -358,7 +371,7 @@ class IndependentProductPSpace(ProductPSpace):
     A probability space resulting from the merger of two independent probability
     spaces.
 
-    Often created using the function, pspace
+    Often created using the function, pspace.
     """
 
     def __new__(cls, *spaces):
@@ -509,7 +522,7 @@ class IndependentProductPSpace(ProductPSpace):
 
 class ProductDomain(RandomDomain):
     """
-    A domain resulting from the merger of two independent domains
+    A domain resulting from the merger of two independent domains.
 
     See Also
     ========
@@ -616,8 +629,9 @@ def pspace(expr):
     if all(rv.pspace == rvs[0].pspace for rv in rvs):
         return rvs[0].pspace
     from sympy.stats.compound_rv import CompoundPSpace
+    from sympy.stats.stochastic_process import StochasticPSpace
     for rv in rvs:
-        if isinstance(rv.pspace, CompoundPSpace):
+        if isinstance(rv.pspace, (CompoundPSpace, StochasticPSpace)):
             return rv.pspace
     # Otherwise make a product space
     return IndependentProductPSpace(*[rv.pspace for rv in rvs])
@@ -650,7 +664,11 @@ def rs_swap(a, b):
 
 
 def given(expr, condition=None, **kwargs):
-    r""" Conditional Random Expression
+    r""" Conditional Random Expression.
+
+    Explanation
+    ===========
+
     From a random expression and a condition on that expression creates a new
     probability space from the condition and returns the same expression on that
     conditional probability space.
@@ -732,7 +750,7 @@ def given(expr, condition=None, **kwargs):
 
 def expectation(expr, condition=None, numsamples=None, evaluate=True, **kwargs):
     """
-    Returns the expected value of a random expression
+    Returns the expected value of a random expression.
 
     Parameters
     ==========
@@ -774,7 +792,7 @@ def expectation(expr, condition=None, numsamples=None, evaluate=True, **kwargs):
 def probability(condition, given_condition=None, numsamples=None,
                 evaluate=True, **kwargs):
     """
-    Probability that a condition is true, optionally given a second condition
+    Probability that a condition is true, optionally given a second condition.
 
     Parameters
     ==========
@@ -861,6 +879,9 @@ def density(expr, condition=None, evaluate=True, numsamples=None, **kwargs):
     Probability density of a random expression, optionally given a second
     condition.
 
+    Explanation
+    ===========
+
     This density will take on different forms for different types of
     probability spaces. Discrete variables produce Dicts. Continuous
     variables produce Lambdas.
@@ -905,7 +926,10 @@ def cdf(expr, condition=None, evaluate=True, **kwargs):
     """
     Cumulative Distribution Function of a random expression.
 
-    optionally given a second condition
+    optionally given a second condition.
+
+    Explanation
+    ===========
 
     This density will take on different forms for different types of
     probability spaces.
@@ -945,9 +969,9 @@ def cdf(expr, condition=None, evaluate=True, **kwargs):
 
 def characteristic_function(expr, condition=None, evaluate=True, **kwargs):
     """
-    Characteristic function of a random expression, optionally given a second condition
+    Characteristic function of a random expression, optionally given a second condition.
 
-    Returns a Lambda
+    Returns a Lambda.
 
     Examples
     ========
@@ -1018,10 +1042,11 @@ def where(condition, given_condition=None, **kwargs):
     return pspace(condition).where(condition, **kwargs)
 
 
+@doctest_depends_on(modules=('scipy',))
 def sample(expr, condition=None, size=(), library='scipy',
            numsamples=1, seed=None, **kwargs):
     """
-    A realization of the random expression
+    A realization of the random expression.
 
     Parameters
     ==========
@@ -1040,7 +1065,9 @@ def sample(expr, condition=None, size=(), library='scipy',
         Choose any of the available options to sample from as string,
         by default is 'scipy'
     numsamples : int
-        Number of samples, each with size as ``size``
+        Number of samples, each with size as ``size``. The ``numsamples`` parameter is
+        deprecated and is only provided for compatibility with v1.8. Use a list comprehension
+        or an additional dimension in ``size`` instead.
     seed :
         An object to be used as seed by the given external library for sampling `expr`.
         Following is the list of possible types of object for the supported libraries,
@@ -1054,62 +1081,82 @@ def sample(expr, condition=None, size=(), library='scipy',
         No modifications to environment's global seed settings
         are done by this argument.
 
+    Returns
+    =======
+
+    sample: float/list/numpy.ndarray
+        one sample or a collection of samples of the random expression.
+
+        - sample(X) returns float/numpy.float64/numpy.int64 object.
+        - sample(X, size=int/tuple) returns numpy.ndarray object.
+
     Examples
     ========
 
     >>> from sympy.stats import Die, sample, Normal, Geometric
     >>> X, Y, Z = Die('X', 6), Die('Y', 6), Die('Z', 6) # Finite Random Variable
-
-    >>> die_roll = sample(X + Y + Z) # doctest: +SKIP
-    >>> next(die_roll) # doctest: +SKIP
-    6
+    >>> die_roll = sample(X + Y + Z)
+    >>> die_roll # doctest: +SKIP
+    3
     >>> N = Normal('N', 3, 4) # Continuous Random Variable
-    >>> samp = next(sample(N)) # doctest: +SKIP
-    >>> samp in N.pspace.domain.set # doctest: +SKIP
+    >>> samp = sample(N)
+    >>> samp in N.pspace.domain.set
     True
-    >>> samp = next(sample(N, N>0)) # doctest: +SKIP
-    >>> samp > 0 # doctest: +SKIP
+    >>> samp = sample(N, N>0)
+    >>> samp > 0
     True
-    >>> samp_list = next(sample(N, size=4)) # doctest: +SKIP
-    >>> [sam in N.pspace.domain.set for sam in samp_list] # doctest: +SKIP
+    >>> samp_list = sample(N, size=4)
+    >>> [sam in N.pspace.domain.set for sam in samp_list]
     [True, True, True, True]
+    >>> sample(N, size = (2,3)) # doctest: +SKIP
+    array([[5.42519758, 6.40207856, 4.94991743],
+       [1.85819627, 6.83403519, 1.9412172 ]])
     >>> G = Geometric('G', 0.5) # Discrete Random Variable
-    >>> samp_list = next(sample(G, size=3)) # doctest: +SKIP
+    >>> samp_list = sample(G, size=3)
     >>> samp_list # doctest: +SKIP
-    array([10,  4,  1])
-    >>> [sam in G.pspace.domain.set for sam in samp_list] # doctest: +SKIP
+    [1, 3, 2]
+    >>> [sam in G.pspace.domain.set for sam in samp_list]
     [True, True, True]
     >>> MN = Normal("MN", [3, 4], [[2, 1], [1, 2]]) # Joint Random Variable
-    >>> samp_list = next(sample(MN, size=4)) # doctest: +SKIP
+    >>> samp_list = sample(MN, size=4)
     >>> samp_list # doctest: +SKIP
-    array([[4.22564264, 3.23364418],
-           [3.41002011, 4.60090908],
-           [3.76151866, 4.77617143],
-           [4.71440865, 2.65714157]])
-    >>> [tuple(sam) in MN.pspace.domain.set for sam in samp_list] # doctest: +SKIP
+    [array([2.85768055, 3.38954165]),
+     array([4.11163337, 4.3176591 ]),
+     array([0.79115232, 1.63232916]),
+     array([4.01747268, 3.96716083])]
+    >>> [tuple(sam) in MN.pspace.domain.set for sam in samp_list]
     [True, True, True, True]
 
+    .. versionchanged:: 1.7.0
+        sample used to return an iterator containing the samples instead of value.
 
-    Returns
-    =======
-
-    sample: iterator object
-        iterator object containing the sample/samples of given expr
+    .. versionchanged:: 1.9.0
+        sample returns values or array of values instead of an iterator and numsamples is deprecated.
 
     """
-    ### TODO: Remove the user warnings in the future releases
-    message = ("The return type of sample has been changed to return an "
-                  "iterator object since version 1.7. For more information see "
-                  "https://github.com/sympy/sympy/issues/19061")
-    warnings.warn(filldedent(message))
-    return sample_iter(expr, condition, size=size, library=library,
+
+    iterator = sample_iter(expr, condition, size=size, library=library,
                                                         numsamples=numsamples, seed=seed)
 
+    if numsamples != 1:
+        SymPyDeprecationWarning(
+                 feature="numsamples parameter",
+                 issue=21723,
+                 deprecated_since_version="1.9",
+                 useinstead="a list comprehension or an additional dimension in ``size``").warn()
+
+        return [next(iterator) for i in range(numsamples)]
+
+    return next(iterator)
+
 
 def quantile(expr, evaluate=True, **kwargs):
     r"""
     Return the :math:`p^{th}` order quantile of a probability distribution.
 
+    Explanation
+    ===========
+
     Quantile is defined as the value at which the probability of the random
     variable is less than or equal to the given probability.
 
@@ -1156,7 +1203,7 @@ def sample_iter(expr, condition=None, size=(), library='scipy',
                     numsamples=S.Infinity, seed=None, **kwargs):
 
     """
-    Returns an iterator of realizations from the expression given a condition
+    Returns an iterator of realizations from the expression given a condition.
 
     Parameters
     ==========
@@ -1225,28 +1272,39 @@ def sample_iter(expr, condition=None, size=(), library='scipy',
                 sub[arg] = RandomSymbol(arg.symbol, arg.pspace)
         expr = expr.subs(sub)
 
+    def fn_subs(*args):
+        return expr.subs({rv: arg for rv, arg in zip(rvs, args)})
+
+    def given_fn_subs(*args):
+        if condition is not None:
+            return condition.subs({rv: arg for rv, arg in zip(rvs, args)})
+        return False
+
     if library == 'pymc3':
         # Currently unable to lambdify in pymc3
         # TODO : Remove 'pymc3' when lambdify accepts 'pymc3' as module
         fn = lambdify(rvs, expr, **kwargs)
     else:
         fn = lambdify(rvs, expr, modules=library, **kwargs)
+
+
     if condition is not None:
         given_fn = lambdify(rvs, condition, **kwargs)
 
     def return_generator_infinite():
         count = 0
-        np = import_module('numpy')
-        if np:
-            rand_state = np.random.default_rng(seed=seed)
-        else:
-            rand_state = None
+        _size = (1,)+((size,) if isinstance(size, int) else size)
         while count < numsamples:
-            d = ps.sample(size=size, library=library, seed=rand_state)  # a dictionary that maps RVs to values
-            args = [d[rv] for rv in rvs]
+            d = ps.sample(size=_size, library=library, seed=seed)  # a dictionary that maps RVs to values
+            args = [d[rv][0] for rv in rvs]
 
             if condition is not None:  # Check that these values satisfy the condition
-                gd = given_fn(*args)
+                # TODO: Replace the try-except block with only given_fn(*args)
+                # once lambdify works with unevaluated SymPy objects.
+                try:
+                    gd = given_fn(*args)
+                except (NameError, TypeError):
+                    gd = given_fn_subs(*args)
                 if gd != True and gd != False:
                     raise ValueError(
                         "Conditions must not contain free symbols")
@@ -1261,13 +1319,18 @@ def sample_iter(expr, condition=None, size=(), library='scipy',
         while faulty:
             d = ps.sample(size=(numsamples,) + ((size,) if isinstance(size, int) else size),
                           library=library, seed=seed) # a dictionary that maps RVs to values
+
             faulty = False
             count = 0
             while count < numsamples and not faulty:
                 args = [d[rv][count] for rv in rvs]
-
                 if condition is not None:  # Check that these values satisfy the condition
-                    gd = given_fn(*args)
+                    # TODO: Replace the try-except block with only given_fn(*args)
+                    # once lambdify works with unevaluated SymPy objects.
+                    try:
+                        gd = given_fn(*args)
+                    except (NameError, TypeError):
+                        gd = given_fn_subs(*args)
                     if gd != True and gd != False:
                         raise ValueError(
                             "Conditions must not contain free symbols")
@@ -1279,7 +1342,12 @@ def sample_iter(expr, condition=None, size=(), library='scipy',
         count = 0
         while count < numsamples:
             args = [d[rv][count] for rv in rvs]
-            yield fn(*args)
+            # TODO: Replace the try-except block with only fn(*args)
+            # once lambdify works with unevaluated SymPy objects.
+            try:
+                yield fn(*args)
+            except (NameError, TypeError):
+                yield fn_subs(*args)
             count += 1
 
     if numsamples is S.Infinity:
@@ -1303,7 +1371,7 @@ def sample_iter_subs(expr, condition=None, size=(),
 def sampling_P(condition, given_condition=None, library='scipy', numsamples=1,
                evalf=True, seed=None, **kwargs):
     """
-    Sampling version of P
+    Sampling version of P.
 
     See Also
     ========
@@ -1335,7 +1403,7 @@ def sampling_P(condition, given_condition=None, library='scipy', numsamples=1,
 def sampling_E(expr, given_condition=None, library='scipy', numsamples=1,
                evalf=True, seed=None, **kwargs):
     """
-    Sampling version of E
+    Sampling version of E.
 
     See Also
     ========
@@ -1356,7 +1424,7 @@ def sampling_E(expr, given_condition=None, library='scipy', numsamples=1,
 def sampling_density(expr, given_condition=None, library='scipy',
                     numsamples=1, seed=None, **kwargs):
     """
-    Sampling version of density
+    Sampling version of density.
 
     See Also
     ========
@@ -1375,7 +1443,7 @@ def sampling_density(expr, given_condition=None, library='scipy',
 
 def dependent(a, b):
     """
-    Dependence of two random expressions
+    Dependence of two random expressions.
 
     Two expressions are independent if knowledge of one does not change
     computations on the other.
@@ -1412,7 +1480,7 @@ def dependent(a, b):
 
 def independent(a, b):
     """
-    Independence of two random expressions
+    Independence of two random expressions.
 
     Two expressions are independent if knowledge of one does not change
     computations on the other.
@@ -1476,6 +1544,7 @@ def rv_subs(expr, symbols=None):
     swapdict = {rv: rv.symbol for rv in symbols}
     return expr.subs(swapdict)
 
+
 class NamedArgsMixin:
     _argnames = ()  # type: tTuple[str, ...]
 
@@ -1486,6 +1555,66 @@ class NamedArgsMixin:
             raise AttributeError("'%s' object has no attribute '%s'" % (
                 type(self).__name__, attr))
 
+
+class Distribution(Basic):
+
+    def sample(self, size=(), library='scipy', seed=None):
+        """ A random realization from the distribution """
+
+        module = import_module(library)
+        if library in {'scipy', 'numpy', 'pymc3'} and module is None:
+            raise ValueError("Failed to import %s" % library)
+
+        if library == 'scipy':
+            # scipy does not require map as it can handle using custom distributions.
+            # However, we will still use a map where we can.
+
+            # TODO: do this for drv.py and frv.py if necessary.
+            # TODO: add more distributions here if there are more
+            # See links below referring to sections beginning with "A common parametrization..."
+            # I will remove all these comments if everything is ok.
+
+            from sympy.stats.sampling.sample_scipy import do_sample_scipy
+            import numpy
+            if seed is None or isinstance(seed, int):
+                rand_state = numpy.random.default_rng(seed=seed)
+            else:
+                rand_state = seed
+            samps = do_sample_scipy(self, size, rand_state)
+
+        elif library == 'numpy':
+            from sympy.stats.sampling.sample_numpy import do_sample_numpy
+            import numpy
+            if seed is None or isinstance(seed, int):
+                rand_state = numpy.random.default_rng(seed=seed)
+            else:
+                rand_state = seed
+            _size = None if size == () else size
+            samps = do_sample_numpy(self, _size, rand_state)
+        elif library == 'pymc3':
+            from sympy.stats.sampling.sample_pymc3 import do_sample_pymc3
+            import logging
+            logging.getLogger("pymc3").setLevel(logging.ERROR)
+            import pymc3
+            with pymc3.Model():
+                if do_sample_pymc3(self):
+                    samps = pymc3.sample(draws=prod(size), chains=1, compute_convergence_checks=False,
+                            progressbar=False, random_seed=seed, return_inferencedata=False)[:]['X']
+                    samps = samps.reshape(size)
+                else:
+                    samps = None
+
+        else:
+            raise NotImplementedError("Sampling from %s is not supported yet."
+                                      % str(library))
+
+        if samps is not None:
+            return samps
+        raise NotImplementedError(
+            "Sampling for %s is not currently implemented from %s"
+            % (self, library))
+
+
 def _value_check(condition, message):
     """
     Raise a ValueError with message if condition is False, else
@@ -1586,7 +1715,7 @@ def _symbol_converter(sym):
     True
     """
     if isinstance(sym, str):
-            sym = Symbol(sym)
+        sym = Symbol(sym)
     if not isinstance(sym, Symbol):
         raise TypeError("%s is neither a Symbol nor a string"%(sym))
     return sym
