@@ -1,17 +1,79 @@
-from __future__ import print_function, division
-
-from typing import Any
-
 from sympy import Basic
 from sympy import S
 from sympy.core.expr import Expr
 from sympy.core.numbers import Integer
 from sympy.core.sympify import sympify
-from sympy.core.compatibility import SYMPY_INTS, Iterable
+from sympy.core.kind import Kind, NumberKind, UndefinedKind
+from sympy.core.compatibility import SYMPY_INTS
+from sympy.printing.defaults import Printable
 
 import itertools
+from collections.abc import Iterable
 
-class NDimArray(object):
+
+class ArrayKind(Kind):
+    """
+    Kind for N-dimensional array in SymPy.
+
+    This kind represents the multidimensional array that algebraic
+    operations are defined. Basic class for this kind is ``NDimArray``,
+    but any expression representing the array can have this.
+
+    Parameters
+    ==========
+
+    element_kind : Kind
+        Kind of the element. Default is :obj:NumberKind `<sympy.core.kind.NumberKind>`,
+        which means that the array contains only numbers.
+
+    Examples
+    ========
+
+    Any instance of array class has ``ArrayKind``.
+
+    >>> from sympy import NDimArray
+    >>> NDimArray([1,2,3]).kind
+    ArrayKind(NumberKind)
+
+    Although expressions representing an array may be not instance of
+    array class, it will have ``ArrayKind`` as well.
+
+    >>> from sympy import Integral
+    >>> from sympy.tensor.array import NDimArray
+    >>> from sympy.abc import x
+    >>> intA = Integral(NDimArray([1,2,3]), x)
+    >>> isinstance(intA, NDimArray)
+    False
+    >>> intA.kind
+    ArrayKind(NumberKind)
+
+    Use ``isinstance()`` to check for ``ArrayKind` without specifying
+    the element kind. Use ``is`` with specifying the element kind.
+
+    >>> from sympy.tensor.array import ArrayKind
+    >>> from sympy.core.kind import NumberKind
+    >>> boolA = NDimArray([True, False])
+    >>> isinstance(boolA.kind, ArrayKind)
+    True
+    >>> boolA.kind is ArrayKind(NumberKind)
+    False
+
+    See Also
+    ========
+
+    shape : Function to return the shape of objects with ``MatrixKind``.
+
+    """
+    def __new__(cls, element_kind=NumberKind):
+        obj = super().__new__(cls, element_kind)
+        obj.element_kind = element_kind
+        return obj
+
+    def __repr__(self):
+        return "ArrayKind(%s)" % self.element_kind
+
+
+class NDimArray(Printable):
     """
 
     Examples
@@ -65,11 +127,21 @@ class NDimArray(object):
     """
 
     _diff_wrt = True
+    is_scalar = False
 
     def __new__(cls, iterable, shape=None, **kwargs):
         from sympy.tensor.array import ImmutableDenseNDimArray
         return ImmutableDenseNDimArray(iterable, shape, **kwargs)
 
+    @property
+    def kind(self):
+        elem_kinds = set(e.kind for e in self._array)
+        if len(elem_kinds) == 1:
+            elemkind, = elem_kinds
+        else:
+            elemkind = UndefinedKind
+        return ArrayKind(elemkind)
+
     def _parse_index(self, index):
         if isinstance(index, (SYMPY_INTS, Integer)):
             raise ValueError("Only a tuple index is accepted")
@@ -145,6 +217,10 @@ class NDimArray(object):
             elif isinstance(iterable, SparseNDimArray):
                 return iterable._shape, iterable._sparse_array
 
+            # Construct N-dim array from another N-dim array:
+            elif isinstance(iterable, NDimArray):
+                shape = iterable.shape
+
             # Construct N-dim array from an iterable (numpy arrays included):
             elif isinstance(iterable, Iterable):
                 iterable, shape = cls._scan_iterable_shape(iterable)
@@ -153,10 +229,6 @@ class NDimArray(object):
             elif isinstance(iterable, MatrixBase):
                 shape = iterable.shape
 
-            # Construct N-dim array from another N-dim array:
-            elif isinstance(iterable, NDimArray):
-                shape = iterable.shape
-
             else:
                 shape = ()
                 iterable = (iterable,)
@@ -240,38 +312,17 @@ class NDimArray(object):
         [[1, 0], [0, y]]
 
         """
-        from sympy import Derivative
+        from sympy.tensor.array.array_derivatives import ArrayDerivative
         kwargs.setdefault('evaluate', True)
-        return Derivative(self.as_immutable(), *args, **kwargs)
+        return ArrayDerivative(self.as_immutable(), *args, **kwargs)
 
-    def _accept_eval_derivative(self, s):
-        return s._visit_eval_derivative_array(self)
-
-    def _visit_eval_derivative_scalar(self, base):
+    def _eval_derivative(self, base):
         # Types are (base: scalar, self: array)
         return self.applyfunc(lambda x: base.diff(x))
 
-    def _visit_eval_derivative_array(self, base):
-        # Types are (base: array/matrix, self: array)
-        from sympy import derive_by_array
-        return derive_by_array(base, self)
-
     def _eval_derivative_n_times(self, s, n):
         return Basic._eval_derivative_n_times(self, s, n)
 
-    def _eval_derivative(self, arg):
-        return self.applyfunc(lambda x: x.diff(arg))
-
-    def _eval_derivative_array(self, arg):
-        from sympy import derive_by_array
-        from sympy import Tuple
-        from sympy.matrices.common import MatrixCommon
-
-        if isinstance(arg, (Iterable, Tuple, MatrixCommon, NDimArray)):
-            return derive_by_array(self, arg)
-        else:
-            return self.applyfunc(lambda x: x.diff(arg))
-
     def applyfunc(self, f):
         """Apply a function to each element of the N-dim array.
 
@@ -293,52 +344,19 @@ class NDimArray(object):
 
         return type(self)(map(f, Flatten(self)), self.shape)
 
-    def __str__(self):
-        """Returns string, allows to use standard functions print() and str().
-
-        Examples
-        ========
-
-        >>> from sympy import MutableDenseNDimArray
-        >>> a = MutableDenseNDimArray.zeros(2, 2)
-        >>> a
-        [[0, 0], [0, 0]]
-
-        """
+    def _sympystr(self, printer):
         def f(sh, shape_left, i, j):
             if len(shape_left) == 1:
-                return "["+", ".join([str(self[self._get_tuple_index(e)]) for e in range(i, j)])+"]"
+                return "["+", ".join([printer._print(self[self._get_tuple_index(e)]) for e in range(i, j)])+"]"
 
             sh //= shape_left[0]
             return "[" + ", ".join([f(sh, shape_left[1:], i+e*sh, i+(e+1)*sh) for e in range(shape_left[0])]) + "]" # + "\n"*len(shape_left)
 
         if self.rank() == 0:
-            return self[()].__str__()
+            return printer._print(self[()])
 
         return f(self._loop_size, self.shape, 0, self._loop_size)
 
-    def __repr__(self):
-        return self.__str__()
-
-    # We don't define _repr_png_ here because it would add a large amount of
-    # data to any notebook containing SymPy expressions, without adding
-    # anything useful to the notebook. It can still enabled manually, e.g.,
-    # for the qtconsole, with init_printing().
-    def _repr_latex_(self):
-        """
-        IPython/Jupyter LaTeX printing
-
-        To change the behavior of this (e.g., pass in some settings to LaTeX),
-        use init_printing(). init_printing() will also enable LaTeX printing
-        for built in numeric types like ints and container types that contain
-        SymPy objects, like lists and dictionaries of expressions.
-        """
-        from sympy.printing.latex import latex
-        s = latex(self, mode='plain')
-        return "$\\displaystyle %s$" % s
-
-    _repr_latex_orig = _repr_latex_  # type: Any
-
     def tolist(self):
         """
         Converting MutableDenseNDimArray to one-dim list
@@ -370,7 +388,7 @@ class NDimArray(object):
         from sympy.tensor.array.arrayop import Flatten
 
         if not isinstance(other, NDimArray):
-            raise TypeError(str(other))
+            return NotImplemented
 
         if self.shape != other.shape:
             raise ValueError("array shape mismatch")
@@ -382,7 +400,7 @@ class NDimArray(object):
         from sympy.tensor.array.arrayop import Flatten
 
         if not isinstance(other, NDimArray):
-            raise TypeError(str(other))
+            return NotImplemented
 
         if self.shape != other.shape:
             raise ValueError("array shape mismatch")
@@ -424,7 +442,7 @@ class NDimArray(object):
         result_list = [other*i for i in Flatten(self)]
         return type(self)(result_list, self.shape)
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         from sympy.matrices.matrices import MatrixBase
         from sympy.tensor.array import SparseNDimArray
         from sympy.tensor.array.arrayop import Flatten
@@ -439,7 +457,7 @@ class NDimArray(object):
         result_list = [i/other for i in Flatten(self)]
         return type(self)(result_list, self.shape)
 
-    def __rdiv__(self, other):
+    def __rtruediv__(self, other):
         raise NotImplementedError('unsupported operation on NDimArray')
 
     def __neg__(self):
@@ -498,9 +516,6 @@ class NDimArray(object):
     def __ne__(self, other):
         return not self == other
 
-    __truediv__ = __div__
-    __rtruediv__ = __rdiv__
-
     def _eval_transpose(self):
         if self.rank() != 2:
             raise ValueError("array rank not 2")
