@@ -3,8 +3,6 @@
 See sympy.unify for module level docstring
 See sympy.unify.core for algorithmic docstring """
 
-from __future__ import print_function, division
-
 from sympy.core import Basic, Add, Mul, Pow
 from sympy.core.operations import AssocOp, LatticeOp
 from sympy.matrices import MatAdd, MatMul, MatrixExpr
@@ -66,14 +64,14 @@ def construct(t):
         return t.op(*map(construct, t.args))
 
 def rebuild(s):
-    """ Rebuild a SymPy expression
+    """ Rebuild a SymPy expression.
 
-    This removes harm caused by Expr-Rules interactions
+    This removes harm caused by Expr-Rules interactions.
     """
     return construct(deconstruct(s))
 
 def unify(x, y, s=None, variables=(), **kwargs):
-    """ Structural unification of two expressions/patterns
+    """ Structural unification of two expressions/patterns.
 
     Examples
     ========
@@ -116,11 +114,11 @@ def unify(x, y, s=None, variables=(), **kwargs):
     """
     decons = lambda x: deconstruct(x, variables)
     s = s or {}
-    s = dict((decons(k), decons(v)) for k, v in s.items())
+    s = {decons(k): decons(v) for k, v in s.items()}
 
     ds = core.unify(decons(x), decons(y), s,
                                      is_associative=is_associative,
                                      is_commutative=is_commutative,
                                      **kwargs)
     for d in ds:
-        yield dict((construct(k), construct(v)) for k, v in d.items())
+        yield {construct(k): construct(v) for k, v in d.items()}
