@@ -8,14 +8,34 @@ python.
 This module provides a common interface for different external backends, such
 as f2py, fwrap, Cython, SWIG(?) etc. (Currently only f2py and Cython are
 implemented) The goal is to provide access to compiled binaries of acceptable
-performance with a one-button user interface, i.e.
+performance with a one-button user interface, e.g.,
 
     >>> from sympy.abc import x,y
-    >>> expr = ((x - y)**(25)).expand()
-    >>> binary_callable = autowrap(expr)
-    >>> binary_callable(1, 2)
+    >>> expr = (x - y)**25
+    >>> flat = expr.expand()
+    >>> binary_callable = autowrap(flat)
+    >>> binary_callable(2, 3)
     -1.0
 
+Although a SymPy user might primarily be interested in working with
+mathematical expressions and not in the details of wrapping tools
+needed to evaluate such expressions efficiently in numerical form,
+the user cannot do so without some understanding of the
+limits in the target language. For example, the expanded expression
+contains large coefficients which result in loss of precision when
+computing the expression:
+
+    >>> binary_callable(3, 2)
+    0.0
+    >>> binary_callable(4, 5), binary_callable(5, 4)
+    (-22925376.0, 25165824.0)
+
+Wrapping the unexpanded expression gives the expected behavior:
+
+    >>> e = autowrap(expr)
+    >>> e(4, 5), e(5, 4)
+    (-1.0, 1.0)
+
 The callable returned from autowrap() is a binary python function, not a
 SymPy object.  If it is desired to use the compiled function in symbolic
 expressions, it is better to use binary_function() which returns a SymPy
@@ -30,11 +50,6 @@ lambdify().
     >>> (2*f(x, y) + y).evalf(2, subs={x: 1, y:2})
     0.e-110
 
-The idea is that a SymPy user will primarily be interested in working with
-mathematical expressions, and should not have to learn details about wrapping
-tools in order to evaluate expressions numerically, even if they are
-computationally expensive.
-
 When is this useful?
 
     1) For computations on large arrays, Python iterations may be too slow,
@@ -65,8 +80,6 @@ When is this module NOT the best approach?
 
 """
 
-from __future__ import print_function, division
-
 import sys
 import os
 import shutil
@@ -76,7 +89,7 @@ from string import Template
 from warnings import warn
 
 from sympy.core.cache import cacheit
-from sympy.core.compatibility import range, iterable
+from sympy.core.compatibility import iterable
 from sympy.core.function import Lambda
 from sympy.core.relational import Eq
 from sympy.core.symbol import Dummy, Symbol
@@ -96,7 +109,7 @@ class CodeWrapError(Exception):
     pass
 
 
-class CodeWrapper(object):
+class CodeWrapper:
     """Base Class for code wrappers"""
     _filename = "wrapped_code"
     _module_basename = "wrapper_module"
@@ -133,7 +146,8 @@ class CodeWrapper(object):
             routines, self.filename, True, self.include_header,
             self.include_empty)
 
-    def wrap_code(self, routine, helpers=[]):
+    def wrap_code(self, routine, helpers=None):
+        helpers = helpers or []
         if self.filepath:
             workdir = os.path.abspath(self.filepath)
         else:
@@ -299,7 +313,7 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
 
         self._need_numpy = False
 
-        super(CythonCodeWrapper, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @property
     def command(self):
@@ -312,7 +326,7 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
         codefilename = "%s.%s" % (self.filename, self.generator.code_extension)
 
         # pyx
-        with open(pyxfilename, 'w') as f:
+        with open(os.path.join(build_dir, pyxfilename), 'w') as f:
             self.dump_pyx([routine], f, self.filename)
 
         # setup.py
@@ -377,16 +391,16 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
             local_decs = []
             for arg, val in py_inf.items():
                 proto = self._prototype_arg(arg)
-                mat, ind = val
-                local_decs.append("    cdef {0} = {1}.shape[{2}]".format(proto, mat, ind))
-            local_decs.extend(["    cdef {0}".format(self._declare_arg(a)) for a in py_loc])
+                mat, ind = [self._string_var(v) for v in val]
+                local_decs.append("    cdef {} = {}.shape[{}]".format(proto, mat, ind))
+            local_decs.extend(["    cdef {}".format(self._declare_arg(a)) for a in py_loc])
             declarations = "\n".join(local_decs)
             if declarations:
                 declarations = declarations + "\n"
 
             # Function Body
             args_c = ", ".join([self._call_arg(a) for a in routine.arguments])
-            rets = ", ".join([str(r.name) for r in py_rets])
+            rets = ", ".join([self._string_var(r.name) for r in py_rets])
             if routine.results:
                 body = '    return %s(%s)' % (routine.name, args_c)
                 if rets:
@@ -444,14 +458,14 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
             self._need_numpy = True
             ndim = len(arg.dimensions)
             mtype = np_types[t]
-            return mat_dec.format(mtype=mtype, ndim=ndim, name=arg.name)
+            return mat_dec.format(mtype=mtype, ndim=ndim, name=self._string_var(arg.name))
         else:
-            return "%s %s" % (t, str(arg.name))
+            return "%s %s" % (t, self._string_var(arg.name))
 
     def _declare_arg(self, arg):
         proto = self._prototype_arg(arg)
         if arg.dimensions:
-            shape = '(' + ','.join(str(i[1] + 1) for i in arg.dimensions) + ')'
+            shape = '(' + ','.join(self._string_var(i[1] + 1) for i in arg.dimensions) + ')'
             return proto + " = np.empty({shape})".format(shape=shape)
         else:
             return proto + " = 0"
@@ -459,11 +473,15 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
     def _call_arg(self, arg):
         if arg.dimensions:
             t = arg.get_datatype('c')
-            return "<{0}*> {1}.data".format(t, arg.name)
+            return "<{}*> {}.data".format(t, self._string_var(arg.name))
         elif isinstance(arg, ResultBase):
-            return "&{0}".format(arg.name)
+            return "&{}".format(self._string_var(arg.name))
         else:
-            return str(arg.name)
+            return self._string_var(arg.name)
+
+    def _string_var(self, var):
+        printer = self.generator.printer.doprint
+        return printer(var)
 
 
 class F2PyCodeWrapper(CodeWrapper):
@@ -481,7 +499,7 @@ class F2PyCodeWrapper(CodeWrapper):
                 warn(msg.format(k))
             kwargs.pop(k, None)
 
-        super(F2PyCodeWrapper, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @property
     def command(self):
@@ -521,7 +539,7 @@ def _validate_backend_language(backend, language):
     if not langs:
         raise ValueError("Unrecognized backend: " + backend)
     if language.upper() not in langs:
-        raise ValueError(("Backend {0} and language {1} are "
+        raise ValueError(("Backend {} and language {} are "
                           "incompatible").format(backend, language))
 
 
@@ -532,7 +550,8 @@ def autowrap(expr, language=None, backend='f2py', tempdir=None, args=None,
     """Generates python callable binaries based on the math expression.
 
     Parameters
-    ----------
+    ==========
+
     expr
         The SymPy expression that should be wrapped as a binary routine.
     language : string, optional
@@ -554,13 +573,14 @@ def autowrap(expr, language=None, backend='f2py', tempdir=None, args=None,
     verbose : bool, optional
         If True, autowrap will not mute the command line backends. This can be
         helpful for debugging.
-    helpers : iterable, optional
+    helpers : 3-tuple or iterable of 3-tuples, optional
         Used to define auxiliary expressions needed for the main expr. If the
-        main expression needs to call a specialized function it should be put
-        in the ``helpers`` iterable. Autowrap will then make sure that the
+        main expression needs to call a specialized function it should be
+        passed in via ``helpers``. Autowrap will then make sure that the
         compiled main expression can link to the helper routine. Items should
-        be tuples with (<funtion_name>, <sympy_expression>, <arguments>). It
-        is mandatory to supply an argument sequence to helper routines.
+        be 3-tuples with (<function_name>, <sympy_expression>,
+        <argument_tuple>). It is mandatory to supply an argument sequence to
+        helper routines.
     code_gen : CodeGen instance
         An instance of a CodeGen subclass. Overrides ``language``.
     include_dirs : [string]
@@ -582,13 +602,15 @@ def autowrap(expr, language=None, backend='f2py', tempdir=None, args=None,
         'extra_compile_args'.
 
     Examples
-    --------
+    ========
+
     >>> from sympy.abc import x, y, z
     >>> from sympy.utilities.autowrap import autowrap
     >>> expr = ((x - y + z)**(13)).expand()
     >>> binary_func = autowrap(expr)
     >>> binary_func(1, 4, 2)
     -1.0
+
     """
     if language:
         if not isinstance(language, type):
@@ -596,7 +618,12 @@ def autowrap(expr, language=None, backend='f2py', tempdir=None, args=None,
     else:
         language = _infer_language(backend)
 
-    helpers = [helpers] if helpers else ()
+    # two cases 1) helpers is an iterable of 3-tuples and 2) helpers is a
+    # 3-tuple
+    if iterable(helpers) and len(helpers) != 0 and iterable(helpers[0]):
+        helpers = helpers if helpers else ()
+    else:
+        helpers = [helpers] if helpers else ()
     args = list(args) if iterable(args, exclude=set) else args
 
     if code_gen is None:
@@ -643,7 +670,8 @@ def binary_function(symfunc, expr, **kwargs):
     with implemented_function().
 
     Parameters
-    ----------
+    ==========
+
     symfunc : sympy Function
         The function to bind the callable to.
     expr : sympy Expression
@@ -652,7 +680,8 @@ def binary_function(symfunc, expr, **kwargs):
         Any kwargs accepted by autowrap.
 
     Examples
-    --------
+    ========
+
     >>> from sympy.abc import x, y
     >>> from sympy.utilities.autowrap import binary_function
     >>> expr = ((x - y)**(25)).expand()
@@ -663,6 +692,7 @@ def binary_function(symfunc, expr, **kwargs):
     2*f(x, y)
     >>> f(x, y).evalf(2, subs={x: 1, y: 2})
     -1.0
+
     """
     binary = autowrap(expr, **kwargs)
     return implemented_function(symfunc, binary)
@@ -784,7 +814,7 @@ class UfuncifyCodeWrapper(CodeWrapper):
                 warn(msg.format(k))
             kwargs.pop(k, None)
 
-        super(UfuncifyCodeWrapper, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @property
     def command(self):
@@ -866,16 +896,17 @@ class UfuncifyCodeWrapper(CodeWrapper):
         funcname
             Name of the main function to be returned.
         """
-        if (funcname is None) and (len(routines) == 1):
-            funcname = routines[0].name
-        elif funcname is None:
-            msg = 'funcname must be specified for multiple output routines'
-            raise ValueError(msg)
+        if funcname is None:
+            if len(routines) == 1:
+                funcname = routines[0].name
+            else:
+                msg = 'funcname must be specified for multiple output routines'
+                raise ValueError(msg)
         functions = []
         function_creation = []
         ufunc_init = []
         module = self.module_name
-        include_file = "\"{0}.h\"".format(prefix)
+        include_file = "\"{}.h\"".format(prefix)
         top = _ufunc_top.substitute(include_file=include_file, module=module)
 
         name = funcname
@@ -917,7 +948,7 @@ class UfuncifyCodeWrapper(CodeWrapper):
         docstring = '"Created in SymPy with Ufuncify"'
 
         # Function Creation
-        function_creation.append("PyObject *ufunc{0};".format(r_index))
+        function_creation.append("PyObject *ufunc{};".format(r_index))
 
         # Ufunc initialization
         init_form = _ufunc_init_form.substitute(module=module,
@@ -970,7 +1001,8 @@ def ufuncify(args, expr, language=None, backend='numpy', tempdir=None,
     """Generates a binary function that supports broadcasting on numpy arrays.
 
     Parameters
-    ----------
+    ==========
+
     args : iterable
         Either a Symbol or an iterable of symbols. Specifies the argument
         sequence for the function.
@@ -1004,8 +1036,9 @@ def ufuncify(args, expr, language=None, backend='numpy', tempdir=None,
         These kwargs will be passed to autowrap if the `f2py` or `cython`
         backend is used and ignored if the `numpy` backend is used.
 
-    Note
-    ----
+    Notes
+    =====
+
     The default backend ('numpy') will create actual instances of
     ``numpy.ufunc``. These support ndimensional broadcasting, and implicit type
     conversion. Use of the other backends will result in a "ufunc-like"
@@ -1013,8 +1046,9 @@ def ufuncify(args, expr, language=None, backend='numpy', tempdir=None,
     arguments, and will not perform any type conversions.
 
     References
-    ----------
-    [1] http://docs.scipy.org/doc/numpy/reference/ufuncs.html
+    ==========
+
+    .. [1] http://docs.scipy.org/doc/numpy/reference/ufuncs.html
 
     Examples
     ========
@@ -1046,6 +1080,7 @@ def ufuncify(args, expr, language=None, backend='numpy', tempdir=None,
     TypeError: Argument '_x' has incorrect type (expected numpy.ndarray, got int)
     >>> f_cython(np.array([1.0]), np.array([2.0]))
     array([ 3.])
+
     """
 
     if isinstance(args, Symbol):
@@ -1075,7 +1110,7 @@ def ufuncify(args, expr, language=None, backend='numpy', tempdir=None,
             expr = [expr]
         if len(expr) == 0:
             raise ValueError('Expression iterable has zero length')
-        if (len(expr) + len(args)) > maxargs:
+        if len(expr) + len(args) > maxargs:
             msg = ('Cannot create ufunc with more than {0} total arguments: '
                    'got {1} in, {2} out')
             raise ValueError(msg.format(maxargs, len(args), len(expr)))
